/*==========================================================================
  Copyright (c) Fabasoft R&D GmbH, A-4020 Linz, 1995-2015.

  All rights reserved. All hardware and software names used are registered
  trade names and/or registered trademarks of the respective manufacturers.

  The user of this computer program acknowledges that the above copyright
  notice, which constitutes the Universal Copyright Convention, will be
  attached at the position in the function of the computer program which the
  author has deemed to sufficiently express the reservation of copyright.
  It is prohibited for customers, users and/or third parties to remove,
  modify or move this copyright notice.
  ==========================================================================*/

#ifndef _APM_API_H_
#define _APM_API_H_

#if defined(WIN32)
# define APM_WIN
#else
# define APM_UNIX
#endif

#if defined(APM_WEB_API)
# define APM_THREAD_TRANSACTION
# define APM_EXPLICIT_TRANSACTION
#endif

#if !defined(APM_EXPLICIT_TRANSACTION)
# define APM_THREAD_TRANSACTION
#endif

#include <stdio.h>

///
/// Platform independent definition of basic integer data types
///

#if defined(APM_UNIX)
# include <inttypes.h>
#endif

#if defined(APM_WIN)
  typedef unsigned __int8  UInt8;
  typedef unsigned __int16 UInt16;
  typedef unsigned __int32 UInt32;
  typedef unsigned __int64 UInt64;

  typedef __int8  Int8;
  typedef __int16 Int16;
  typedef __int32 Int32;
  typedef __int64 Int64;
#  define _APM_WARN_UNUSED

# if defined(_MSC_VER)
#  define _APM_WARN_DEPRECATED __declspec(deprecated)
# else
#  define _APM_WARN_DEPRECATED
# endif

#elif defined(APM_UNIX)
#ifndef FALSE
# define FALSE  		   0
#endif

#ifndef TRUE
# define TRUE   		   1
#endif
  typedef uint8_t UInt8;
  typedef uint16_t UInt16;
#if defined(MACOSX)
  // already declared in CarbonCore.framework/Headers/MacTypes.h
#else
  typedef uint32_t UInt32;
#endif
  typedef uint64_t UInt64;

  typedef int8_t  Int8;
  typedef int16_t Int16;
  typedef int32_t Int32;
  typedef int64_t Int64;
# if defined(__GNUC__) && !defined(__INTEL_COMPILER)
#  define _APM_WARN_UNUSED  __attribute__ ((warn_unused_result))
#  define _APM_WARN_DEPRECATED __attribute__ ((deprecated))
# else
#  define _APM_WARN_UNUSED
#  define _APM_WARN_DEPRECATED
# endif
#endif

#if defined(APM_WIN)
# if defined(APMEXPORT)
#  define APMDECL_EXTERN __declspec(dllexport)
#  define APMFUNC __cdecl
# elif defined(APMIMPORT)
#  define APMDECL_EXTERN __declspec(dllimport)
#  define APMFUNC __cdecl
# else
#  define APMDECL_EXTERN
#  define APMFUNC __cdecl
# endif
#else
# if defined(APMEXPORT)
#  define APMDECL_EXTERN __attribute__((visibility("default")))
#  define APMFUNC
# else
#  define APMDECL_EXTERN
#  define APMFUNC
# endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

  ///
  /// Type definition of data-types used in Software-Telemetry(r) API
  ///
  /// \brief APMModuleHandle is used to associate events to their originating module
  ///
  /// A valid APMModuleHandle is returned calling APMRegisterModule with a
  /// APMRegisterModuleDescription containing a utf8modulename.
  /// Use this handle in subsequent calls to APMModuleRegisterEvent or any of the
  /// APMEvent* functions.
  /// Different modules will be displayed in different bars during the analysis.
  ///
  typedef UInt32 APMModuleHandle;

  ///
  /// \brief APMApplicationHandle is used to identify applications
  ///
  /// A valid APMApplicationHandle is returned calling APMRegisterApplication
  /// Use this handle in subsequent calls.
  ///
  typedef UInt32 APMApplicationHandle;

  ///
  /// \brief APMTransactionHandle provides identity for a APMDetachThread/APMAttachThread pair
  ///
  /// A valid APMTransactionHandle is returned calling APMDetachThread
  /// Use this handle in one subsequent call to APMAttachThread to continue recording of a request.
  ///
  typedef UInt64 APMTransactionHandle;

  ///
  /// \brief APMEventId is used to store application specific event id's
  ///
  /// An event-id value is used to associate an event with the corresponding text
  /// registered by the module with APMModuleRegisterEvent
  ///
  typedef UInt64 APMEventId;

  ///
  /// \brief APMEventLevel is used to filter events by importance
  ///
  /// Events are logged only, if the value is less ore equal to the one used
  /// when starting the Software-Telemetry session.
  ///
  /// The following predefined values are valid when calling any APMEvent* function:
  /// APM_EVENT_LEVEL_LOG    (0): The event is always logged even if no
  ///   Software-Telemetry session has been started. Use this event level
  ///   for permanent logging of a small number of events which you want to
  ///   process statistically
  ///
  /// APM_EVENT_LEVEL_IPC     (10):
  /// APM_EVENT_LEVEL_NORMAL  (50):
  /// APM_EVENT_LEVEL_DETAIL  (60):
  /// APM_EVENT_LEVEL_DEBUG   (70):
  ///
  typedef UInt8  APMEventLevel;

#define APM_EVENT_LEVEL_LOG       00
#define APM_EVENT_LEVEL_IPC       10
#define APM_EVENT_LEVEL_NORMAL    50
#define APM_EVENT_LEVEL_DETAIL    60
#define APM_EVENT_LEVEL_DEBUG     70

  ///
  /// APM_FLAG_NONE           (0x00):
  /// APM_FLAG_ENTER            0x01
  /// APM_FLAG_LEAVE            0x02
  /// APM_FLAG_WAIT             0x04
  /// APM_FLAG_WARNING          0x08
  /// APM_FLAG_ERROR            0x10
  /// APM_FLAG_PROCESS_INFO     0x20
  ///
  typedef UInt8  APMEventFlags;

#define APM_FLAG_NONE             0x00
#define APM_FLAG_ENTER            0x01
#define APM_FLAG_LEAVE            0x02
#define APM_FLAG_WAIT             0x04
#define APM_FLAG_WARNING          0x08
#define APM_FLAG_ERROR            0x10
#define APM_FLAG_PROCESS_INFO     0x20


  /// APM_PARAMETERTYPE_INT32   1
  /// APM_PARAMETERTYPE_INT64   2
  /// APM_PARAMETERTYPE_STRING  3
  typedef UInt8  APMParameterType;

#define APM_PARAMETERTYPE_INT32   1
#define APM_PARAMETERTYPE_INT64   2
#define APM_PARAMETERTYPE_STRING  3

  ///
  /// predefined modules
  /// applications may use predefined modules and events
  /// to use a predefined module, call APMRegisterModule with
  /// enumeventdescriptionscallback set to NULL.
  ///

#define APM_MODULE_NAME_OS                 "Operating System"
#define APM_EVENT_OS_CREATE                0x0000
#define APM_EVENT_OS_OPEN                  0x0001
#define APM_EVENT_OS_DELETE                0x0002
#define APM_EVENT_OS_READ                  0x0003
#define APM_EVENT_OS_WRITE                 0x0004
#define APM_EVENT_OS_CLOSE                 0x0005
#define APM_EVENT_OS_FLUSH                 0x0006
#define APM_EVENT_OS_RENAME                0x0007
#define APM_EVENT_OS_COPY                  0x0008
#define APM_EVENT_OS_MOVE                  0x0009
#define APM_EVENT_OS_LOCK                  0x000A
#define APM_EVENT_OS_UNLOCK                0x000B
#define APM_EVENT_OS_EXECUTE               0x0100
#define APM_EVENT_OS_WAIT                  0x0200
#define APM_EVENT_OS_SLEEP                 0x0201

#define APM_EVENT_OS_CRITICALSECTION_WAIT  0x0300
#define APM_EVENT_OS_CRITICALSECTION_ENTER 0x0301
#define APM_EVENT_OS_CRITICALSECTION_LEAVE 0x0302

#define APM_MODULE_NAME_XMLHTTP            "XMLHttpRequest"
#define APM_EVENT_XMLHTTP_SEND             0
#define APM_EVENT_XMLHTTP_LOADING          1
#define APM_EVENT_XMLHTTP_LOADED           2
#define APM_EVENT_XMLHTTP_INTERACTIVE      3
#define APM_EVENT_XMLHTTP_COMPLETE         4
#define APM_EVENT_XMLHTTP_PROCESSING       10
#define APM_EVENT_XMLHTTP_PROCESSED        11

  ///
  /// predefined events
  /// the following events are automatically available under
  /// all modules
  ///

#define APM_EVENT_ERROR                0x10001000
#define APM_EVENT_WARNING              0x10001001
#define APM_EVENT_INFO                 0x10001002
#define APM_EVENT_TRACE                0x10001003

  ///
  /// \brief APMStringLength is used to store the length of string parameters
  ///
  /// A string parameter is internally represented by the length (APMStringLength)
  /// and APMStringLength characters.
  ///
  typedef UInt16 APMStringLength;

  ///
  /// \brief APMSessionId is used to store session id's
  ///
  /// A telemetry session is internally identified by a APMSessionId
  ///
  typedef UInt32 APMSessionId;

  ///
  /// \brief APMLogicalClock is used to store a logical clock
  ///
  /// The APMLogicalClock is a process local increasing value.
  ///
  typedef UInt64 APMLogicalClock;

  ///
  /// applications may register a callback when calling APMRegisterApplication.
  /// the application should react to the callback by repeatedly calling
  ///   APMRegisterFilterValue for to register all valid values,
  ///   that may be specified when starting a Software-Telemetry session.
  /// whenever the client needs to display a selection of filter values,
  ///   this callback will be called on any application service providing
  ///   this information.
  ///
  typedef void (APMFUNC *EnumAppFilterValuesCallback)(
    APMApplicationHandle app);

  ///
  /// values of type APMContext will be passed during interprocess-communication
  ///   to provide the caller context to the invoked program.
  ///
  typedef unsigned char APMContext[16];

  ///
  /// values of type APMSyncMark will be passed during interprocess-communication
  ///   to provide time sequence information.
  ///
  typedef unsigned char APMSyncMark[16];

#if !defined(APM_WEB_API)

  ///
  /// Call APMConnect to establish a connection to the app.telemetry Agent
  ///
  APMDECL_EXTERN void APMFUNC APMConnect();

  ///
  /// Call APMDisconnect to disconnect from the app.telemetry Agent, this will also cleanup the app.telemetry communication thread and will disable automatic reconnection
  /// to reestablish the connection to the app.telemetry Agent call APMConnect
  ///
  APMDECL_EXTERN void APMFUNC APMDisconnect();

#endif

  ///
  /// Call APMRegisterApplication once to provide information about the context of
  ///   the application
  ///
  APMDECL_EXTERN APMApplicationHandle APMFUNC APMRegisterApplication(
    const char *utf8applname,         // Fabasoft Folio 2007
    const char *utf8applid,           // 1.1007
    const char *utf8appltier,         // Fabasoft Components COO-Service
    const char *utf8appltierid,       // 1 (unique per applid)
    EnumAppFilterValuesCallback filterCallback) _APM_WARN_UNUSED;

  ///
  /// Call APMUnregisterApplication once to cleanup data structures
  ///
  APMDECL_EXTERN void APMFUNC APMUnregisterApplication(
    APMApplicationHandle app);

  ///
  /// Call APMRegisterModule once to register a software module
  ///
  APMDECL_EXTERN APMModuleHandle APMFUNC APMRegisterModule(
    const char *utf8modulename) _APM_WARN_UNUSED;

  ///
  /// Call APMUnregisterModule once to unregister a software module
  ///
  APMDECL_EXTERN void APMFUNC APMUnregisterModule(
    APMModuleHandle module);

  ///
  /// Call APMRegisterFilterValue for each valid filter value
  ///
  APMDECL_EXTERN void APMFUNC APMRegisterFilterValue(
    APMApplicationHandle app,
    const char *value,
    const char *description);

  ///
  /// Call APMModuleRegisterEvent to provide information about every eventid used
  ///   for event tracing
  ///
  APMDECL_EXTERN void APMFUNC APMModuleRegisterEvent(
    APMModuleHandle module,
    APMEventId eventid,
    const char *description,
    const char *parameterdescription);

#if defined(APM_THREAD_TRANSACTION)

  ///
  /// Call APMGetCurrentApplication to get the application handle of the current
  ///   thread. This function returns the application handle of the latest
  ///   APMCreateContext or APMAttachContext request.
  ///
  APMDECL_EXTERN APMApplicationHandle APMFUNC APMGetCurrentApplication() _APM_WARN_UNUSED;

  ///
  /// Call APMCreateContext to create a new Software-Telemetry context.
  ///   If the filter value matches a currently started session filter,
  ///   logging is started for the current execution thread
  ///   Make sure to call APMReleaseContext when exiting the context
  ///
  APMDECL_EXTERN void APMFUNC APMCreateContext(
    APMApplicationHandle app,
    const char *filtervalue);

  ///
  /// Call APMGetContext to get a context handle, which can be passed to another
  ///   threads or processes. This context information is used by the
  ///   analysis components to track the control flow through the
  ///   distributed environment.
  ///
  APMDECL_EXTERN int APMFUNC APMGetContext(
    APMContext *context);

  ///
  /// Call APMAttachContext to restore the context acquired through
  ///   APMGetContext in another thread or process.
  ///
  APMDECL_EXTERN void APMFUNC APMAttachContext(
    APMApplicationHandle app,
    const APMContext *context);

  ///
  /// Call APMReleaseContext to finish logging and to flush logged information
  ///   to the telemetry infrastructure.
  ///   Make sure to call APMReleaseContext when exiting the context
  ///
  APMDECL_EXTERN void APMFUNC APMReleaseContext();

  ///
  /// Call APMGetSyncMark to get a synchronization handle that can be
  ///   passed to another thread or process to synchronize control flow.
  ///
  APMDECL_EXTERN int APMFUNC APMGetSyncMark(
    APMSyncMark *context);

  ///
  /// Call APMSetSyncMark to pass a synchronization handle you got from
  ///   APMGetSyncMark for synchronizing the sequence of events.
  ///
  APMDECL_EXTERN void APMFUNC APMSetSyncMark(
    const APMSyncMark *context);

  ///
  /// Call APMHasActiveContext to test, if a Software-Telemetry session is active
  ///   for some logging level.
  ///   This information should be used to avoid costly operations preparing
  ///   parameters for events that are currently not logged.
  ///
  APMDECL_EXTERN int APMFUNC APMHasActiveContext(
    APMEventLevel level);

  ///
  /// Call APMEvent* functions to log information
  ///
  /// \param [in] module Module handle returned by the APMRegisterModule function
  /// \param [in] eventid ID of the event being processed
  /// \param [in] level Event detail level for filtering
  /// \param [in] flags Flags to mark events with Enter-, Leave- or Wait-semantic
  ///
  APMDECL_EXTERN void APMFUNC APMEvent(
    APMModuleHandle module,
    APMEventId eventid,
    APMEventLevel level,
    APMEventFlags flags);

  ///
  /// Use APMEventStr to provide an additional string value as parameter
  /// \param [in] param utf8 encoded string to be logged
  ///
  APMDECL_EXTERN void APMFUNC APMEventStr(
    APMModuleHandle module,
    APMEventId eventid,
    APMEventLevel level,
    APMEventFlags flags,
    const char *param);

  ///
  /// Use APMEventArgs to provide additional parameters
  /// \param [in] paramCount Count of parameters to be passed
  /// \param [in] paramTypes Pointer to an array of size paramCount passing the
  ///   type information for the parameters to be passed
  /// \param [in] paramValues Pointer to a memory block containing the parameter values
  /// \param [in] paramValueSize Size of the parameter values in bytes
  ///
  APMDECL_EXTERN void APMFUNC APMEventArgs(
    APMModuleHandle module,
    APMEventId eventid,
    APMEventLevel level,
    APMEventFlags flags,
    UInt8 paramCount,
    APMParameterType *paramTypes,
    void *paramValues,
    unsigned paramValueSize);

#endif

  ///
  /// The APMParam* functions implement a simple generic way to pass parameters to
  ///   the APMEvent function
  ///
  /// To pass parameters call the functions in the following order
  ///   1. Allocate a parameter handle calling APMParamInit()
  ///   2. Call any of the APMParamAdd* functions passing the parameter handle returned
  ///      in step 1. and the parameter value
  ///   3. Call APMEventParam to log the event including the parameters
  ///   4. Call APMParamFree to release the memory allocated for the parameters
  ///
  typedef void *APMParamHandle;

  APMDECL_EXTERN APMParamHandle APMFUNC APMParamInit() _APM_WARN_UNUSED;
  APMDECL_EXTERN void APMFUNC APMParamAdd32(APMParamHandle param, UInt32 value);
  APMDECL_EXTERN void APMFUNC APMParamAdd64(APMParamHandle param, UInt64 value);
  APMDECL_EXTERN void APMFUNC APMParamAddString(APMParamHandle param, const char *value);
  APMDECL_EXTERN void APMFUNC APMParamFree(APMParamHandle param);

#if defined(APM_THREAD_TRANSACTION)

  APMDECL_EXTERN void APMFUNC APMEventParam(APMModuleHandle module,
    APMEventId eventid,
    APMEventLevel level,
    APMEventFlags flags,
    APMParamHandle param);

  ///
  /// Call APMReport to create a new feedback report.
  ///   The filter value will be matched to the description using the registered filter values.
  ///   A key-value may be generated by the calling application to associate further descriptions
  ///   to a feedback. If empty, a new feedback report will be generated on each call.
  ///   Textual description of the feedback entered by the user or generated by the application.
  ///   The APMReport function may be called more than one time. The descriptions of calls
  ///   with equal filter-values and equal nonempty key-values will be collected into one report.
  ///
  APMDECL_EXTERN void APMFUNC APMReport(
    APMApplicationHandle app,
    const char *filtervalue,
    const char *reportkey,
    const char *description);

  ///
  /// Call APMReportValue to add values to a previously created feedback report.
  ///   Data will be added as key/value-pairs and associated to the report using the
  ///   key-values. The key value is limited to 128 characters, the value is limited
  ///   to 32 kBytes.
  ///
  APMDECL_EXTERN void APMFUNC APMReportValue(
    APMApplicationHandle app,
    const char *reportkey,
    const char *key,
    const char *value);

#endif

  ///
  /// Call APMReportContent to add binary values to a previously created feedback report.
  ///   The filename value is limited to 128 characters and must be unique per report.
  ///
  APMDECL_EXTERN void APMFUNC APMReportContent(
    APMApplicationHandle app,
    const char *reportkey,
    const char *filename,
    const char *mimetype,
    const void *data,
    UInt64      size);

  ///
  /// Call APMReportFile to add file data to a previously created feedback report.
  ///   The filename value is limited to 128 characters and must be unique per report.
  ///
  APMDECL_EXTERN void APMFUNC APMReportFile(
    APMApplicationHandle app,
    const char *reportkey,
    const char *filename,
    const char *mimetype,
    FILE *filehandle);

  ///
  /// Call APMIsConnected to test, if the application has successfully registered to
  ///   an telemetry agent.
  ///
  APMDECL_EXTERN int APMFUNC APMIsConnected() _APM_WARN_UNUSED;

  ///
  /// Call APMIsCompatible to test, if the application was linked with a version that
  ///   is compatible with the dynamically loaded library.
  /// \param [in] size Size of the allocated memory region in bytes, allocate at least (200) bytes.
  /// \param [out] message Pointer to a memory region of size[param 1] where the message
  ///   will be stored. ex: "Compatibility text. app.telemetry Software-Telemetry 2010 Fall Release 5.3.5.8888\0"
  ///   The version or product name might be missing depending on the version used.
  ///
  APMDECL_EXTERN int APMFUNC APMIsCompatible(UInt32 size, char *message);

  ///
  /// Call APMDetachThread before leaving an application layer that is subsequently continued in the
  /// same or in another thread of the same process.
  /// \param [out] handle Handle to provide identity of the Detach-/AttachThread pair.
  ///
  APMDECL_EXTERN int APMFUNC APMDetachThread(APMTransactionHandle *handle);

  ///
  /// Call APMAttachThread when resuming an application layer that has been detached using
  /// APMThreadDetach
  /// \param [in] handle Handle to provide identity of the Detach-/AttachThread pair.
  ///
  APMDECL_EXTERN void APMFUNC APMAttachThread(APMTransactionHandle handle);

  ///
  /// Call APMGetApplicationLogLevel to get the configured log level for an application.
  ///
  APMDECL_EXTERN APMEventLevel APMFUNC APMGetApplicationLogLevel(
    APMApplicationHandle application);

#if defined(APM_EXPLICIT_TRANSACTION)

  ///
  /// Call APMGetCurrentApplication to get the application handle of the current
  ///   thread. This function returns the application handle of the latest
  ///   APMCreateContext or APMAttachContext request.
  ///
  APMDECL_EXTERN APMApplicationHandle APMFUNC APMTxGetCurrentApplication(APMTransactionHandle tx) _APM_WARN_UNUSED;

  ///
  /// Call APMCreateContext to create a new Software-Telemetry context.
  ///   If the filter value matches a currently started session filter,
  ///   logging is started for the current execution thread
  ///   Make sure to call APMReleaseContext when exiting the context
  ///
  APMDECL_EXTERN APMTransactionHandle APMFUNC APMTxCreateContext(
    APMTransactionHandle tx,
    APMApplicationHandle app,
    const char *filtervalue);

  ///
  /// Call APMGetContext to get a context handle, which can be passed to another
  ///   threads or processes. This context information is used by the
  ///   analysis components to track the control flow through the
  ///   distributed environment.
  ///
  APMDECL_EXTERN int APMFUNC APMTxGetContext(
    APMTransactionHandle tx,
    APMContext *context);

  ///
  /// Call APMAttachContext to restore the context acquired through
  ///   APMGetContext in another thread or process.
  ///
  APMDECL_EXTERN APMTransactionHandle APMFUNC APMTxAttachContext(
    APMTransactionHandle tx,
    APMApplicationHandle app,
    const APMContext *context);

  ///
  /// Call APMReleaseContext to finish logging and to flush logged information
  ///   to the telemetry infrastructure.
  ///   Make sure to call APMReleaseContext when exiting the context
  ///
  APMDECL_EXTERN void APMFUNC APMTxReleaseContext(
    APMTransactionHandle tx);

  ///
  /// Call APMGetSyncMark to get a synchronization handle that can be
  ///   passed to another thread or process to synchronize control flow.
  ///
  APMDECL_EXTERN int APMFUNC APMTxGetSyncMark(
    APMTransactionHandle tx,
    APMSyncMark *context);

  ///
  /// Call APMSetSyncMark to pass a synchronization handle you got from
  ///   APMGetSyncMark for synchronizing the sequence of events.
  ///
  APMDECL_EXTERN void APMFUNC APMTxSetSyncMark(
    APMTransactionHandle tx,
    const APMSyncMark *context);

  ///
  /// Call APMHasActiveContext to test, if a Software-Telemetry session is active
  ///   for some logging level.
  ///   This information should be used to avoid costly operations preparing
  ///   parameters for events that are currently not logged.
  ///
  APMDECL_EXTERN int APMFUNC APMTxHasActiveContext(
    APMTransactionHandle tx,
    APMEventLevel level);

  ///
  /// Call APMEvent* functions to log information
  ///
  /// \param [in] module Module handle returned by the APMRegisterModule function
  /// \param [in] eventid ID of the event being processed
  /// \param [in] level Event detail level for filtering
  /// \param [in] flags Flags to mark events with Enter-, Leave- or Wait-semantic
  ///
  APMDECL_EXTERN void APMFUNC APMTxEvent(
    APMTransactionHandle tx,
    APMModuleHandle module,
    APMEventId eventid,
    APMEventLevel level,
    APMEventFlags flags);

  ///
  /// Use APMEventStr to provide an additional string value as parameter
  /// \param [in] param utf8 encoded string to be logged
  ///
  APMDECL_EXTERN void APMFUNC APMTxEventStr(
    APMTransactionHandle tx,
    APMModuleHandle module,
    APMEventId eventid,
    APMEventLevel level,
    APMEventFlags flags,
    const char *param);

  ///
  /// Use APMEventArgs to provide additional parameters
  /// \param [in] paramCount Count of parameters to be passed
  /// \param [in] paramTypes Pointer to an array of size paramCount passing the
  ///   type information for the parameters to be passed
  /// \param [in] paramValues Pointer to a memory block containing the parameter values
  /// \param [in] paramValueSize Size of the parameter values in bytes
  ///
  APMDECL_EXTERN void APMFUNC APMTxEventArgs(
    APMTransactionHandle tx,
    APMModuleHandle module,
    APMEventId eventid,
    APMEventLevel level,
    APMEventFlags flags,
    UInt8 paramCount,
    APMParameterType *paramTypes,
    void *paramValues,
    unsigned paramValueSize);

  APMDECL_EXTERN void APMFUNC APMTxEventParam(
    APMTransactionHandle tx,
    APMModuleHandle module,
    APMEventId eventid,
    APMEventLevel level,
    APMEventFlags flags,
    APMParamHandle param);

  ///
  /// Call APMReport to create a new feedback report.
  ///   The filter value will be matched to the description using the registered filter values.
  ///   A key-value may be generated by the calling application to associate further descriptions
  ///   to a feedback. If empty, a new feedback report will be generated on each call.
  ///   Textual description of the feedback entered by the user or generated by the application.
  ///   The APMReport function may be called more than one time. The descriptions of calls
  ///   with equal filter-values and equal nonempty key-values will be collected into one report.
  ///
  APMDECL_EXTERN void APMFUNC APMTxReport(
    APMTransactionHandle tx,
    APMApplicationHandle app,
    const char *filtervalue,
    const char *reportkey,
    const char *description);

  ///
  /// Call APMReportValue to add values to a previously created feedback report.
  ///   Data will be added as key/value-pairs and associated to the report using the
  ///   key-values. The key value is limited to 128 characters, the value is limited
  ///   to 32 kBytes.
  ///
  APMDECL_EXTERN void APMFUNC APMTxReportValue(
    APMTransactionHandle tx,
    APMApplicationHandle app,
    const char *reportkey,
    const char *key,
    const char *value);

#endif

#if defined(APM_WEB_API)

  APMDECL_EXTERN int APMFUNC APMWebGetData(
    APMApplicationHandle app,
    void *buffer,
    unsigned int *size);

  APMDECL_EXTERN void APMFUNC APMWebSetData(
    APMApplicationHandle app,
    const void *buffer,
    unsigned int size);

  APMDECL_EXTERN int APMFUNC APMWebHasData(
    APMApplicationHandle app);

  typedef void *APMWebPersistStorageHandle;
  typedef APMWebPersistStorageHandle (APMFUNC *APMWebPersistCreateFunction)(void **telemetrydatabuffer, size_t *bufferlength);
  typedef void (APMFUNC *APMWebPersistWriteApplicationDataFunction)(APMWebPersistStorageHandle handle, void *data, size_t length);
  typedef void (APMFUNC *APMWebPersistFlushDataFunction)(APMWebPersistStorageHandle handle);
  typedef void (APMFUNC *APMWebPersistCloseFunction)(APMWebPersistStorageHandle handle, int remove);

  struct IAPMWebPersistStore
  {
    APMWebPersistCreateFunction               createfn;
    APMWebPersistWriteApplicationDataFunction writeappdatafn;
    APMWebPersistFlushDataFunction            flushfn;
    APMWebPersistCloseFunction                closefn;
  };

  typedef void *APMWebRecoverApplicationHandle;
  APMDECL_EXTERN APMWebRecoverApplicationHandle APMFUNC APMWebRecoverApplication(void *appdata, size_t appdatasize, void *telemetrydata, size_t telemetrydatasize);
  APMDECL_EXTERN void APMFUNC APMWebRecoverClose(APMWebRecoverApplicationHandle app);
  APMDECL_EXTERN int  APMFUNC APMWebRecoverGetData(APMWebRecoverApplicationHandle app, void *buffer, unsigned int *size);
  APMDECL_EXTERN void APMFUNC APMWebRecoverConfirmData(APMWebRecoverApplicationHandle app);

  APMDECL_EXTERN void APMFUNC APMWebSetPersistHandler(IAPMWebPersistStore *handler);
  APMDECL_EXTERN void APMFUNC APMWebFlush();

#endif

#ifdef __cplusplus
}
#endif

#undef _APM_WARN_UNUSED

#endif

