/* ==========================================================================
 * Copyright (c) Fabasoft R&D GmbH, A-4020 Linz, 1995-2015.
 *
 * All rights reserved. All hardware and software names used are registered
 * trade names and/or registered trademarks of the respective manufacturers.
 *
 * The user of this computer program acknowledges that the above copyright
 * notice, which constitutes the Universal Copyright Convention, will be
 * attached at the position in the function of the computer program which the
 * author has deemed to sufficiently express the reservation of copyright.
 * It is prohibited for customers, users and/or third parties to remove,
 * modify or move this copyright notice.
 * ========================================================================= */

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.InetSocketAddress;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;

import sun.misc.BASE64Decoder;

import com.apptelemetry.apm.APM;
import com.apptelemetry.apm.APMApplication;
import com.apptelemetry.apm.APMModule;
import com.apptelemetry.apm.APMParameter;
import com.sun.net.httpserver.Headers;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

/**
 * Main application class for handling command line parameter and starting the web server.
 */
public class HttpStockServiceInstrumented {
	private static Logger logger = Logger.getLogger(HttpStockServiceInstrumented.class.getName());
	private static final int DEFAULT_PORT = 8085;
	private static int serverPort = DEFAULT_PORT;

	// global Software-Telemetry API
	static APMApplication application;
	static APMModule module;

	static final int EVENT_HTTP_GET_REQUEST = 200;
	static final int EVENT_HTTP_POST_REQUEST = 201;
	static final int EVENT_PROCESS_REQUEST = 210;
	static final int EVENT_PROCESS_REQUEST_PARAM_IN = 211;
	static final int EVENT_PROCESS_REQUEST_PARAM_OUT = 212;
	static final int EVENT_PROCESS_REQUEST_PARAMS = 213;

	static void registerEvents(APMModule module) {
		module.RegisterEvent(EVENT_HTTP_GET_REQUEST, "HTTP Get Request", "");
		module.RegisterEvent(EVENT_HTTP_POST_REQUEST, "HTTP Post Request", "");
		module.RegisterEvent(EVENT_PROCESS_REQUEST, "Process HTTP Request", "");
		module.RegisterEvent(EVENT_PROCESS_REQUEST_PARAM_IN, "Process HTTP Request - Enter", "request");
		module.RegisterEvent(EVENT_PROCESS_REQUEST_PARAM_OUT, "Process HTTP Request - Leave", "response");
		module.RegisterEvent(EVENT_PROCESS_REQUEST_PARAMS, "Process HTTP Request - Evaluate Parameters", "id;thread;stock");
	}

	public static void main(String[] args) {
		// ----- command line parameter handling -----
		handleCmdLineParams(args);
		logger.fine("Defined server port: " + serverPort);

		// ~~~ Software-Telemetry: register Software-Telemetry application and the module ~~~
		application = new APMApplication("Fabasoft app.telemetry", "Demo", "Backend Services", "Stock Service");
		module = new APMModule("Stock Service Request Handler");
		registerEvents(module);
		// filters are registered on server side

		// ----- creating and starting the web server -----
		InetSocketAddress addr = new InetSocketAddress(serverPort);
		HttpServer server;
		try {
			server = HttpServer.create(addr, 0);
			// assign HTTP request handler (with the implementation of the request handling from other application (client))
			server.createContext("/", new StockHandler());
			server.setExecutor(Executors.newCachedThreadPool());
			server.start();
			logger.info("HTTP Server is listening on port: " + serverPort);
			logger.info("Waiting for incoming requests ...");

		}
		catch (IOException ex) {
			logger.log(Level.SEVERE, "Fatal: HTTP server could not be created! Application will exit.", ex);
		}

	}

	private static void handleCmdLineParams(String[] args) {
		logger.info("***** Fabasoft app.telemetry 2010 **********");
		logger.info("***** Java Sample Application (Stock Service) *****");
		logger.info("***** with Software-Telemetry instrumentation *****");
		if ((args != null) && (args.length > 0)) {
			if (args[0].contains("help")) {
				System.out.println("*************************************************");
				System.out.println("***** Fabasoft app.telemetry 2010 ********");
				System.out.println("**** Java Sample Application (Stock Service) ****");
				System.out.println("**** with Software-Telemetry instrumentation ****");
				System.out.println("*************************************************");
				System.out.println("Command line parameters:");
				System.out.println("--help");
				System.out.println("--port <serverport>");
				System.exit(1);
			}
			else {
				for (int i = 0; i < args.length; i++) {
					String lowArg = args[i].toLowerCase();
					if (lowArg.contains("port")) {
						if (args.length <= i) {
							logger.severe("Tried to load new port, but no value available at command line");
						}
						else {
							String portStr = args[++i];
							try {
								int port = Integer.parseInt(portStr);
								serverPort = port;
							}
							catch (NumberFormatException ex) {
								logger.warning("Specified port is not a valid integer number: " + portStr);
							}
						}
					}
				} // for
			}
		}
	}

	/**
	 * Implementation of request handling.
	 * Perform action that is requested by other application (client).
	 */
	private static class StockHandler implements HttpHandler {
		private static Logger logger = Logger.getLogger(HttpStockServiceInstrumented.class.getName());

		private static BASE64Decoder base64Decoder = new BASE64Decoder();

		@Override
		public void handle(HttpExchange exchange) throws IOException {
			logger.info("Incoming request from: " + exchange.getRemoteAddress().toString());
			String requestMethod = exchange.getRequestMethod();
			logger.finer("HTTP-Method=" + requestMethod + ", protocol=" + exchange.getProtocol());

			// ~~~ Software-Telemetry: get instrumentation context from request headers and attach current instrumentation to incoming request  ~~~
			byte[] telemetryContext;
			Headers requestHeaders = exchange.getRequestHeaders();
			String contextString = null;
			if (requestHeaders.containsKey("TelemetryContext")) {
				contextString = requestHeaders.getFirst("TelemetryContext");
			}
			else if (requestHeaders.containsKey("telemetrycontext")) {
				contextString = requestHeaders.getFirst("telemetrycontext");
			}
			if (contextString != null) {
				telemetryContext = base64Decoder.decodeBuffer(contextString);
				application.AttachContext(telemetryContext);
			}
			else {
				logger.warning("Software-Telemetry context could not be attached because it was not found in the HTTP-request headers!");
			}

			// ~~~ Software-Telemetry: instrument with ENTER and LEAVE events on normal level
			if (requestMethod.equalsIgnoreCase("GET")) {
				module.Event(EVENT_HTTP_GET_REQUEST, APM.EVENT_LEVEL_NORMAL, APM.FLAG_ENTER);
				Headers responseHeaders = exchange.getResponseHeaders();
				responseHeaders.set("Content-Type", "text/plain");
				exchange.sendResponseHeaders(200, 0);

				logger.finer("Handle GET request with default response.");
				writeOutputString(exchange.getResponseBody(), processData(null));

				module.Event(EVENT_HTTP_GET_REQUEST, APM.EVENT_LEVEL_NORMAL, APM.FLAG_LEAVE);

			}
			else if (requestMethod.equalsIgnoreCase("POST")) {
				module.Event(EVENT_HTTP_POST_REQUEST, APM.EVENT_LEVEL_NORMAL, APM.FLAG_ENTER);
				// read request
				String requestInput = readInputAsString(exchange.getRequestBody());

				// write response
				Headers responseHeaders = exchange.getResponseHeaders();
				responseHeaders.set("Content-Type", "text/plain");
				exchange.sendResponseHeaders(200, 0);

				logger.finer("Handle POST request with given data.");
				writeOutputString(exchange.getResponseBody(), processData(requestInput));
				module.Event(EVENT_HTTP_POST_REQUEST, APM.EVENT_LEVEL_NORMAL, APM.FLAG_LEAVE);
			}

			// ~~~ Software-Telemetry: leave stock service module and return back to "server" ~~~
			application.ReleaseContext();
		}

		/**
		 * Process input data and build up result data.
		 * Returns number of available stock items for id x.
		 * If threads >= 10, server can handle request without wait,
		 * otherwise server is busy and will wait 10-n seconds.
		 *
		 * @param input
		 *            Request data string in format: "id=<x>,threads=<n>
		 * @return Response data string (number of available stock items for given id)
		 */
		private String processData(String input) {
			module.EventStr(EVENT_PROCESS_REQUEST_PARAM_IN, APM.EVENT_LEVEL_DETAIL, APM.FLAG_ENTER, input);
			String response = "0";
			if ((input == null) || input.isEmpty()) {
				// return default response
				response = "0";
				logger.info("Returning value '" + response + "' to client.");
				module.EventStr(EVENT_PROCESS_REQUEST, APM.EVENT_LEVEL_DEBUG, APM.FLAG_WARNING, "No input data");
			}
			else {
				logger.fine("Processing request data: " + input);
				String[] fields = input.split(",");
				String fieldId = "", fieldThreads = "";
				for (String f : fields) {
					if (f.toLowerCase().contains("id")) {
						fieldId = f;
					}
					else if (f.toLowerCase().contains("thread")) {
						fieldThreads = f;
					}
				}

				long id = getLongValueFromKeyValueField(fieldId);
				module.EventStr(EVENT_PROCESS_REQUEST, APM.EVENT_LEVEL_DEBUG, APM.FLAG_NONE, "Calculating available stock count for ID: " + id);
				int threads = (int)getLongValueFromKeyValueField(fieldThreads);
				logger.finer("Request values: id=" + id + ", threads=" + threads);
				int stock = Math.abs(((int)(id * Math.random())));
				response = String.valueOf(stock);
				APMParameter param = new APMParameter();
				param.AddInt64(id);
				param.AddInt32(threads);
				param.AddInt32(stock);
				module.EventParam(EVENT_PROCESS_REQUEST_PARAMS, APM.EVENT_LEVEL_DEBUG, APM.FLAG_NONE, param);

				int sleepTime = 0;
				if ((threads < 10) && (threads >= 0)) {
					sleepTime = 10 - threads;
				}
				logger.info("Returning value '" + response + "' to client after sleep time of " + sleepTime + " sec. ...");
				if (sleepTime > 0) {
					module.EventStr(EVENT_PROCESS_REQUEST, APM.EVENT_LEVEL_DEBUG, APM.FLAG_ENTER, "Not enough worker threads! Wait time (sec): "
							+ String.valueOf(sleepTime));
					try {
						Thread.sleep(1000 * sleepTime);
					}
					catch (InterruptedException e) {
					}
					module.Event(EVENT_PROCESS_REQUEST, APM.EVENT_LEVEL_DEBUG, APM.FLAG_LEAVE);
				}
			}
			module.EventStr(EVENT_PROCESS_REQUEST_PARAM_OUT, APM.EVENT_LEVEL_DETAIL, APM.FLAG_LEAVE, response);
			return response;
		}

		private long getLongValueFromKeyValueField(String keyValueField) {
			long value = -1;
			String[] fields = keyValueField.split("=");
			if (fields.length == 2) {
				String valueStr = fields[1];
				try {
					value = Long.parseLong(valueStr);
				}
				catch (NumberFormatException ex) {
					logger.finer("Value could not be parsed as long: " + keyValueField);
				}
			}
			return value;
		}

		/**
		 * Read input and close input stream at the end.
		 *
		 * @param inStream
		 *            The input stream to read data from.
		 * @return String with the full content of the input stream.
		 */
		private static String readInputAsString(InputStream inStream) {
			StringBuffer response = new StringBuffer();
			String line;
			try {
				BufferedReader inReader = new BufferedReader(new InputStreamReader(inStream));
				while ((line = inReader.readLine()) != null) {
					response.append(line);
				}
			}
			catch (IOException ex) {
				logger.warning("Input could not be read from stream!");
			}
			finally {
				if (inStream != null) {
					try {
						inStream.close();
					}
					catch (IOException e) {
						logger.warning("Input stream could not be closed.");
					}
				}
			}
			logger.finest("Read input from stream: " + response.toString());
			return response.toString();
		}

		/**
		 * Write data to output stream.
		 *
		 * @param outStream
		 *            Target stream to write output to.
		 * @param text
		 *            (String) data to write to output stream.
		 */
		private static void writeOutputString(OutputStream outStream, String text) {
			logger.finest("Writing output to stream: " + text);
			BufferedWriter outWriter = new BufferedWriter(new OutputStreamWriter(outStream));
			try {
				outWriter.write(text);
				outWriter.flush();
				outStream.flush();
				outStream.close();
			}
			catch (IOException e) {
				logger.warning("Output could not be written to stream!");
			}
			finally {
				if (outStream != null) {
					try {
						outStream.close();
					}
					catch (IOException e) {
						logger.warning("Output stream could not be closed.");
					}
				}
			}
		}
	} // StockHandler

}
