/* ==========================================================================
 * Copyright (c) Fabasoft R&D GmbH, A-4020 Linz, 1995-2015.
 *
 * All rights reserved. All hardware and software names used are registered
 * trade names and/or registered trademarks of the respective manufacturers.
 *
 * The user of this computer program acknowledges that the above copyright
 * notice, which constitutes the Universal Copyright Convention, will be
 * attached at the position in the function of the computer program which the
 * author has deemed to sufficiently express the reservation of copyright.
 * It is prohibited for customers, users and/or third parties to remove,
 * modify or move this copyright notice.
 * ========================================================================= */

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.InetSocketAddress;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;

// import for processing telemetry context (Base64-conversion)
import sun.misc.BASE64Decoder;

// +++ APM-API: importing classes from softwaretelemetry.jar +++
import com.apptelemetry.apm.APM;
import com.apptelemetry.apm.APMApplication;
import com.apptelemetry.apm.APMModule;
import com.apptelemetry.apm.APMParameter;
// imports used for the lightweight HTTP-server from SUN for processing HTTP-requests
import com.sun.net.httpserver.Headers;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

/**
 * Java application for handling HTTP requests which are instrumented by means of the APM Software-Telemetry API.
 * All code parts that are part of the Software-Telemetry code instrumentation are marked with the following comment: +++ APM-API: ... +++
 */
public class SimpleHttpStockServiceInstrumented {
	private static Logger logger = Logger.getLogger(SimpleHttpStockServiceInstrumented.class.getName());
	private static final int SERVER_LISTEN_PORT = 8085;

	// +++ APM-API: global Software-Telemetry API fields +++
	public static final int EVENT_HTTP_GET_REQUEST = 200;
	public static final int EVENT_HTTP_POST_REQUEST = 201;
	public static final int EVENT_PROCESS_REQUEST = 210;
	public static final int EVENT_PROCESS_REQUEST_PARAM_IN = 211;
	public static final int EVENT_PROCESS_REQUEST_PARAM_OUT = 212;
	public static final int EVENT_PROCESS_REQUEST_PARAMS = 213;

	static APMApplication application;
	static APMModule module;

	// +++ APM-API: register Software-Telemetry events globally +++
	static void registerEvents(APMModule module) {
		module.RegisterEvent(EVENT_HTTP_GET_REQUEST, "HTTP Get Request", "");
		module.RegisterEvent(EVENT_HTTP_POST_REQUEST, "HTTP Post Request", "");
		module.RegisterEvent(EVENT_PROCESS_REQUEST, "Process HTTP Request", "");
		module.RegisterEvent(EVENT_PROCESS_REQUEST_PARAM_IN, "Process HTTP Request - Enter", "request");
		module.RegisterEvent(EVENT_PROCESS_REQUEST_PARAM_OUT, "Process HTTP Request - Leave", "response");
		module.RegisterEvent(EVENT_PROCESS_REQUEST_PARAMS, "Process HTTP Request - Evaluate Parameters", "id;thread;stock");
	}

	public static void main(String[] args) {
		// +++ APM-API: register Software-Telemetry application and the module and the events +++
		application = new APMApplication("Fabasoft app.telemetry", "Demo", "Backend Services", "Stock Service");
		module = new APMModule("Stock Service Request Handler");
		registerEvents(module);

		// ----- creating and starting the web server -----
		InetSocketAddress addr = new InetSocketAddress(SERVER_LISTEN_PORT);
		HttpServer server;
		try {
			server = HttpServer.create(addr, 0);
			// assign HTTP request handler (with the implementation of the request handling from other application (client))
			server.createContext("/", new StockHandler());
			server.setExecutor(Executors.newCachedThreadPool());
			server.start();
			logger.info("HTTP Server is listening on port: " + SERVER_LISTEN_PORT);
			logger.info("Waiting for incoming requests ...");

		}
		catch (IOException ex) {
			logger.log(Level.SEVERE, "Fatal: HTTP server could not be created! Application will exit.", ex);
		}
	}

	/**
	 * Implementation of HTTP-request handling.
	 * Perform action on incoming request (invoked by other application (OnlineShop web formular)).
	 */
	private static class StockHandler implements HttpHandler {
		private static Logger logger = Logger.getLogger(SimpleHttpStockServiceInstrumented.class.getName());

		private static BASE64Decoder base64Decoder = new BASE64Decoder();

		@Override
		public void handle(HttpExchange exchange) throws IOException {
			// +++ APM-API (OPTIONAL): create a local module-based context so that the user also could filter for this Java module only.
			//SimpleHttpStockServiceInstrumented.api.CreateContext(StockServiceRequestStrudlModule.JAVA_MODULE_FILTER_VALUE);

			logger.info("Incoming request from: " + exchange.getRemoteAddress().toString());
			String requestMethod = exchange.getRequestMethod();
			logger.finer("HTTP-Method=" + requestMethod + ", protocol=" + exchange.getProtocol());

			// get instrumentation context from request headers and attach current instrumentation to incoming request
			byte[] telemetryContext;
			Headers requestHeaders = exchange.getRequestHeaders();
			String contextString = null;
			contextString = requestHeaders.getFirst("TelemetryContext");
			if (contextString != null) {
				// +++ APM-API: get and decode (Base64) telemetry context from web request and attach to this context,
				// so that the following events can be assigned correctly +++
				telemetryContext = base64Decoder.decodeBuffer(contextString);
				application.AttachContext(telemetryContext);
			}
			else {
				logger.warning("Software-Telemetry context could not be attached because it was not found in the HTTP-request headers!");
			}

			// +++ APM-API: instrument the main request with ENTER and LEAVE events on normal level +++
			if (requestMethod.equalsIgnoreCase("POST")) {
				// +++ APM-API: request enter event
				module.Event(EVENT_HTTP_POST_REQUEST, APM.EVENT_LEVEL_NORMAL, APM.FLAG_ENTER);
				// read request
				String requestInput = readInputAsString(exchange.getRequestBody());

				// write response
				Headers responseHeaders = exchange.getResponseHeaders();
				responseHeaders.set("Content-Type", "text/plain");
				exchange.sendResponseHeaders(200, 0);

				// processing the request inside the helper method processData(...)
				writeOutputString(exchange.getResponseBody(), processData(requestInput));
				// +++ APM-API: request leave event
				module.Event(EVENT_HTTP_POST_REQUEST, APM.EVENT_LEVEL_NORMAL, APM.FLAG_LEAVE);
			}

			// +++ APM-API: leave stock service module and return back to "server" (release all contexts: the one that was created and the one that was attached to) +++
			application.ReleaseContext();
		}

		/**
		 * Process input data and build up result data.
		 * Returns number of available stock items for id x.
		 * If threads >= 10, server can handle request without wait,
		 * otherwise server is busy and will wait 10-n seconds.
		 *
		 * @param input
		 *            Request data string in format: "id=<x>,threads=<n>
		 * @return Response data string (number of available stock items for given id)
		 */
		private String processData(String input) {
			// +++ APM-API: instrumentation point with input as parameter +++
			module.EventStr(EVENT_PROCESS_REQUEST_PARAM_IN, APM.EVENT_LEVEL_DETAIL, APM.FLAG_ENTER, input);
			String response = "0";
			if ((input == null) || input.isEmpty()) {
				// return default response
				response = "0";
				logger.info("Returning value '" + response + "' to client.");
				module.EventStr(EVENT_PROCESS_REQUEST, APM.EVENT_LEVEL_DEBUG, APM.FLAG_WARNING, "No input data");

			}
			else {
				logger.fine("Processing request data: " + input);
				// split up the fields (id=...,threads=...) from the request data into values
				String[] fields = input.split(",");
				String fieldId = "", fieldThreads = "";
				for (String f : fields) {
					if (f.toLowerCase().contains("id")) {
						fieldId = f;
					}
					else if (f.toLowerCase().contains("thread")) {
						fieldThreads = f;
					}
				}

				long id = getLongValueFromKeyValueField(fieldId);
				// +++ APM-API: instrumentation point with id as parameter +++
				module.EventStr(EVENT_PROCESS_REQUEST, APM.EVENT_LEVEL_DEBUG, APM.FLAG_NONE, "Calculating available stock count for ID: " + id);
				int threads = (int)getLongValueFromKeyValueField(fieldThreads);
				int stock = Math.abs(((int)(id * Math.random())));
				response = String.valueOf(stock);

				// +++ APM-API: instrumentation point with a structured parameter (create the APMParameter object and pass it to the EventParam method +++
				APMParameter param = new APMParameter();
				param.AddInt64(id);
				param.AddInt32(threads);
				param.AddInt32(stock);
				module.EventParam(EVENT_PROCESS_REQUEST_PARAMS, APM.EVENT_LEVEL_DEBUG, APM.FLAG_NONE, param);
				logger.finer("Request values: id=" + id + ", threads=" + threads);

				// sleep some time if fewer than 10 threads defined (to simulate busy server)
				int sleepTime = 0;
				if ((threads < 10) && (threads >= 0)) {
					sleepTime = 10 - threads;
				}
				logger.info("Returning value '" + response + "' to client after sleep time of " + sleepTime + " sec. ...");
				if (sleepTime > 0) {
					// +++ APM-API: instrumentation point where thread goes to sleep for some time
					// (WAIT-flag is set on the server side and not here for the deepest module, because this module is not waiting for other instrumented modules) +++
					module.EventStr(EVENT_PROCESS_REQUEST, APM.EVENT_LEVEL_DEBUG, (APM.FLAG_ENTER),
							"Not enough worker threads! Wait time (sec): " + String.valueOf(sleepTime));
					try {
						Thread.sleep(1000 * sleepTime);
					}
					catch (InterruptedException e) {
					}
					module.Event(EVENT_PROCESS_REQUEST, APM.EVENT_LEVEL_DEBUG, (APM.FLAG_LEAVE));
				}
			}

			module.EventStr(EVENT_PROCESS_REQUEST_PARAM_OUT, APM.EVENT_LEVEL_DETAIL, APM.FLAG_LEAVE, response);
			return response;
		}

		// ~~~~~~~~~~ PRIVATE HELPER METHODS ~~~~~~~~~~

		/**
		 * Get a long numerical value from the "key=value" field
		 *
		 * @param keyValueField
		 *            "key=value" string
		 * @return the value as long number
		 */
		private long getLongValueFromKeyValueField(String keyValueField) {
			long value = -1;
			String[] fields = keyValueField.split("=");
			if (fields.length == 2) {
				String valueStr = fields[1];
				try {
					value = Long.parseLong(valueStr);
				}
				catch (NumberFormatException ex) {
					logger.finer("Value could not be parsed as long: " + keyValueField);
				}
			}
			return value;
		}

		/**
		 * Read input and close input stream at the end.
		 *
		 * @param inStream
		 *            The input stream to read data from.
		 * @return String with the full content of the input stream.
		 */
		private static String readInputAsString(InputStream inStream) {
			StringBuffer response = new StringBuffer();
			String line;
			try {
				BufferedReader inReader = new BufferedReader(new InputStreamReader(inStream));
				while ((line = inReader.readLine()) != null) {
					response.append(line);
				}
			}
			catch (IOException ex) {
				logger.warning("Input could not be read from stream!");
			}
			finally {
				if (inStream != null) {
					try {
						inStream.close();
					}
					catch (IOException e) {
						logger.warning("Input stream could not be closed.");
					}
				}
			}
			logger.finest("Read input from stream: " + response.toString());
			return response.toString();
		}

		/**
		 * Write data to output stream.
		 *
		 * @param outStream
		 *            Target stream to write output to.
		 * @param text
		 *            (String) data to write to output stream.
		 */
		private static void writeOutputString(OutputStream outStream, String text) {
			logger.finest("Writing output to stream: " + text);
			BufferedWriter outWriter = new BufferedWriter(new OutputStreamWriter(outStream));
			try {
				outWriter.write(text);
				outWriter.flush();
				outStream.flush();
				outStream.close();
			}
			catch (IOException e) {
				logger.warning("Output could not be written to stream!");
			}
			finally {
				if (outStream != null) {
					try {
						outStream.close();
					}
					catch (IOException e) {
						logger.warning("Output stream could not be closed.");
					}
				}
			}
		}

	} // StockHandler

}
