/*==========================================================================
  Copyright (c) Fabasoft R&D GmbH, A-4020 Linz, 1995-2014.

  All rights reserved. All hardware and software names used are registered
  trade names and/or registered trademarks of the respective manufacturers.

  The user of this computer program acknowledges that the above copyright
  notice, which constitutes the Universal Copyright Convention, will be
  attached at the position in the function of the computer program which the
  author has deemed to sufficiently express the reservation of copyright.
  It is prohibited for customers, users and/or third parties to remove,
  modify or move this copyright notice.
  ==========================================================================*/

#include <ngx_config.h>
#include <ngx_core.h>
#include <ngx_http.h>

#include <softwaretelemetry.h>

typedef struct {
    ngx_flag_t   enabled;

    ngx_str_t    app_name;
    ngx_str_t    app_id;
    ngx_str_t    app_tier_name;
    ngx_str_t    app_tier_id;

    ngx_str_t    context_header;
    u_char      *context_header_lc;
    ngx_str_t    syncmark_header;
    u_char      *syncmark_header_lc;

    ngx_uint_t   context_header_hash;
    ngx_uint_t   syncmark_header_hash;  

    APMApplicationHandle app_handle;
    APMModuleHandle module_handle;

    ngx_int_t    thread_handle_index;
    ngx_int_t    content_length_index;
    ngx_int_t    proxy_host_index;
    ngx_int_t    proxy_port_index;
} ngx_http_apm_conf_t;

static ngx_http_output_header_filter_pt  ngx_http_next_header_filter;
static ngx_http_output_body_filter_pt    ngx_http_next_body_filter;

static ngx_int_t ngx_http_apm_filter_init(ngx_conf_t *cf);
static void *ngx_http_apm_create_loc_conf(ngx_conf_t *cf);
static char *ngx_http_apm_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child);
static ngx_int_t ngx_http_apm_init_handler(ngx_http_request_t *r);
static ngx_int_t ngx_http_apm_get_variable(ngx_http_request_t *r, ngx_uint_t index, ssize_t *value);
static ngx_int_t ngx_http_apm_set_variable(ngx_http_request_t *r, ngx_uint_t index, ssize_t value);

static void ngx_http_apm_detach_thread(ngx_http_request_t *r, ngx_http_apm_conf_t *conf)
{
    APMTransactionHandle transaction_handle;
    if (APMDetachThread(&transaction_handle)) {
        ngx_http_apm_set_variable(r, conf->thread_handle_index, (ssize_t)transaction_handle);
    }
}

static ngx_int_t ngx_http_apm_attach_thread(ngx_http_request_t *r, ngx_http_apm_conf_t *conf)
{
    ssize_t transaction_handle;
    if (NGX_OK == ngx_http_apm_get_variable(r, conf->thread_handle_index, &transaction_handle)) {
        APMAttachThread((APMTransactionHandle)transaction_handle);
        return NGX_OK;
    }
    return NGX_ERROR;
}

static ngx_command_t  ngx_http_apm_commands[] = {

    { ngx_string("apm_enabled"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
      ngx_conf_set_flag_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_apm_conf_t, enabled),
      NULL },

    { ngx_string("apm_app_name"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_conf_set_str_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_apm_conf_t, app_name),
      NULL },

    { ngx_string("apm_app_id"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_conf_set_str_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_apm_conf_t, app_id),
      NULL },

    { ngx_string("apm_app_tier_name"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_conf_set_str_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_apm_conf_t, app_tier_name),
      NULL },

    { ngx_string("apm_app_tier_id"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_conf_set_str_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_apm_conf_t, app_tier_id),
      NULL },

    { ngx_string("apm_context_header"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_conf_set_str_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_apm_conf_t, context_header),
      NULL },

    { ngx_string("apm_syncmark_header"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_conf_set_str_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_apm_conf_t, syncmark_header),
      NULL },

      ngx_null_command
};


static ngx_http_module_t  ngx_http_apm_module_ctx = {
    NULL,                                  /* preconfiguration */
    ngx_http_apm_filter_init,              /* postconfiguration */

    NULL,                                  /* create main configuration */
    NULL,                                  /* init main configuration */

    NULL,                                  /* create server configuration */
    NULL,                                  /* merge server configuration */

    ngx_http_apm_create_loc_conf,          /* create location configuration */
    ngx_http_apm_merge_loc_conf            /* merge location configuration */
};


ngx_module_t  ngx_http_apm_module = {
    NGX_MODULE_V1,
    &ngx_http_apm_module_ctx,              /* module context */
    ngx_http_apm_commands,                 /* module directives */
    NGX_HTTP_MODULE,                       /* module type */
    NULL,                                  /* init master */
    NULL,                                  /* init module */
    NULL,                                  /* init process */
    NULL,                                  /* init thread */
    NULL,                                  /* exit thread */
    NULL,                                  /* exit process */
    NULL,                                  /* exit master */
    NGX_MODULE_V1_PADDING
};

static const char *
apm_pcstrdup(ngx_pool_t *pool, ngx_str_t *src)
{
    u_char  *dst;

    dst = ngx_pnalloc(pool, src->len + 1);
    if (dst == NULL) {
        return NULL;
    }

    ngx_memcpy(dst, src->data, src->len);
    dst[src->len] = '\0';

    return (const char *)dst;
}

static void
apm_add_header_variable(ngx_http_request_t *r, ngx_table_elt_t *header, APMParamHandle ph)
{
  if (header) {
    APMParamAddString(ph, apm_pcstrdup(r->pool, &header->value));
  }
  else {
    APMParamAddString(ph, "");
  }
}

static void
ngx_http_apm_set_header_base64(ngx_http_request_t *r, ngx_table_elt_t *header, ngx_str_t *src)
{
    ngx_str_t base64;

    base64.len = ngx_base64_encoded_length(src->len);
    base64.data = ngx_palloc(r->pool, base64.len);
    if (base64.data == NULL) {
        return;
    }

    ngx_encode_base64(&base64, src);
    header->value.data = base64.data;
    header->value.len = base64.len;
}

static ngx_int_t
ngx_http_apm_init_handler(ngx_http_request_t *r)
{
    ngx_http_apm_conf_t           *cfg;

    cfg = ngx_http_get_module_loc_conf(r, ngx_http_apm_module);
    if (cfg == NULL) {
        return NGX_OK;
    }

    if (cfg->enabled) {
        ngx_list_part_t             *part;
        ngx_table_elt_t             *header;
        ngx_table_elt_t             *context_header;
        ngx_uint_t		     i, j, hash;
        u_char                      *p;
        size_t                       len;
        APMContext                   context_token;
        APMParamHandle		     ph;
        ngx_str_t                    token;

        if (cfg->app_handle == 0) {
            cfg->app_handle = APMRegisterApplication(
                apm_pcstrdup(r->pool, &cfg->app_name),
                apm_pcstrdup(r->pool, &cfg->app_id),
                apm_pcstrdup(r->pool, &cfg->app_tier_name),
                apm_pcstrdup(r->pool, &cfg->app_tier_id),
                NULL);

           cfg->module_handle = APMRegisterModule("Request Processing");
           APMModuleRegisterEvent(cfg->module_handle, 600, "Begin Request", "Content-Length;Content-Type;Remote-Host;Remote Address;Query-String;Request-Method;Application Pool");
           APMModuleRegisterEvent(cfg->module_handle, 601, "End Request", "HTTP Status;Content-Length");
           APMModuleRegisterEvent(cfg->module_handle, 603, "Authorization", "Auth-User;Auth-Type");
           APMModuleRegisterEvent(cfg->module_handle, 606, "Response Info", "Content-Length;Content-Type");
           APMModuleRegisterEvent(cfg->module_handle, 607, "Request Header", "name;value");
           APMModuleRegisterEvent(cfg->module_handle, 608, "Request URL", "url");
           APMModuleRegisterEvent(cfg->module_handle, 610, "Proxy", "host;port;peer");
        }

        if (NGX_ERROR == ngx_http_apm_attach_thread(r, cfg)) {
            APMCreateContext(cfg->app_handle, "");

            part = &r->headers_in.headers.part;
            header = part->elts;
            hash = cfg->context_header_hash;
            p = cfg->context_header_lc;
            len = cfg->context_header.len;

            for (i = 0; /* void */; i++) {
                if (i >= part->nelts) {
                    if (part->next == NULL) {
                        break;
                    }

                    part = part->next;
                    header = part->elts;
                    i = 0;
                }

                if (hash == header[i].hash
                    && len == header[i].key.len
                    && ngx_strncmp(p, header[i].lowcase_key, len) == 0)
                {
                    size_t targetlen;
                    targetlen = ngx_base64_decoded_length(header[i].value.len);
                    if (targetlen >= sizeof(context_token)) {
                        token.data = ngx_pnalloc(r->pool, targetlen);
                        if (token.data == NULL) {
                            return NGX_ERROR;
                        }
                        if (NGX_OK == ngx_decode_base64(&token, &header[i].value)) {
                            ngx_memcpy(&context_token, token.data, token.len);
                            APMAttachContext(cfg->app_handle, (const unsigned char (*)[16])&context_token);
                        }
                        break;
                    }
                }
            }
            ph = APMParamInit();
            apm_add_header_variable(r, r->headers_in.content_length, ph);
            apm_add_header_variable(r, r->headers_in.content_type, ph);
            APMParamAddString(ph, apm_pcstrdup(r->pool, &r->connection->addr_text));
            APMParamAddString(ph, apm_pcstrdup(r->pool, &r->connection->addr_text));
            APMParamAddString(ph, apm_pcstrdup(r->pool, &r->args));
            APMParamAddString(ph, apm_pcstrdup(r->pool, &r->method_name));
            APMEventParam(cfg->module_handle, 600, APM_EVENT_LEVEL_LOG, APM_FLAG_ENTER, ph);
            APMParamFree(ph);

            if (r->headers_in.cookies.nelts) {
                ngx_table_elt_t **pcookie;
                ngx_table_elt_t *cookie;
                pcookie = r->headers_in.cookies.elts;
                for (j = 0; j < r->headers_in.cookies.nelts; j++) {
                    cookie = pcookie[j];
                    ph = APMParamInit();
                    APMParamAddString(ph, apm_pcstrdup(r->pool, &cookie->key));
                    APMParamAddString(ph, apm_pcstrdup(r->pool, &cookie->value));
                    APMEventParam(cfg->module_handle, 607, APM_EVENT_LEVEL_DETAIL, APM_FLAG_NONE, ph);
                    APMParamFree(ph);
                }
            }

            if (r->headers_in.headers.last && APMGetContext(&context_token)) {
                if (i < part->nelts) {
                    context_header = &header[i];
                }
                else {
                    context_header = ngx_list_push(&r->headers_in.headers);
                    if (context_header) {
                        context_header->hash = hash;
                        context_header->key.len = len;
                        context_header->key.data = cfg->context_header.data;
                        context_header->lowcase_key = p;
                    }
                }
                if (context_header) {
                    token.data = (u_char *)&context_token;
                    token.len = sizeof(APMContext);
                    ngx_http_apm_set_header_base64(r, context_header, &token);
                }
            }
        }

        APMEventStr(cfg->module_handle, 608, APM_EVENT_LEVEL_LOG, APM_FLAG_NONE, apm_pcstrdup(r->pool, &r->uri));
        ngx_http_apm_detach_thread(r, cfg);
    }
    return NGX_DECLINED;
}

static ngx_int_t
ngx_http_apm_log_handler(ngx_http_request_t *r)
{
    ngx_http_apm_conf_t           *cfg;
    APMParamHandle                 ph;
    ssize_t                        length;

    cfg = ngx_http_get_module_loc_conf(r, ngx_http_apm_module);
    if (cfg == NULL) {
        return NGX_OK;
    }

    if (cfg->enabled) {
        if (NGX_OK == ngx_http_apm_attach_thread(r, cfg)) {
            if (r->headers_in.authorization || r->headers_in.user.len) {
                ngx_str_t authorization;
                size_t len;

                ngx_str_null(&authorization);
                if (r->headers_in.authorization) {
                    len = r->headers_in.authorization->value.len;
                    authorization.data = r->headers_in.authorization->value.data;
                    authorization.len = 0;
                    while (authorization.len < len && authorization.data[authorization.len] != ' ') {
                        authorization.len++;
                    }
                }
                ph = APMParamInit();
                APMParamAddString(ph, apm_pcstrdup(r->pool, &r->headers_in.user));
                APMParamAddString(ph, apm_pcstrdup(r->pool, &authorization));
                APMEventParam(cfg->module_handle, 603, APM_EVENT_LEVEL_LOG, APM_FLAG_NONE, ph);
                APMParamFree(ph);
            }

            if (r->headers_out.content_length_n >= 0) {
                ph = APMParamInit();
                APMParamAdd64(ph, r->headers_out.content_length_n);
                if (r->headers_out.content_type.len) {
                    APMParamAddString(ph, apm_pcstrdup(r->pool, &r->headers_out.content_type));
                }
                APMEventParam(cfg->module_handle, 606, APM_EVENT_LEVEL_LOG, APM_FLAG_NONE, ph);
                APMParamFree(ph);
            }

            length = 0;
            ngx_http_apm_get_variable(r, cfg->content_length_index, &length);

            ph = APMParamInit();
            APMParamAdd32(ph, r->headers_out.status);
            if (length) {
              APMParamAdd64(ph, (UInt64)length);
            }
            APMEventParam(cfg->module_handle, 601, APM_EVENT_LEVEL_LOG, APM_FLAG_LEAVE, ph);
            APMParamFree(ph);

            APMReleaseContext();
        }
    }
    return NGX_DECLINED;
}

static ngx_int_t
ngx_http_apm_header_filter(ngx_http_request_t *r)
{
    ngx_http_apm_conf_t           *cfg;
    APMParamHandle                 ph;

    cfg = ngx_http_get_module_loc_conf(r, ngx_http_apm_module);
    if (cfg != NULL) {
        ngx_http_apm_attach_thread(r, cfg);

        ngx_list_part_t             *part;
        ngx_table_elt_t             *header, *syncmark_header;
        ngx_uint_t                   i, hash;
        u_char                      *p;
        size_t                       len;
        APMSyncMark                  syncmark_token;
        ngx_str_t                    token;
        ngx_http_variable_value_t   *var;

        ph = APMParamInit();
        var = ngx_http_get_indexed_variable(r, cfg->proxy_host_index);
        if (var != NULL && var->data && var->valid) {
            ngx_str_t value;
            value.data = var->data;
            value.len = var->len;
            APMParamAddString(ph, apm_pcstrdup(r->pool, &value));
        }
        var = ngx_http_get_indexed_variable(r, cfg->proxy_port_index);
        if (var != NULL && var->data && var->valid) {
            ngx_str_t value;
            value.data = var->data;
            value.len = var->len;
            APMParamAddString(ph, apm_pcstrdup(r->pool, &value));
        }
        ngx_http_upstream_t *upstream = r->upstream;
        if (upstream && upstream->peer.name) {
            APMParamAddString(ph, apm_pcstrdup(r->pool, upstream->peer.name));
        }
        APMEventParam(cfg->module_handle, 610, APM_EVENT_LEVEL_LOG, APM_FLAG_NONE, ph);
        APMParamFree(ph);

        part = &r->headers_out.headers.part;
        header = part->elts;
        hash = cfg->syncmark_header_hash;
        p = cfg->syncmark_header_lc;
        len = cfg->syncmark_header.len;

        for (i = 0; /* void */; i++) {
          if (i >= part->nelts) {
             if (part->next == NULL) {
                 break;
             }

             part = part->next;
             header = part->elts;
             i = 0;
          }

#if defined(DEBUG)
          ph = APMParamInit();
          APMParamAddString(ph, apm_pcstrdup(r->pool, &header[i].key));
          APMParamAddString(ph, apm_pcstrdup(r->pool, &header[i].value));
          APMEventParam(cfg->module_handle, 999, APM_EVENT_LEVEL_DEBUG, APM_FLAG_NONE, ph);
          APMParamFree(ph);
#endif
          if (len == header[i].key.len
              && ngx_strncmp(p, header[i].lowcase_key, len) == 0)
          {
             size_t targetlen;
             targetlen = ngx_base64_decoded_length(header[i].value.len);
             if (targetlen >= sizeof(syncmark_token)) {
                 token.data = ngx_pnalloc(r->pool, targetlen);
                 if (token.data == NULL) {
                     return NGX_ERROR;
                 }
                 if (NGX_OK == ngx_decode_base64(&token, &header[i].value)) {
                     ngx_memcpy(&syncmark_token, token.data, token.len);
                     APMSetSyncMark((const unsigned char (*)[16])&syncmark_token);
                 }
                 break;
             }
          }
        }
        if (r->headers_out.headers.last && APMGetSyncMark(&syncmark_token)) {
            if (i < part->nelts) {
                syncmark_header = &header[i];
            }
            else {
                syncmark_header = ngx_list_push(&r->headers_out.headers);
                if (syncmark_header) {
                    syncmark_header->hash = hash;
                    syncmark_header->key.len = len;
                    syncmark_header->key.data = cfg->syncmark_header.data;
                    syncmark_header->lowcase_key = p;
                }
            }
            if (syncmark_header) {
                token.data = (u_char *)&syncmark_token;
                token.len = sizeof(APMContext);
                ngx_http_apm_set_header_base64(r, syncmark_header, &token);
            }
        }
        ngx_http_apm_detach_thread(r, cfg);
    }
    return ngx_http_next_header_filter(r);
}

static ngx_int_t
ngx_http_apm_body_filter(ngx_http_request_t *r, ngx_chain_t *in)
{
    ngx_http_apm_conf_t           *cfg;
    ssize_t                        length;
    ngx_chain_t                   *chain_link;

    cfg = ngx_http_get_module_loc_conf(r, ngx_http_apm_module);

    if (cfg != NULL) {
        length = 0;
        ngx_http_apm_get_variable(r, cfg->content_length_index, &length);

        for (chain_link = in; chain_link; chain_link = chain_link->next) {
            length += (chain_link->buf->last - chain_link->buf->pos);
        }
        ngx_http_apm_set_variable(r, cfg->content_length_index, length);
    }

    return ngx_http_next_body_filter(r, in);
}

static void *
ngx_http_apm_create_loc_conf(ngx_conf_t *cf)
{
    ngx_http_apm_conf_t  *conf;

    conf = ngx_pcalloc(cf->pool, sizeof(ngx_http_apm_conf_t));
    if (conf == NULL) {
        return NULL;
    }

    conf->enabled = NGX_CONF_UNSET_UINT;
    ngx_str_null(&conf->app_name);
    ngx_str_null(&conf->app_id);
    ngx_str_null(&conf->app_tier_name);
    ngx_str_null(&conf->app_tier_id);

    ngx_str_null(&conf->context_header);
    ngx_str_null(&conf->syncmark_header);

    conf->app_handle = 0;
    conf->module_handle = 0;

    return conf;
}


static ngx_int_t
ngx_http_apm_invalid_variable_get(ngx_http_request_t *r,
    ngx_http_variable_value_t *v, uintptr_t data)
{
  v->valid = 0;
  v->not_found = 1;
  return NGX_OK;
}

static char *
ngx_http_apm_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child)
{
    ngx_http_apm_conf_t  *prev = parent;
    ngx_http_apm_conf_t  *conf = child;
    ngx_http_variable_t  *var = NULL;
    ngx_str_t apm_thread_handle = ngx_string("apm_thread_handle");
    ngx_str_t apm_content_length = ngx_string("apm_content_length");
    ngx_str_t proxy_host = ngx_string("proxy_host");
    ngx_str_t proxy_port = ngx_string("proxy_port");

    ngx_conf_merge_value(conf->enabled, prev->enabled, 1);

    ngx_conf_merge_str_value(conf->app_name     , prev->app_name     , "Fabasoft app.telemetry");
    ngx_conf_merge_str_value(conf->app_id       , prev->app_id       , "");
    ngx_conf_merge_str_value(conf->app_tier_name, prev->app_tier_name, "nginx Module");
    ngx_conf_merge_str_value(conf->app_tier_id  , prev->app_tier_id  , "");

    ngx_conf_merge_str_value(conf->context_header , prev->context_header, "x-apm-telemetry-context");
    conf->context_header_lc = ngx_pcalloc(cf->pool, conf->context_header.len + 1);
    if (conf->context_header_lc) {
      conf->context_header_hash = ngx_hash_strlow(conf->context_header_lc, conf->context_header.data, conf->context_header.len);
    }

    ngx_conf_merge_str_value(conf->syncmark_header, prev->syncmark_header, "x-apm-telemetry-syncmark");
    conf->syncmark_header_lc = ngx_pcalloc(cf->pool, conf->syncmark_header.len + 1);
    if (conf->syncmark_header_lc) {
      conf->syncmark_header_hash = ngx_hash_strlow(conf->syncmark_header_lc, conf->syncmark_header.data, conf->syncmark_header.len);
    }

    var = ngx_http_add_variable(cf, &apm_thread_handle, NGX_HTTP_VAR_CHANGEABLE);
    if (var == NULL) {
      return NGX_CONF_ERROR;
    }

    var->get_handler = ngx_http_apm_invalid_variable_get;

    var = ngx_http_add_variable(cf, &apm_content_length, NGX_HTTP_VAR_CHANGEABLE);
    if (var == NULL) {
      return NGX_CONF_ERROR;
    }

    var->get_handler = ngx_http_apm_invalid_variable_get;

    conf->thread_handle_index = ngx_http_get_variable_index(cf, &apm_thread_handle);
    conf->content_length_index = ngx_http_get_variable_index(cf, &apm_content_length);
    conf->proxy_host_index = ngx_http_get_variable_index(cf, &proxy_host);
    conf->proxy_port_index = ngx_http_get_variable_index(cf, &proxy_port);

    return NGX_CONF_OK;
}

static ngx_int_t
ngx_http_apm_get_variable(ngx_http_request_t *r, ngx_uint_t index, ssize_t *value)
{
    ngx_http_variable_value_t *var;
    var = ngx_http_get_indexed_variable(r, index);
    if (var != NULL && var->data && var->valid) {
        *value = ngx_atosz(var->data, var->len);
        var->valid = 0;
        return NGX_OK;
    }
    return NGX_ERROR;
}

static ngx_int_t
ngx_http_apm_set_variable(ngx_http_request_t *r, ngx_uint_t index, ssize_t value)
{
    ngx_http_variable_value_t *var;
    var = ngx_http_get_indexed_variable(r, index);
    if (var == NULL) {
        return NGX_ERROR;
    }
    u_char *p;
    if (var->data) {
        p = var->data;
    }
    else {
        p = ngx_pnalloc(r->pool, NGX_INT64_LEN);
    }
    if (p == NULL) {
        return NGX_ERROR;
    }

    var->len = ngx_sprintf(p, "%z", value) - p;
    var->valid = 1;
    var->no_cacheable = 0;
    var->not_found = 0;
    var->data = p;

    return NGX_OK;
}

static ngx_int_t 
ngx_http_apm_filter_init(ngx_conf_t *cf)
{
    ngx_http_core_main_conf_t  *cmcf;
    ngx_http_handler_pt        *h;

    cmcf = ngx_http_conf_get_module_main_conf(cf, ngx_http_core_module);

    h = ngx_array_push(&cmcf->phases[NGX_HTTP_REWRITE_PHASE].handlers);
    if (h == NULL) {
        return NGX_ERROR;
    }
    *h = ngx_http_apm_init_handler;

    h = ngx_array_push(&cmcf->phases[NGX_HTTP_LOG_PHASE].handlers);
    if (h == NULL) {
        return NGX_ERROR;
    }
    *h = ngx_http_apm_log_handler;

    ngx_http_next_header_filter = ngx_http_top_header_filter;
    ngx_http_top_header_filter = ngx_http_apm_header_filter;

    ngx_http_next_body_filter = ngx_http_top_body_filter;
    ngx_http_top_body_filter = ngx_http_apm_body_filter;

    return NGX_OK;
}
