/*
 * Copyright (c) Fabasoft R&D GmbH, A-4020 Linz, 1995-2015.
 *
 * All rights reserved. All hardware and software names used are registered
 * trade names and/or registered trademarks of the respective manufacturers.
 *
 * The user of this computer program acknowledges that the above copyright
 * notice, which constitutes the Universal Copyright Convention, will be
 * attached at the position in the function of the computer program which the
 * author has deemed to sufficiently express the reservation of copyright.
 * It is prohibited for customers, users and/or third parties to remove,
 * modify or move this copyright notice.
 */
/*jslint es5: true, es3: false */
'use strict';

var Internal;

//
//public API
//

/**
 * Use this function do declare a minimal Fabasoft app.telemetry product version
 * that is capable of executing your modules scripts. If later Fabasoft app.telemetry
 * releases add new classes, attributes, ... any such element will be marked with the
 * first version number that supports it. In case your script uses anything that has a
 * version number in it's comments a call to the API function with the documented values
 * is needed to ensure proper function of module upgrades.
 * @param Integer major minimum required Fabasoft app.telemetry major version number.
 * @param Integer minor minimum required Fabasoft app.telemetry minor version number.
 */
function API(major, minor) {
  return Internal.API(major, minor);
}

/**
 * Initialize the Infrastructure API for the given module.
 * @param {String} moduleId Id of this module can only contain the characters: ```A-Za-z0-9_.+: -```
 */
function config_module(moduleId) {
  return Internal.config_module(moduleId);
}

/**
 * The upgrade transaction callback receives an instance of the Config class with which it can
 * perform modify the infrastructure.
 * @callback upgradeTransactionCallback
 * @param {Config} cfg an instance of the Config class with selectors for infrastructure modification.
 */


/**
 * @param {Integer} version the version number that this transaction updates to.
 * @param {upgradeTransactionCallback} transactionFunction the closure that performs the upgrade,
 *   after it completes the changes will be committed. If this function fails
 *   (throws an exception) none of the changes will be committed and no further
 *   upgrades will be performed.
 */
function upgrade(version, transactionFunction) {
  return Internal.upgrade(version, transactionFunction);
}

/**
 * @param {Object} target the target object that will receive all enumerable own properties of all the subsequent parameters.
 * @param {...Object} obj an object whose enumerable own properties shall be applied onto the target object.
 * @returns {Object} returns the modified target argument.
 */
function extend(target, obj) {
  return Internal.extend.apply(Internal, arguments);
}

/**
 * Log all given arguments to the execution log for debugging purposes.
 * @param {...*} argument all arguments will be converted to a string and then logged adding a single space in between every argument.
 * @function log
 */


/**
 * Infrastructure element Status.
 * @enum
 */
var ElementStatus = {
  /**
   * Element is active.
   * @type Integer
   */
  Active: 0,
  /**
   * Element is inactive.
   * @type Integer
   */
  Inactive: 2,
  /**
   * Element marked for future deletion.
   * @type Integer
   */
  Deleted: 99,
};


/**
 * @class
 * @classdesc Base class of all infrastructure objects.
 * @mixin
 */
function CISElement() { throw new Error("Direct instantiation not allowed"); }
/**
 * The ID of this object.
 * @readonly
 * @type Integer
 */
CISElement.prototype.id = 0;
/**
 * The name of this object.
 * @type String
 */
CISElement.prototype.name = "";
/**
 * The object status.
 * @type ElementStatus
 * @default ElementStatus.Active
 */
CISElement.prototype.status = ElementStatus.Active;
/**
 * The reference of this object. May only contain the characters ```A-Za-z0-9_.+: -```.
 * @default ""
 * @type String
 */
CISElement.prototype.reference = "";
/**
 * A place to store arbitrary string properties. In some cases objects store their configuration inside this hash/dictionary.
 * If you store custom properties make sure you prefix them with your moduleId.
 * @see upgrade_module
 * @type {Object.<string,string>}
 */
CISElement.prototype.properties = {};


/**
 * @class
 * @classdesc The Service class.
 * @extends CISElement
 */
function Service() { throw new Error("Direct instantiation not allowed"); }
/**
 * ID of the agent this service is running on.
 * @type Integer
 * @see Agent#id
 */
Service.prototype.agent = 0;
/**
 * Application Name that this service represents.
 * @type String
 */
Service.prototype.appName = "";
/**
 * Application ID that this service represents.
 * @type String
 */
Service.prototype.appId = "";
/**
 * Application Tier Name that this service represents.
 * @type String
 */
Service.prototype.appTierName = "";
/**
 * Application Tier ID that this service represents.
 * @type String
 */
Service.prototype.appTierId = "";
/**
 * Resource Identifier - needed to identify a service from a cluster group so that a failover does not trigger a creation of a new service object.
 * @type String
 */
Service.prototype.resourceIdentifier = "";


/**
 * Status propagation mode.
 * @enum
 */
var StatusPropagation = {
  /**
   * Propagate the worst child status.
   * @type Integer
   */
  Worst: 0,
  /**
   * Propagate best child status.
   * @type Integer
   */
  Best: 1000,
};


/**
 * @class
 * @classdesc The ServiceGroup class.
 * @extends CISElement
 */
function ServiceGroup() { throw new Error("Direct instantiation not allowed"); }
/**
 * Display the agent name for services in this group.
 * @type Integer
 * @default 1
 */
ServiceGroup.prototype.showAgentName = 1;
/**
 * Application Name that this service represents.
 * @type String
 */
ServiceGroup.prototype.appName = "";
/**
 * Application ID that this service represents.
 * @type String
 */
ServiceGroup.prototype.appId = "";
/**
 * Application Tier Name that this service represents.
 * @type String
 */
ServiceGroup.prototype.appTierName = "";
/**
 * The propagation mode for the status of the children.
 * @type StatusPropagation
 */
ServiceGroup.prototype.propagate = StatusPropagation.Worst;
/**
 * IDs of services directly below this service group.
 * @type {Array.<integer>}
 * @see Service#id
 */
ServiceGroup.prototype.services = [];
/**
 * IDs of services directly below this service group.
 * @type {Array.<integer>}
 * @see CISElement#id
 */
ServiceGroup.prototype.subgroups = [];


/**
 * @enum
 */
var VmServerType = {
  /**
   * No detection for virtual machines.
   * @type Integer
   */
  None: 0,
  /**
   * Virtual machine detection for VMWare agents.
   * @type Integer
   */
  VmWare: 1,
  /**
   * Virtual machine detection for Microsoft HyperV agents.
   * @type Integer
   */
  HyperV: 3,
};


/**
 * @enum
 */
var SnmpVersion = {
  /**
   * SNMP Version 1.
   * @type String
   */
  Version_1: '1',
  /**
   * SNMP Version 2c.
   * @type String
   */
  Version_2c: '2c',
  /**
   * SNMP Version 3.
   * @type String
   */
  Version_3: '3',
};


/**
 * @enum
 */
var AgentProperty = {
  /**
   * SNMP-Community.
   * @type String
   */
  SnmpCommunity: 'snmpcommunity',
  /**
   * SNMP-Version: {@link SnmpVersion}.
   * @type String
   */
  SnmpVersion: 'snmpversion',
};


/**
 * @class
 * @classdesc The Agent class.
 * @extends CISElement
 * @see AgentProperty
 */
function Agent() { throw new Error("Direct instantiation not allowed"); }
/**
 * The name or address of this agent.
 * @type String
 */
Agent.prototype.networkAddress = "";
/**
 * The port used to connect to this agent.
 * @type Integer
 * @default 10001
 */
Agent.prototype.agentPort = 10001;
/**
 * The ID of the proxy agent to use, or 0.
 * @type Integer
 * @see Agent#id
 * @default 0
 */
Agent.prototype.proxyAgent = 0;
/**
 * Type of VM Server this agent represents.
 * @type {VmServerType}
 */
Agent.prototype.vmServerType = VmServerType.None;
/**
 * Cluster Server allowed to use this agent.
 * @type Integer
 * @default 0
 * @see Server#serverId
 */
Agent.prototype.serverId = 0;
/**
 * Detected agent host name.
 * @readonly
 * @type String
 */
Agent.prototype.hostName = "";
/**
 * Detected operating system.
 * @readonly
 * @type String
 */
Agent.prototype.osName = "";
/**
 * Detected amount of CPU cores.
 * @readonly
 * @type Integer
 */
Agent.prototype.cpuCores = 0;
/**
 * 1 if this agent represents a cluster resource, 0 otherwise.
 * @readonly
 * @type Integer
 */
Agent.prototype.isVirtualServer = 0;
/**
 * List of protocols supported by this agent.
 * @type {Array.<string>}
 * @readonly
 * @see ServiceCheckProtocol
 */
Agent.prototype.protocols = [];
/**
 * List of MAC addresses of this agent.
 * @type {Array.<string>}
 * @readonly
 */
Agent.prototype.physicalNetworkAddresses = [];
/**
 * List of cluster network addresses.
 * @type {Array.<string>}
 * @readonly
 */
Agent.prototype.networkAddresses = [];
/**
 * List of cluster network names.
 * @type {Array.<string>}
 * @readonly
 */
Agent.prototype.networkNames = [];
/**
 * List of cluster resources.
 * @type {Array.<string>}
 * @readonly
 */
Agent.prototype.resources = [];


/**
 * @class
 * @classdesc The Server class.
 * @extends CISElement
 */
function Server() { throw new Error("Direct instantiation not allowed"); }
/**
 * The NodeId of this server node. Valid values: 0-255.
 * It is allowed to define multiple servers with a NodeId of 0
 * but no more than one server may have the same non 0 NodeId.
 * Servers with NodeId 0 are only used to perform distributed request resolution
 * whereas servers with NodeId > 0 participate in clustering.
 * @type Integer
 * @default 0
 */
Server.prototype.serverId = 0;
/**
 * Port of the remote Server service.
 * @default 10000
 * @type Integer
 */
Server.prototype.serverPort = 10000;
/**
 * Port of the remote Worker service.
 * @default 10004
 * @type Integer
 */
Server.prototype.workerPort = 10004;
/**
 * Address of the remote server node.
 * @type String
 */
Server.prototype.networkAddress = "";
/**
 * Infrastructure ID mappings for the remote server node. This is only relevant if the {@link Server#serverId} is not 0.
 * Used to allow different database connections to be used by the server nodes.
 * @type {Object.<string,string>}
 * @see CISElement#id
 */
Server.prototype.mappings = {};


/**
 * @class
 * @classdesc The AgentGroup class.
 * @extends CISElement
 */
function AgentGroup() { throw new Error("Direct instantiation not allowed"); }
/**
 * IDs of agents directly below this agent group.
 * @type {Array.<integer>}
 * @see Agent#id
 */
AgentGroup.prototype.agents = [];
/**
 * IDs of agent groups directly below this agent group.
 * @type {Array.<integer>}
 * @see AgentGroup#id
 */
AgentGroup.prototype.subgroups = [];


/**
 * @enum
 */
var ServiceCheckProtocol = {
  /**
   * Microsoft Windows performance counter check.
   * @type String
   */
  winperf: "winperf",
  /**
   * SNMP counter check.
   * @type String
   */
  snmp: "snmp",
  /**
   * Microsoft Windows service availability check.
   * @type String
   */
  winsvc: "winsvc",
  /**
   * Microsoft Windows Cluster resource availability check.
   * @type String
   */
  wincluster: "wincluster",
  /**
   * Red Hat cluster suite service availability check.
   * @type String
   */
  rhelcluster: "rhelcluster",
  /**
   * Linux process availability check.
   * @type String
   */
  linuxsvc: "linuxsvc",
  /**
   * Counter check using existing counter definition.
   * @type String
   */
  counter: "counter",
  /**
   * Counter check using formula.
   * @type String
   */
  formula: "formula",
  /**
   * Linux /proc counter check.
   * @type String
   */
  procfs: "procfs",
  /**
   * Web service availability check.
   * @type String
   */
  httpcheck: "httpcheck",
  /**
   * TCP Ping availability check.
   * @type String
   */
  tcpping: "tcpping",
  /**
   * Counter check using file system.
   * @type String
   */
  file: "file",
  /**
   * Log pool statistics counter check.
   * @type String
   */
  logpool: "logpool",
  /**
   * ICMP Ping availability check.
   * @type String
   */
  icmpping: "icmpping",
  /**
   * Server Statistics check.
   * @type String
   */
  serverstat: "serverstat",
};


/**
 * @enum
 */
var ServiceCheckProperty = {
  /**
   * The protocol required to interpret the counter. See {@link ServiceCheckProtocol}.
   * @type String
   */
  protocol: "protocol",
  /**
   * The counter definition.
   * @type String
   */
  counter: "counter",
  /**
   * The warning limit.
   * @type String
   */
  warning: "warning",
  /**
   * The error limit.
   * @type String
   */
  error: "error",
};


/**
 * @class
 * @classdesc The ServiceCheck class.
 * @extends CISElement
 * @see ServiceCheckProperty
 */
function ServiceCheck() { throw new Error("Direct instantiation not allowed"); }
/**
 * ID of the service that this service check is assigned to.
 * @see Service#id
 * @type Integer
 */
ServiceCheck.prototype.service = 0;
/**
 * Seconds between executions of this check.
 * @type Integer
 */
ServiceCheck.prototype.interval = 0;
/**
 * ID of the default service level definition where availability data must be persisted.
 * @see SLADefinition#id
 * @type Integer
 */
ServiceCheck.prototype.sla = 0;
/**
 * Number of {@link ServiceCheck#interval} before the status becomes marked as too old.
 * @type Integer
 */
ServiceCheck.prototype.timeoutInterval = 0;
/**
 * Number of {@link ServiceCheck#interval} before before a failed check will be reported as error.
 * Note: Does not apply to counter checks.
 * @type Integer
 */
ServiceCheck.prototype.toleranceInterval = 0;
/**
 * ID of the database connection where counter values will be persisted.
 * @see DatabaseConnectionOLEDB#id
 * @see DatabaseConnectionPostgreSQL#id
 * @type Integer
 */
ServiceCheck.prototype.databaseWriter = 0;


/**
 * @class
 * @classdesc The Counter class.
 * @extends CISElement
 */
function Counter() { throw new Error("Direct instantiation not allowed"); }
/**
 * List of formulas that can produce the result.
 * The formula to be used is determined by matching the counter protocols provided by the agent.
 * @type {Array.<string>}
 */
Counter.prototype.calculation = [];


/**
 * @enum
 */
var NotificationChannelType = {
  /**
   * SMTP (e-mail) channel.
   * @type String
   */
  smtp: "smtp",
  /**
   * Command line channel.
   * @type String
   */
  command: "command",
};


/**
 * @enum
 */
var NotificationChannelAuthenticationType = {
  /**
   * No authentication.
   * @type String
   */
  Anonymous: "anonymous",
  /**
   * Authentication with user name and password. Requires the additional properties NotificationChannelProperty.SendUserName and NotificationChannelProperty.SendPassword.
   * @type String
   */
  Basic: "basic",
  /**
   * Integrated Windows Authentication.
   * @type String
   */
  NTLM: "ntlm",
};


/**
 * @enum
 */
var NotificationChannelProperty = {
  /**
   * Command string (only for NotificationChannelType.command).
   * @type String
   */
  command: "command",
  /**
   * Server name (only on Microsoft Windows with NotificationChannelType.smtp).
   * @type String
   */
  Server: "Server",
  /**
   * Server port (only on Microsoft Windows with NotificationChannelType.smtp).
   * @type String
   */
  ServerPort: "ServerPort",
  /**
   * Account name (only on Microsoft Windows with NotificationChannelType.smtp).
   * @type String
   */
  AccountName: "AccountName",
  /**
   * Use SSL (only on Microsoft Windows with NotificationChannelType.smtp).
   * @type String
   */
  UseSSL: "UseSSL",
  /**
   * Server name (only on Microsoft Windows with NotificationChannelType.smtp). See {@link NotificationChannelAuthenticationType} for valid values.
   * @type String
   */
  Authenticate: "Authenticate",
  /**
   * User name of the account used to send with "basic" authentication (only on Microsoft Windows with NotificationChannelType.smtp).
   * @type String
   */
  SendUserName: "SendUserName",
  /**
   * Password of the account used to send with "basic" authentication (only on Microsoft Windows with NotificationChannelType.smtp).
   * @type String
   */
  SendPassword: "SendPassword",
  /**
   * Sender email address (only for NotificationChannelType.smtp)."
   * @type String
   */
  SendEmailAddress: "SendEmailAddress",
  /**
   * Timeout for sending a mail (only on Microsoft Windows with NotificationChannelType.smtp)."
   * @type String
   */
  Timeout: "Timeout",
};


/**
 * @class
 * @classdesc The NotificationChannel class.
 * @extends CISElement
 * @see NotificationChannelProperty
 */
function NotificationChannel() { throw new Error("Direct instantiation not allowed"); }
/**
 * The type of notification channel, set appropriate properties from {@link NotificationChannelProperty} to configure this channel.
 * @type NotificationChannelType
 */
NotificationChannel.prototype.type = NotificationChannelType.smtp;
/**
 * Interval in seconds in which this notification channel checks for status changes to send notifications.
 * @default 10
 * @type Integer
 */
NotificationChannel.prototype.scheduleSeconds = 10;
/**
 * Delay in seconds after a start of the Server service before this notification channel starts monitoring for status changes.
 * @default 60
 * @type Integer
 */
NotificationChannel.prototype.delayOnStartSeconds = 60;


/**
 * @enum
 */
var NotificationSendIf = {
  /**
   * Trigger a notification if the status changes to Ok.
   * @type Integer
   */
  Ok: 1,
  /**
   * Trigger a notification if the status changes to Warning.
   * @type Integer
   */
  Warning: 2,
  /**
   * Trigger a notification if the status changes to Critical.
   * @type Integer
   */
  Critical: 4,
};


/**
 * @class
 * @classdesc The NotificationAccount class.
 * @extends CISElement
 */
function NotificationAccount() { throw new Error("Direct instantiation not allowed"); }
/**
 * ID of the notification channel used to deliver notifications to this account.
 * @see NotificationChannel#id
 * @type Integer
 */
NotificationAccount.prototype.channel = 0;
/**
 * Address to deliver the notification to (for example an E-Mail address).
 * @type String
 */
NotificationAccount.prototype.address = "";
/**
 * Trigger a notification if the status changes to any of the enabled states.
 * @see NotificationSendIf
 * @default NotificationSendIf.Ok | NotificationSendIf.Warning | NotificationSendIf.Critical
 * @type Integer
 */
NotificationAccount.prototype.sendflags = NotificationSendIf.Ok | NotificationSendIf.Warning | NotificationSendIf.Critical;
/**
 * IDs of checks, services or service groups that will generate a notification.
 * @type {Array.<integer>}
 * @see ServiceCheck#id
 * @see Service#id
 * @see ServiceGroup#id
 */
NotificationAccount.prototype.checks = [];
/**
 * IDs of log pools that will trigger a notification for new feedbacks.
 * @type {Array.<integer>}
 * @see APMLogAnalyzer#id
 * @see FormLogAnalyzer#id
 */
NotificationAccount.prototype.logdefinitions = [];


/**
 * @enum
 */
var LogAnalyzerEntryFlags = {
  /**
   * The start of a block.
   * @type Integer
   */
  Block_Start: 0x0004,
  /**
   * The end of a block.
   * @type Integer
   */
  Block_End: 0x0008,
  /**
   * Select first occurrence.
   * @type Integer
   */
  Multiple_First: 0x0000,
  /**
   * Persist all occurrences - requires {@link LogAnalyzerEntry#tableType} to have value LogAnalyzerEntryTableType.Extended.
   * @type Integer
   * @see LogAnalyzerEntryTableType
   */
  Multiple_All: 0x0010,
  /**
   * Persist last occurrence of this event (overwriting previous values).
   * @type Integer
   */
  Multiple_Last: 0x0020,
  /**
   * Count the number of occurrences of the event.
   * @type Integer
   */
  Multiple_Count: 0x0030,
  /**
   * Store the sum of all occurrences.
   * @type Integer
   */
  Multiple_Sum: 0x0040,
  /**
   * Column is a dimension - requires a {@link LogAnalyzerEntry#type} of LogAnalyzerEntryType.Int32, LogAnalyzerEntryType.Int64 or LogAnalyzerEntryType.String.
   * @type Integer
   * @see LogAnalyzerEntryType
   */
  Dimension: 0x0100,
  /**
   * Column is a measure - requires a {@link LogAnalyzerEntry#type} of LogAnalyzerEntryType.Int32, LogAnalyzerEntryType.Int64 or LogAnalyzerEntryType.Duration.
   * @type Integer
   * @see LogAnalyzerEntryType
   */
  Measure: 0x200,
  /**
   * This measure column is null-able, so a count column of not null values is calculated. Only valid for columns with the LogAnalyzerEntryFlags.Measure flag set.
   * @type Integer
   */
  NullableMeasure: 0x0400,
  /**
   * Calculate the minimum. Only valid for columns with the LogAnalyzerEntryFlags.Measure flag set.
   * @type Integer
   */
  With_Minimum: 0x0800,
  /**
   * Calculate the maximum. Only valid for columns with the LogAnalyzerEntryFlags.Measure flag set.
   * @type Integer
   */
  With_Maximum: 0x1000,
  /**
   * The {@link LogAnalyzerEntry#calculation} defines ranges for the value from the column referenced by the {@link LogAnalyzerEntry#parent} attribute.
   * The {@link LogAnalyzerEntry#format} attribute of the column defines a textual representation of the result.
   * @type Integer
   */
  Evaluate_Ranges: 0x10000,
  /**
   * The {@link LogAnalyzerEntry#calculation} defines a regular expression to apply to the value from the column referenced by the {@link LogAnalyzerEntry#parent} attribute.
   * The {@link APMLogAnalyzerEntry#paramId} attribute of the column defines which capture in the regular expression to use as the value.
   * @type Integer
   */
  Evaluate_RegExp: 0x20000,
};


/**
 * @enum
 */
var LogAnalyzerEntryType = {
  /**
   * Block start event type (has no data type).
   * @type Integer
   */
  BlockStart: 0,
  /**
   * 32 bit integer number.
   * @type Integer
   */
  Int32: 1,
  /**
   * 64 bit integer number.
   * @type Integer
   */
  Int64: 2,
  /**
   * Textual value – if this data-type is set, the {@link LogAnalyzerEntry#column_length} attribute must be set too.
   * @type Integer
   */
  String: 3,
  /**
   * Used for calculated duration between two events (when block is set) – unit in database = [100ns]
   * (that means value 10000 in DB = 1ms)
   * @type Integer
   */
  Duration: 4,
  /**
   * 64 bit long value used for storing date/time information.
   * @type Integer
   */
  Timestamp: 5,
  /**
   * 64 bit floating point number.
   * @type Integer
   */
  Double: 6,
};


/**
 * @enum
 */
var LogAnalyzerEntryDatatype = {
  /**
   * 32 bit integer number.
   * @type Integer
   */
  Int32: 1,
  /**
   * 64 bit integer number.
   * @type Integer
   */
  Int64: 2,
  /**
   * Textual value – if this data-type is set, the {@link LogAnalyzerEntry#column_length} attribute must be set too.
   * @type Integer
   */
  String: 3,
  /**
   * Used for calculated duration between two events (when block is set) – unit in database = 100ns
   * (that means value 10000 in DB = 1ms).
   * @type Integer
   */
  Duration: 4,
  /**
   * 64 bit long value used for storing date/time information.
   * @type Integer
   */
  Timestamp: 5,
  /**
   * 64 bit floating point number.
   * @type Integer
   */
  Double: 6,
};


/**
 * @enum
 */
var LogAnalyzerEntryTableType = {
  /**
   * Base table where all the default columns are also stored, must only be used once for any Log Pool.
   * @type Integer
   */
  Request: 1,
  /**
   * Additional table for specific data that could occur more often than once for a request (like SQL queries).
   * @type Integer
   */
  Extended: 2,
};


/**
 * Telemetry data recording level.
 * @enum
 */
var LogAnalyzerLevel = {
  /**
   * Only minimal data required for log entries are recorded.
   * @type Integer
   */
  Log: 0,
  /**
   * Standard recording level.
   * @type Integer
   */
  Normal: 50,
  /**
   * More details.
   * @type Integer
   */
  Detail: 60,
  /**
   * Most detailed recording level (may result in a lot of data).
   * @type Integer
   */
  Debug: 70,
};


/**
 * Telemetry data recording level.
 * @enum
 */
var LogAnalyzerStatisticsStatus = {
  /**
   * No statistics.
   * @type Integer
   */
  None: 0,
  /**
   * Precalculated aggregations stored on database.
   * @type Integer
   */
  Precalculated: 1,
  /**
   * Results calculated on demand using the request records from the database.
   * @type Integer
   */
  OnDemand: 2,
};


/**
 * @class
 * @classdesc The LogAnalyzerEntry class.
 */
function LogAnalyzerEntry() { throw new Error("Direct instantiation not allowed"); }
/**
 * The name of this column.
 * @type String
 */
LogAnalyzerEntry.prototype.name = "";
/**
 * The label of this column. If a label is not provided the {@link LogAnalyzerEntry#name} will be displayed instead.
 * @type String
 */
LogAnalyzerEntry.prototype.label = "";
/**
 * Bit field of flags for this column.
 * @type LogAnalyzerEntryFlags
 */
LogAnalyzerEntry.prototype.flags = 0;
/**
 * Type of this column.
 * @type LogAnalyzerEntryType
 */
LogAnalyzerEntry.prototype.type = 0;
/**
 * Format of this column. This could be for example "fsc:reference" or "enum:1:0-1s;2:1-5s;3:5-15s;4:>15s"
 * @type String
 * @see LogAnalyzerEntryFlags
 * @example <caption>Example for LogAnalyzerEntryFlags.Evaluate_Ranges:</caption>
 * col.calculation = "10000000;50000000;150000000";
 * col.format = "enum:1:0-1s;2:1-5s;3:5-15s;4:>15s";
 * // a value of 11345000
 * // would result in the value: "1-5s"
 */
LogAnalyzerEntry.prototype.format = "";
/**
 * The name of the database table this column uses.
 * @type String
 */
LogAnalyzerEntry.prototype.tableName = "";
/**
 * Type of table this column uses. Ignored unless {@link LogAnalyzerEntry#tableName} is set.
 * @type LogAnalyzerEntryTableType
 */
LogAnalyzerEntry.prototype.tableType = 0;
/**
 * The type used to persist this column on the database. Ignored unless {@link LogAnalyzerEntry#tableName} is set.
 * @type LogAnalyzerEntryDatatype
 */
LogAnalyzerEntry.prototype.columnDatatype = 0;
/**
 * The length of this column on the database. Ignored unless {@link LogAnalyzerEntry#tableName} is set.
 * @type Integer
 */
LogAnalyzerEntry.prototype.columnLength = 0;
/**
 * The name of this column used to persist on the database. Ignored unless {@link LogAnalyzerEntry#tableName} is set.
 * @type String
 */
LogAnalyzerEntry.prototype.columnName = "";
/**
 * Width hint used to display this column in the Fabasoft app.telemetry client. Use 0 to use a default width.
 * @type Integer
 * @default 0
 */
LogAnalyzerEntry.prototype.displayWidth = 0;
/**
 * Display priority used to influence the initial ordering of columns.
 * @type Integer
 * @default 1
 */
LogAnalyzerEntry.prototype.displayPriority = 1;
/**
 * Name of the parent column.
 * @see LogAnalyzerEntry#name
 * @type String
 */
LogAnalyzerEntry.prototype.parent = "";
/**
 * A list of ranges or a PCRE compatible regular expression used to populate the column based on the content of the {@link LogAnalyzerEntry#parent} columns content.
 * Only evaluated if the {@link LogAnalyzerEntry#flags} has the LogAnalyzerEntryFlags.Evaluate_Ranges or the LogAnalyzerEntryFlags.Evaluate_RegExp flag set.
 * See {@link LogAnalyzerEntry#format} for more examples.
 * @type String
 * @example <caption>Example for LogAnalyzerEntryFlags.Evaluate_RegExp:</caption>
 * col.calculation = "[\w]+://[^/]*([^\?]+)";
 * col.paramId = "1";
 * // for a column that contains an URL like: "http://example.org/some/path/file.html?query"
 * // the result would be: "/some/path/file.html"
 */
LogAnalyzerEntry.prototype.calculation = "";


/**
 * @class
 * @classdesc The APMLogAnalyzerEntry class.
 * @extends LogAnalyzerEntry
 */
function APMLogAnalyzerEntry() { throw new Error("Direct instantiation not allowed"); }
/**
 * The name of the module.
 * @type String
 */
APMLogAnalyzerEntry.prototype.module = "";
/**
 * The id of the event. Make sure you treat this value as a string because 64bit values can't be represented in JavaScript.
 * @type String
 */
APMLogAnalyzerEntry.prototype.eventId = "0";
/**
 * The id of the parameter of the event (starting at 1).
 * @type Integer
 */
APMLogAnalyzerEntry.prototype.paramId = 0;
/**
 * The block name. Requires either the LogAnalyzerEntryFlags.Block_Start or the LogAnalyzerEntryFlags.Block_End flag to be set.
 * @type String
 * @see LogAnalyzerEntryFlags
 */
APMLogAnalyzerEntry.prototype.block = "";


/**
 * @class
 * @classdesc The LogAnalyzerBase class.
 * @mixin
 */
function LogAnalyzerBase() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type LogAnalyzerType
 */
LogAnalyzerBase.prototype.logPoolType = 0;
/**
 * Authorized user and group names separated by semicolons.
 * @type String
 */
LogAnalyzerBase.prototype.group = "";


/**
 * @class
 * @classdesc The LogAnalyzerWithDataMixin class.
 * @mixin
 */
function LogAnalyzerWithDataMixin() { throw new Error("Direct instantiation not allowed"); }
/**
 * Number of minutes requests are stored in memory.
 * @type Integer
 * @default 30
 */
LogAnalyzerWithDataMixin.prototype.onlineTimeout = 0;
/**
 * ID of the database connection where request data will be persisted.
 * @type Integer
 * @see DatabaseConnectionOLEDB#id
 * @see DatabaseConnectionPostgreSQL#id
 */
LogAnalyzerWithDataMixin.prototype.database = 0;
/**
 * Number of days requests are kept on database.
 * Set databaseTimeout to 0 to disable database cleanup.
 * @type Integer
 * @default 0
 */
LogAnalyzerWithDataMixin.prototype.databaseTimeout = 0;
/**
 * ID of the service desk which to send feedbacks to.
 * @type Integer
 * @see ServiceDesk#id
 */
LogAnalyzerWithDataMixin.prototype.serviceDesk = 0;
/**
 * Telemetry data recording detail level to be used for the corresponding services.
 * @type LogAnalyzerLevel
 * @default LogAnalyzerLevel.Log
 */
LogAnalyzerWithDataMixin.prototype.loglevel = LogAnalyzerLevel.Log;
/**
 * Prefix for all database tables created in the context of the LogAnalyzer.
 * Must be unique within a database.
 * @type String
 */
LogAnalyzerWithDataMixin.prototype.tablePrefix = "";
/**
 * Time resolution for default statistics in 100ns (default 10 minutes).
 * @type String
 * @default "6000000000"
 */
LogAnalyzerWithDataMixin.prototype.timeResolution = "6000000000";
/**
 * Statistics available for the LogAnalyzer.
 * @type LogAnalyzerStatisticsStatus
 * @default LogAnalyzerStatisticsStatus.Precalculated
 */
LogAnalyzerWithDataMixin.prototype.statisticsStatus = LogAnalyzerStatisticsStatus.Precalculated;
/**
 * List of LogStatistic definitions to be precalculated for this LogAnalyzer.
 * @type {Array.<Integer>}
 * @See LogStatistic#id
 */
LogAnalyzerWithDataMixin.prototype.statistics = [];
/**
 * LogAnalyzer specific column definitions.
 * @type {Array.<APMLogAnalyzerEntry>}
 */
LogAnalyzerWithDataMixin.prototype.columns = [];


/**
 * @class
 * @classdesc The LogAnalyzerFilterMixin class.
 * @mixin
 */
function LogAnalyzerFilterMixin() { throw new Error("Direct instantiation not allowed"); }
/**
 * If set, only requests from services with a matching appName are processed by the LogAnalyzer.
 * @type String
 */
LogAnalyzerFilterMixin.prototype.appName = "";
/**
 * If set, only requests from services with a matching appId are processed by the LogAnalyzer.
 * @type String
 */
LogAnalyzerFilterMixin.prototype.appId = "";
/**
 * If set, only requests from services with a matching appTierName are processed by the LogAnalyzer.
 * @type String
 */
LogAnalyzerFilterMixin.prototype.appTierName = "";
/**
 * If set, only requests from services with a matching appTierId are processed by the LogAnalyzer.
 * @type String
 */
LogAnalyzerFilterMixin.prototype.appTierId = "";


/**
 * @class
 * @mixin
 */
function LogAnalyzerFilteredSubAnalyzerMixin() { throw new Error("Direct instantiation not allowed"); }
/**
 * Requests from the base log analyzer will be filtered by this filter.
 * @type String
 */
LogAnalyzerFilteredSubAnalyzerMixin.prototype.filterString = "";
/**
 * Base LogAnalyzer ID. Required for FilterLogAnalyzer and ViewLogAnalyzer.
 * @type Integer
 * @see APMLogAnalyzer#id
 * @see WebTimingLogAnalyzer#id
 * @see FormLogAnalyzer#id
 * @see FilterLogAnalyzer#id
 * @see ViewLogAnalyzer#id
 */
LogAnalyzerFilteredSubAnalyzerMixin.prototype.baseLogPoolId = 0;


/**
 * @class
 * @classdesc The APMLogAnalyzer class.
 * @extends CISElement
 * @extends LogAnalyzerBase
 * @extends LogAnalyzerWithDataMixin
 * @extends LogAnalyzerFilterMixin
 */
function APMLogAnalyzer() { throw new Error("Direct instantiation not allowed"); }
/**
 * Compute categorization for requests with a duration longer than the threshold (in milliseconds).
 * Set the categorizationThreshold to 0 for to disable categorization.
 * @type String
 * @default "0"
 */
APMLogAnalyzer.prototype.categorizationThreshold = "0";


/**
 * @class
 * @classdesc The WebTimingLogAnalyzer class.
 * @extends CISElement
 * @extends LogAnalyzerBase
 * @extends LogAnalyzerWithDataMixin
 * @extends LogAnalyzerFilterMixin
 */
function WebTimingLogAnalyzer() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type String
 * @readonly
 * @default "Fabasoft app.telemetry"
 */
WebTimingLogAnalyzer.prototype.appName = "";
/**
 * This is the unique ID of the WebTimingLogAnalyzer used for recording web timing data.
 * Unique IDs are created in form "APMnnnnn" where nnnnn represents a 5 digit number.
 * @type String
 */
LogAnalyzerFilterMixin.prototype.appId = "";
/**
 * @type String
 * @readonly
 * @default "Web Telemetry"
 */
WebTimingLogAnalyzer.prototype.appTierName = "";


/**
 * @class
 * @classdesc The FormLogAnalyzer class.
 * @extends CISElement
 * @extends LogAnalyzerBase
 * @extends LogAnalyzerWithDataMixin
 * @extends LogAnalyzerFilterMixin
 */
function FormLogAnalyzer() { throw new Error("Direct instantiation not allowed"); }
/**
 * The form ID.
 * @type String
 */
FormLogAnalyzer.prototype.formId = "";


/**
 * A filter log analyzer receives all requests from the base log analyzer filtered by the {@link LogAnalyzerFilteredSubAnalyzerMixin#filterString}.
 * Request data may be stored on a database.
 * Columns may be added, changed or removed setting additional entries in the columns array.
 * @class
 * @extends CISElement
 * @extends LogAnalyzerBase
 * @extends LogAnalyzerFilteredSubAnalyzerMixin
 */
function FilterLogAnalyzer() { throw new Error("Direct instantiation not allowed"); }


/**
 * A ViewLogAnalyzer defines a filtered view to requests from the base log analyzer filtered by the {@link LogAnalyzerFilteredSubAnalyzerMixin#filterString}.
 * @class
 * @extends CISElement
 * @extends LogAnalyzerBase
 * @extends LogAnalyzerFilteredSubAnalyzerMixin
 */
function ViewLogAnalyzer() { throw new Error("Direct instantiation not allowed"); }


/**
 * @class
 * @classdesc The LogStatCalculation class.
 * @extends LogAnalyzerEntry
 */
function LogStatCalculation() { throw new Error("Direct instantiation not allowed"); }


/**
 * A LogStatistic defines an additional persisted statistic computed on top of all requests of a LogAnalyzer.
 * To assign a LogStatistic to a LogAnalyzer, add the ID of the LogStatistic in the statistics property of the LogAnaylzer.
 * @class
 * @classdesc The LogStatistic class.
 * @extends CISElement
 */
function LogStatistic() { throw new Error("Direct instantiation not allowed"); }
/**
 * Name extension to uniquely name database tables (databasePrefix + tableExtension + tableName).
 * @type String
 */
LogStatistic.prototype.tableExtension = "";
/**
 * Time resolution for statistics in 100ns (default 1 day).
 * @type String
 * @default "864000000000"
 */
LogStatistic.prototype.resolution = "864000000000";
/**
 * Number of minutes to wait after the end of the interval to compute values.
 * @type Integer
 * @default 10
 */
LogStatistic.prototype.schedule = "10";
/**
 * Filter to be applied to the query.
 * @type String
 */
LogStatistic.prototype.filter = "";
/**
 * Subset of the dimensions of the LogAnalyzer to be calculated for this statistics.
 * @type {Array.<string>}
 */
LogStatistic.prototype.dimensions = [];
/**
 * Additional columns to be calculated for the statistic based on the request data properties.
 * @type {Array.<LogStatCalculation>}
 */
LogStatistic.prototype.calculations = [];


/**
 * @enum
 */
var DatabaseConnectionType = {
  /**
   * PostgreSQL database.
   * @type Integer
   */
  PostgreSQL: 1,
  /**
   * Microsoft SQL Server database.
   * @type Integer
   */
  OLEDB: 2,
};


/**
 * @class
 * @classdesc The DatabaseConnectionBase class.
 * @mixin
 */
function DatabaseConnectionBase() { throw new Error("Direct instantiation not allowed"); }
/**
 * Database system type.
 * @type DatabaseConnectionType
 */
DatabaseConnectionBase.prototype.system = 0;
/**
 * Connection string.
 * PostgreSQL: If set, all other connection parameters will be ignored.
 * OLEDB: Either a provider or a connection string must be specified.
 * @type String
 */
DatabaseConnectionBase.prototype.connectionString = "";
/**
 * Database host name.
 * @type String
 * @default "localhost"
 */
DatabaseConnectionBase.prototype.host = "";
/**
 * Database name;
 * @type String
 */
DatabaseConnectionBase.prototype.database = "";
/**
 * Database user name.
 * @type String
 */
DatabaseConnectionBase.prototype.username = "";
/**
 * Database user password.
 * @type String
 */
DatabaseConnectionBase.prototype.password = "";


/**
 * @class
 * @classdesc The DatabaseConnectionOLEDB class.
 * @extends CISElement
 * @extends DatabaseConnectionBase
 */
function DatabaseConnectionOLEDB() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type DatabaseConnectionType
 * @readonly
 * @default DatabaseConnectionType.OLEDB
 */
DatabaseConnectionOLEDB.prototype.system = DatabaseConnectionType.OLEDB;
/**
 * @type String
 * @default "SQLOLEDB.1"
 */
DatabaseConnectionOLEDB.prototype.provider = "SQLOLEDB.1";


/**
 * @class
 * @classdesc The DatabaseConnectionPostgreSQL class.
 * @extends CISElement
 * @extends DatabaseConnectionBase
 */
function DatabaseConnectionPostgreSQL() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type DatabaseConnectionType
 * @readonly
 * @default DatabaseConnectionType.PostgreSQL
 */
DatabaseConnectionPostgreSQL.prototype.system = DatabaseConnectionType.PostgreSQL;
/**
 * Database port number.
 * @type Integer
 * @default 5432
 */
DatabaseConnectionPostgreSQL.prototype.port = 5432;


/**
 * @enum
 */
var ServerInfoProperty = {
  /**
   * URL to be navigated to when hitting the logout button.
   * @type String
   */
  logouturl: "logouturl",
  /**
   * Default group where all authenticated users belong to, if they are not explicitly assigned to a user group.
   * @type String
   */
  defaultgroup: "defaultgroup",
  /**
   * Default database where all FormLogAnalyzers data will be written to by default.
   * This database is used also for storing OAuth tokens and form usage statistics.
   * @type String
   */
  defaultdatabase: "defaultdatabase",
  /**
   * This ID of the NotificationChannel used to send mails in response to an incoming feedback.
   * @type String
   */
  defaultchannel: "defaultchannel",
  /**
   * URL to the app.telemetry used in links within the IT Infrastructure synchronization (e.g. https://myapmserver/apptelemetry).
   * @type String
   */
  url: "url",
  /**
   * Default time zone name. Check you Fabasoft app.telemetry installation for valid values or look at {@link https://en.wikipedia.org/wiki/List_of_tz_database_time_zones}
   * @type String
   */
  timezone: "timezone",
};


/**
 * @class
 * @classdesc The ServerInfo class.
 * @extends CISElement
 * @see ServerInfoProperty
 */
function ServerInfo() { throw new Error("Direct instantiation not allowed"); }
/**
 * Operating system of the machine hosting the Fabasoft app.teleemtry Server service.
 * @type String
 * @readonly
 */
ServerInfo.prototype.serverOperatingSystem = "";
/**
 * Fabasoft app.teleemtry Server version.
 * @type String
 * @readonly
 */
ServerInfo.prototype.serverVersion = "";
/**
 * Cluster ID of the Fabasoft app.telemetry Server.
 * @type Integer
 * @readonly
 */
ServerInfo.prototype.serverId = 0;
/**
 * Timeout in days for deleting Software-Telemetry data from the file system.
 * Set to 0 to disable automatic cleanup of Software-Telemetry data by time.
 * @type Integer
 * @default 0
 */
ServerInfo.prototype.telemetryDataExpiration = 0;
/**
 * Size limit in GiB for Software-Telemetry data on the file system.
 * Set to 0 to disable automatic cleanup of Software-Telemetry data by size.
 * The limit may be exceeded temporarily.
 * @type Integer
 * @default 0
 */
ServerInfo.prototype.telemetryDataSizeLimit = 0;
/**
 * Size limit in GiB which to keep free on the file system where Software-Telemetry data are located.
 * Software-Telemetry data will be deleted to free disk space in case of disk space shortage.
 * The actual free disk space may not be detectable correctly by Fabasoft app.telemetry on all devices.
 * @type Integer
 * @default 0
 */
ServerInfo.prototype.telemetryDataSizeFree = 0;
/**
 * Timeout in days for status change entries on database.
 * @type Integer
 * @default 30
 */
ServerInfo.prototype.statusDataExpiration = 30;
/**
 * Activate activity statistics (set to 1 to activate).
 * @type Integer
 * @default 0
 */
ServerInfo.prototype.activityActive = 0;
/**
 * Timeout in days for activity statistics data on the file system.
 * Set to 0 to disable automatic cleanup of activity statistics data by time.
 * @type Integer
 * @default 0
 */
ServerInfo.prototype.activityDataExpiration = 0;
/**
 * Limit in GiB for activity statistics data on file system.
 * Set to 0 to disable automatic cleanup of activity statistics data by size.
 * @type Integer
 * @default 0
 */
ServerInfo.prototype.activityDataSizeLimit = 0;
/**
 * Timeout in hours for in memory counter data.
 * @type Integer
 * @default 1
 */
ServerInfo.prototype.counterOnlineTimeout = 1;
/**
 * Timeout in days for in counter data stored on database.
 * Set to 0 to disable automatic cleanup.
 * @type Integer
 * @default 0
 */
ServerInfo.prototype.counterDatabaseTimeout = 0;
/**
 * Database systems provided by the Fabasoft app.telemetry Server.
 * @type {Array.<string>}
 * @readonly
 */
ServerInfo.prototype.supportedDatabases = [];


/**
 * @enum
 */
var TimeRangeType = {
   /**
    * @type Integer
    */
  Core: 0,
  /**
   * @type Integer
   */
  Maintenance: 1,
  /**
   * @type Integer
   */
  Remaining: 2,
  /**
   * @type Integer
   */
  Sendmail: 3,
  /**
   * @type Integer
   */
  Nomail: 4,
};


/**
 * @class
 * @classdesc The TimeRange class.
 */
function TimeRange() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type String
 */
TimeRange.prototype.name = "";
/**
 * @type TimeRangeType
 * @default TimeRangeType.Core
 */
TimeRange.prototype.type = TimeRangeType.Core;
/**
 * Start time of the interval in milliseconds since January 1, 1970 00:00:00.
 * @type Integer
 */
TimeRange.prototype.startTimestamp = 0;
/**
 * End time of the interval in milliseconds since January 1, 1970 00:00:00.
 * @type Integer
 */
TimeRange.prototype.endTimestamp = 0;


/**
 * @class
 * @classdesc The TimeRangeRecurrent class.
 */
function TimeRangeRecurrent() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type String
 */
TimeRangeRecurrent.prototype.name = "";
/**
 * @type TimeRangeType
 * @default TimeRangeType.Core
 */
TimeRangeRecurrent.prototype.type = TimeRangeType.Core;
/**
 * Start date of the interval where this rule is valid in milliseconds since January 1, 1970 00:00:00.
 * @type Integer
 */
TimeRangeRecurrent.prototype.startDate = 0;
/**
 * End date of the interval where this rule is valid in milliseconds since January 1, 1970 00:00:00.
 * @type Integer
 */
TimeRangeRecurrent.prototype.endDate = 0;
/**
 * Start time of the interval on the specified day in milliseconds since midnight.
 * @type Integer
 */
TimeRangeRecurrent.prototype.startTime = 0;
/**
 * End time of the interval on the specified day in milliseconds since midnight.
 * @type Integer
 */
TimeRangeRecurrent.prototype.endTime = 0;
/**
 * Apply the interval on Sunday.
 * @type Boolean
 */
TimeRangeRecurrent.prototype.weekDaySun = false;
/**
 * Apply the interval on Monday.
 * @type Boolean
 */
TimeRangeRecurrent.prototype.weekDayMon = false;
/**
 * Apply the interval on Tuesday.
 * @type Boolean
 */
TimeRangeRecurrent.prototype.weekDayTue = false;
/**
 * Apply the interval on Wednesday.
 * @type Boolean
 */
TimeRangeRecurrent.prototype.weekDayWed = false;
/**
 * Apply the interval on Thursday.
 * @type Boolean
 */
TimeRangeRecurrent.prototype.weekDayThu = false;
/**
 * Apply the interval on Friday.
 * @type Boolean
 */
TimeRangeRecurrent.prototype.weekDayFri = false;
/**
 * Apply the interval on Saturday.
 * @type Boolean
 */
TimeRangeRecurrent.prototype.weekDaySat = false;


/**
 * @class
 * @classdesc The SLADefinition class.
 * @extends CISElement
 */
function SLADefinition() { throw new Error("Direct instantiation not allowed"); }
/**
 * Refer to a HolidayDefinition to take the standard holidays from.
 * @type Integer
 */
SLADefinition.prototype.holiday = 0;
/**
 * Time zone to be used for SLA calculations.
 * @type String
 */
SLADefinition.prototype.timeZoneId = "";
/**
 * List of time ranges to be taken into account during calculation.
 * @type {Array.<TimeRange>}
 */
SLADefinition.prototype.timeRanges = [];
/**
 * List of recurring time ranges to be taken into account.
 * @type {Array.<TimeRangeRecurrent>}
 */
SLADefinition.prototype.recurrentTimeRanges = [];
/**
 * Database to be used to store the service check results in.
 * @type Integer
 * @see DatabaseConnectionOLEDB#id
 * @see DatabaseConnectionPostgreSQL#id
 */
SLADefinition.prototype.database = 0;
/**
 * Database table name.
 * @type String
 * @default "availabilitydata"
 */
SLADefinition.prototype.tablename = "availabilitydata";


/**
 * @class
 * @classdesc The HolidayEntry class.
 */
function HolidayEntry() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type String
 */
HolidayEntry.prototype.name = "";
/**
 * @type Boolean
 * @default false
 */
HolidayEntry.prototype.annual = false;
/**
 * @type Boolean
 * @default true
 */
HolidayEntry.prototype.allday = true;
/**
 * Start time of the interval on the specified day in milliseconds since midnight.
 * @type Integer
 */
HolidayEntry.prototype.startTime = 0;
/**
 * End time of the interval on the specified day in milliseconds since midnight.
 * @type Integer
 */
HolidayEntry.prototype.endTime = 0;
/**
 * Set to "easter" to calculate dates relative to easter feast.
 * @type String
 */
HolidayEntry.prototype.calculation = "";
/**
 * Offset from easter feast calculation (days).
 * @type Integer
 */
HolidayEntry.prototype.offset = 0;
/**
 * Fixed date for single holiday or annual holiday in milliseconds since January 1, 1970 00:00:00.
 * @type Integer
 */
HolidayEntry.prototype.date = 0;


/**
 * @class
 * @classdesc The Holiday class.
 * @extends CISElement
 */
function Holiday() { throw new Error("Direct instantiation not allowed"); }
/**
 * List of HolidayEntries.
 * @type {Array.<HolidayEntry>}
 */
Holiday.prototype.entries = [];


/**
 * @enum
 */
var ServiceDeskAthenticationType = {
  /**
   * Basic authentication (username/password).
   * @type String
   */
  basic: "basic",
  /**
   * Certificate authentication (requires a private key of a client certificate).
   * @type String
   */
  certificate: "certificate",
  /**
   * Kerberos authentication (authenticate as the windows user the service is running under).
   * @type String
   */
  kerberos: "kerberos",
  /**
   * Use a OAuth 2 to authenticate with a web service.
   * @type String
   */
  oauth2: "oauth2",
};


/**
 * @class
 * @classdesc The ServiceDeskMixin class.
 * @mixin
 */
function ServiceDeskMixin() { throw new Error("Direct instantiation not allowed"); }
/**
 * WSDL URL of the service desk (e.g. https://at.cloud.fabasoft.com/folio/fscdav/wsdl?websvc=FSCSERVICEDESK_111_100_ServiceDeskWebService).
 * @type String
 */
ServiceDeskMixin.prototype.url = "";
/**
 * Authentication type.
 * @type ServiceDeskAthenticationType
 */
ServiceDeskMixin.prototype.authtype = "";
/**
 * User name used for automatic transfer of feedbacks to the service desk.
 * @type String
 */
ServiceDeskMixin.prototype.oauthUsername = "";
/**
 * User name for basic authentication.
 * @type String
 */
ServiceDeskMixin.prototype.username = "";
/**
 * Password for basic authentication.
 * @type String
 */
ServiceDeskMixin.prototype.password = "";
/**
 * Passphrase for private key of the certificate (LINUX only).
 * @type String
 */
ServiceDeskMixin.prototype.certificatePassword = "";
/**
 * LINUX: Name to the certificate file (located in /etc/app.telemetry/).
 * Microsoft Windows: CN of the certificate (located in the certificate store of the service user)
 * @type String
 */
ServiceDeskMixin.prototype.certificate = "";
/**
 * HTTP proxy server name.
 * @type String
 */
ServiceDeskMixin.prototype.proxyServer = "";
/**
 * User name for the HTTP proxy server if required.
 * @type String
 */
ServiceDeskMixin.prototype.proxyUsername = "";
/**
 * Password for the HTTP proxy server if required.
 * @type String
 */
ServiceDeskMixin.prototype.proxyPassword = "";
/**
 * ID of the ConfigObject holding the OAuth configuration parameters.
 * @type String
 * @see ConfigObject#id
 */
ServiceDeskMixin.prototype.oauthTarget = "";


/**
 * @class
 * @classdesc The ServiceDesk class.
 * @extends CISElement
 * @extends ServiceDeskMixin
 */
function ServiceDesk() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type String
 * @readonly
 */
ServiceDesk.prototype.type = "fabasoft_servicedesk";


/**
 * @class
 * @classdesc The ServiceDeskFabasoftCloudTicketing class.
 * @extends CISElement
 * @extends ServiceDeskMixin
 */
function ServiceDeskFabasoftCloudTicketing() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type String
 * @readonly
 */
ServiceDeskFabasoftCloudTicketing.prototype.type = "fabasoft_cloud_ticketing_app";


/**
 * Represents a list of charts contained in a column by default.
 * @class
 * @classdesc The DashboardColumn class.
 */
function DashboardColumn() { throw new Error("Direct instantiation not allowed"); }
/**
 * List of charts contained in a column by default.
 * @type {Array.<Integer>}
 * @see Chart#id
 */
DashboardColumn.prototype.charts = [];


/**
 * @class
 * @classdesc The Dashboard class.
 * @extends CISElement
 */
function Dashboard() { throw new Error("Direct instantiation not allowed"); }
/**
 * Dashboard is visible to everyone if set to 1.
 * @type Integer
 */
Dashboard.prototype.public = 0;
/**
 * Authorized user and group names separated by semicolons.
 * @type String
 */
Dashboard.prototype.group = "";
/**
 * List of charts.
 * @type {Array.<integer>}
 * @see Chart#id
 */
Dashboard.prototype.charts = [];
/**
 * List of columns containing default chart order.
 * @type {Array.<DashboardColumn>}
 */
Dashboard.prototype.columns = [];


/**
 * @enum
 */
var ChartControlType = {
  /**
   * @type String
   */
  Gauge: "gauge",
  /**
   * @type String
   */
  Table: "table",
  /**
   * @type String
   */
  Line: "line",
  /**
   * @type String
   */
  Bar: "bar",
  /**
   * @type String
   */
  Pie: "pie",
  /**
   * @type String
   */
  Status: "statuslist",
  /**
   * @type String
   */
  Remote: "remote",
};


/**
 * @class
 * @mixin
 */
function ChartControl() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartControlType
 */
ChartControl.prototype.type = "";
/**
 * @type Integer
 */
ChartControl.prototype.minWidth = 0;
/**
 * @type Integer
 */
ChartControl.prototype.minHeight = 0;
/**
 * @type String
 */
ChartControl.prototype.mode = "";
/**
 * @type String
 */
ChartControl.prototype.format = "";


/**
 * @class
 * @extends ChartControl
 */
function ChartControlGauge() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartControlType
 * @readonly
 * @default ChartControlType.Gauge
 */
ChartControlGauge.prototype.type = ChartControlType.Gauge;
/**
 * @type String
 */
ChartControlGauge.prototype.minValue = "";
/**
 * @type String
 */
ChartControlGauge.prototype.maxValue = "";
/**
 * @type String
 */
ChartControlGauge.prototype.greenMinValue = "";
/**
 * @type String
 */
ChartControlGauge.prototype.greenMaxValue = "";
/**
 * @type String
 */
ChartControlGauge.prototype.yellowMinValue = "";
/**
 * @type String
 */
ChartControlGauge.prototype.yellowMaxValue = "";
/**
 * @type String
 */
ChartControlGauge.prototype.redMinValue = "";
/**
 * @type String
 */
ChartControlGauge.prototype.redMaxValue = "";


/**
 * @class
 * @extends ChartControl
 */
function ChartControlPie() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartControlType
 * @readonly
 * @default ChartControlType.Pie
 */
ChartControlPie.prototype.type = ChartControlType.Pie;
/**
 * @type String
 */
ChartControlPie.prototype.showLegend = "";


/**
 * @class
 * @extends ChartControl
 */
function ChartControlLine() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartControlType
 * @readonly
 * @default ChartControlType.Line
 */
ChartControlLine.prototype.type = ChartControlType.Line;
/**
 * @type String
 */
ChartControlLine.prototype.minValue = "";
/**
 * @type String
 */
ChartControlLine.prototype.maxValue = "";
/**
 * @type String
 */
ChartControlLine.prototype.showLegend = "";
/**
 * @type String
 */
ChartControlLine.prototype.showGrid = "";


/**
 * @class
 * @extends ChartControl
 */
function ChartControlBar() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartControlType
 * @readonly
 * @default ChartControlType.Bar
 */
ChartControlBar.prototype.type = ChartControlType.Bar;
/**
 * @type String
 */
ChartControlBar.prototype.minValue = "";
/**
 * @type String
 */
ChartControlBar.prototype.maxValue = "";
/**
 * @type String
 */
ChartControlBar.prototype.showLegend = "";
/**
 * @type String
 */
ChartControlBar.prototype.showGrid = "";
/**
 * @type String
 */
ChartControlBar.prototype.direction = "";
/**
 * @type String
 */
ChartControlBar.prototype.axisLabelRotation = "";


/**
 * @enum
 */
var ChartDatasourceType = {
  /**
   * @type String
   */
  SLA: "SLA",
  /**
   * @type String
   */
  Baseline: "SWTBaseline",
  /**
   * @type String
   */
  Performance: "SWTPerformance",
  /**
   * @type String
   */
  CounterCheck: "CounterCheck",
  /**
   * @type String
   */
  Status: "Status",
  /**
   * @type String
   */
  CSV: "CSV",
  /**
   * @type String
   */
  Logstat: "SWTLogStatistics",
  /**
   * @type String
   */
  Remote: "Remote",
};


/**
 * @class
 * @mixin
 */
function ChartDatasource() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartDatasourceType
 */
ChartDatasource.prototype.type = "";


/**
 * @class
 * @extends ChartDatasource
 */
function ChartDatasourceSLA() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartDatasourceType
 * @readonly
 * @default ChartDatasourceType.SLA
 */
ChartDatasourceSLA.prototype.type = ChartDatasourceType.SLA;
/**
 * Specifies which values to display (multiple values separated by semicolons).
 * Availability %: core and/or remaining
 * Time: core_avail, core_unavail, core_unknown, remaining_avail, remaining_unavail, remaining_unknown, maintenance_total
 * @type String
 */
ChartDatasourceSLA.prototype.columns = "";
/**
 * Time range of statistic:
 *    today, yesterday, day-n, yyyy-mm-dd ... statistic of a single day
 *    week, lastweek, week-n ... statistic of a week
 *    month, lastmonth, month-n, yyyy-mm ... statistic of a month
 *    quarter, lastquarter, quarter-n ... statistic of a quarter year
 *    yyyy-mm-dd-yyyy-mm-dd .. date range
 * @type String
 */
ChartDatasourceSLA.prototype.timeRange = "";
/**
 * SLADefinition used for calculation.
 * If undefined the SLDefinition of the first service check will be used.
 * @type Integer
 * @see SLADefinition#id
 */
ChartDatasourceSLA.prototype.sla = 0;
/**
 * List of service checks used for calculation.
 * If more than one service check is provided, the service is counted as online if any check result is valid at the particular time.
 * @type {Array.<integer>}
 * @see ServiceCheck#id
 */
ChartDatasourceSLA.prototype.checks = [];


/**
 * The baseline data source provides 24 hour statistics of log analyzer data.
 * @class
 * @extends ChartDatasource
 */
function ChartDatasourceBaseline() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartDatasourceType
 * @readonly
 * @default ChartDatasourceType.Baseline
 */
ChartDatasourceBaseline.prototype.type = ChartDatasourceType.Baseline;
/**
 * LogAnalyzer ID. Statistics must be enabled on this LogAnalyzer.
 * @type Integer
 * @see APMLogAnalyzer#id
 * @see WebTimingLogAnalyzer#id
 * @see FilterLogAnalyzer#id
 */
ChartDatasourceBaseline.prototype.logpool = 0;
/**
 * Counter to be selected (e.g. "count")
 * @type String
 */
ChartDatasourceBaseline.prototype.query = "";
/**
 * Filter for specific requests (only dimension columns may be referenced in the filter).
 * @type String
 */
ChartDatasourceBaseline.prototype.filter = "";
/**
 * Group by a dimension column to show on line per dimension value.
 * @type String
 */
ChartDatasourceBaseline.prototype.groupBy = "";
/**
 * List of time ranges.
 * today, yesterday, day-n, yyyy-mm-dd
 * @type {Array.<string>}
 */
ChartDatasourceBaseline.prototype.timeRanges = [];


/**
 * The performance data source provides statistics about current request duration.
 * @class
 * @extends ChartDatasource
 */
function ChartDatasourcePerformance() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartDatasourceType
 * @readonly
 * @default ChartDatasourceType.Performance
 */
ChartDatasourcePerformance.prototype.type = ChartDatasourceType.Performance;
/**
 * LogAnalyzer ID.
 * @type Integer
 * @see APMLogAnalyzer#id
 * @see WebTimingLogAnalyzer#id
 * @see FilterLogAnalyzer#id
 */
ChartDatasourcePerformance.prototype.logpool = 0;
/**
 * Time in minutes. The statistic is being calculate over the requests currently in memory.
 * So the statistics interval is limited by the LogAnalyzer.onlineTime.
 * @type String
 * @see LogAnalyzerWithDataMixin#onlineTimeout
 */
ChartDatasourcePerformance.prototype.timeRange = "";
/**
 * Set to "percent" to display percentages.
 * @type String
 */
ChartDatasourcePerformance.prototype.column = "";
/**
 * Filter requests included in calculation by an SQL like WHERE clause (e.g. "duration < 600000000").
 * @type String
 */
ChartDatasourcePerformance.prototype.filter = "";
/**
 * List of thresholds or intervals for categorization separated by comma.
 * e.g: 1,2,3,4,5,5-,0-0.3
 * @type String
 */
ChartDatasourcePerformance.prototype.durationClass = "";


/**
 * The counter check data source provides current counter values and counter line charts.
 * Data will be provides from the in memory counter values.
 * If the timeRange exceeds the data available in memory and counter data are persisted on database, values will be complemented by those provided by the database.
 * @class
 * @extends ChartDatasource
 */
function ChartDatasourceCountercheck() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartDatasourceType
 * @readonly
 * @default ChartDatasourceType.Countercheck
 */
ChartDatasourceCountercheck.prototype.type = ChartDatasourceType.Countercheck;
/**
 * Select the time frame.
 * current ... display the current value
 * current-n ... last n seconds
 * minute-n ... last n minutes
 * @type String
 */
ChartDatasourceCountercheck.prototype.timeRange = "";
/**
 * List of counter checks.
 * @type {Array.<integer>}
 * @see ServiceCheck#id
 */
ChartDatasourceCountercheck.prototype.checks = [];


/**
 * The status data source provides current status information.
 * @class
 * @extends ChartDatasource
 */
function ChartDatasourceStatus() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartDatasourceType
 * @readonly
 * @default ChartDatasourceType.Status
 */
ChartDatasourceStatus.prototype.type = ChartDatasourceType.Status;
/**
 * List of service groups, services and service checks.
 * @type {Array.<integer>}
 * @see ServiceGroup#id
 * @see Service#id
 * @see ServiceCheck#id
 */
ChartDatasourceStatus.prototype.items = [];


/**
 * @class
 * @extends ChartDatasource
 */
function ChartDatasourceCSV() { throw new Error("Direct instantiation not allowed"); }
/**
 * The CSV data source provides values taken from CSV files.
 * @type ChartDatasourceType
 * @readonly
 * @default ChartDatasourceType.CSV
 */
ChartDatasourceCSV.prototype.type = ChartDatasourceType.CSV;
/**
 * Name of the CSV source file located under "/var/opt/app.telemetry/worker/chart" respectively "%PROGRAMDATA%\Fabasoft app.telemetry\worker\chart".
 * @type String
 */
ChartDatasourceCSV.prototype.filename = "";


/**
 * The log statistics data source provides statistics based on LogStatistics.
 * @class
 * @extends ChartDatasource
 */
function ChartDatasourceLogstat() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartDatasourceType
 * @readonly
 * @default ChartDatasourceType.Logstat
 */
ChartDatasourceLogstat.prototype.type = ChartDatasourceType.Logstat;
/**
 * LogAnalyzer Id.
 * @type Integer
 * @see APMLogAnalyzer#id
 * @see WebTimingLogAnalyzer#id
 * @see FilterLogAnalyzer#id
 */
ChartDatasourceLogstat.prototype.logpool = 0;
/**
 * LogStatistic ID. If 0 the default log pool statistic is used.
 * @type Integer
 */
ChartDatasourceLogstat.prototype.statistic = 0;
/**
 * Time range
 *   last-n ... last n values will be displayed.
 * @type String
 */
ChartDatasourceLogstat.prototype.timeRange = "";
/**
 * Filter data by specific dimension values.
 * @type String
 */
ChartDatasourceLogstat.prototype.filter = "";
/**
 * Group data by a dimension.
 * @type String
 */
ChartDatasourceLogstat.prototype.groupBy = "";
/**
 * Order grouped dimension by columns.
 * @type String
 */
ChartDatasourceLogstat.prototype.orderBy = "";
/**
 * Display only first n dimension values.
 * @type Integer
 */
ChartDatasourceLogstat.prototype.elementCount = 0;
/**
 * List of measures to be displayed.
 * @type {Array.<string>}
 */
ChartDatasourceLogstat.prototype.measures = [];


/**
 * A web page referred by an URL will be displayed within an iframe in a chart.
 * @class
 * @extends ChartDatasource
 */
function ChartDatasourceRemote() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type ChartDatasourceType
 * @readonly
 * @default ChartDatasourceType.Remote
 */
ChartDatasourceRemote.prototype.type = ChartDatasourceType.Remote;
/**
 * URL.
 * @type String
 */
ChartDatasourceRemote.prototype.url = "";


/**
 * @class
 * @extends CISElement
 */
function Chart() { throw new Error("Direct instantiation not allowed"); }
/**
 * The update interval of the chart data in seconds.
 * @type Integer
 */
Chart.prototype.updateInterval = 60;
/**
 * The control used to display the data.
 * @type ChartControl
 */
Chart.prototype.control = null;
/**
 * The source for the data to display.
 * @type ChartDatasource
 */
Chart.prototype.datasource = null;


/**
 * @enum
 */
var NotificationSendtimeFlags = {
  /**
   * @type Integer
   */
  Core: 1,
  /**
   * @type Integer
   */
  Remaining: 2,
  /**
   * @type Integer
   */
  Maintenance: 4,
};


/**
 * Configure a NotificationSchedule to define the time intervals when notifications may be sent.
 * @class
 * @extends CISElement
 */
function NotificationSchedule() { throw new Error("Direct instantiation not allowed"); }
/**
 * HolidayDefinition ID.
 * @type Integer
 * @see Holiday#id
 */
NotificationSchedule.prototype.holiday = 0;
/**
 * Time zone to be used for the NotificationSchedule.
 * @type String
 */
NotificationSchedule.prototype.timeZoneId = "";
/**
 * List of time ranges to be taken into account during calculation.
 * @type {Array.<TimeRange>}
 */
NotificationSchedule.prototype.timeRanges = [];
/**
 * List of recurring time ranges to be taken into account.
 * @type {Array.<TimeRangeRecurrent>}
 */
NotificationSchedule.prototype.recurrentTimeRanges = [];
/**
 * SLADefinition ID where the timeRanges and recurrentTimeRanges are taken from.
 * @type Integer
 * @see SLADefinition#id
 */
NotificationSchedule.prototype.sla = 0;
/**
 * Send notifications if current time is included in core, remaining and/or maintenance time.
 * @type NotificationSendtimeFlags
 * @default NotificationSendtimeFlags.Core | NotificationSendtimeFlags.Remaining
 */
NotificationSchedule.prototype.sendtime = NotificationSendtimeFlags.Core | NotificationSendtimeFlags.Remaining;
/**
 * List of service groups, services and service checks to which this definition will apply.
 * @type {Array.<integer>}
 * @see ServiceGroup#id
 * @see Service#id
 * @see ServiceCheck#id
 */
NotificationSchedule.prototype.checks = [];


/**
 * @class
 * @extends CISElement
 */
function ConfigObject() { throw new Error("Direct instantiation not allowed"); }
/**
 * The name of the category of the configuration object.
 * @type String
 */
ConfigObject.prototype.category = "";
/**
 * An index to allow ordering of multiple configuration objects with the same category to express preference.
 * A lower value represents a higher priority.
 * @type Integer
 */
ConfigObject.prototype.index = 0;


/**
 * @enum
 */
var AccessType = {
  /**
   * No access.
   * @type Integer
   */
  None: 0,
  /**
   * Read access.
   * @type Integer
   */
  Read: 1,
  /**
   * Write access.
   * @type Integer
   */
  Write: 2,
};


/**
 * @class
 */
function AccessEntry() { throw new Error("Direct instantiation not allowed"); }
/**
 * User name.
 * @type String
 */
AccessEntry.prototype.user = "";
/**
 * Access type.
 * @type AccessType
 * @default AccessType.None
 */
AccessEntry.prototype.access = AccessType.None;


/**
 * @enum
 */
var LabelType = {
  /**
   * Type for Labels that show up on the "Application Overview" in the Fabasoft app.telemetry Client.
   * @type Integer
   */
  Application: 1,
};


/**
 * @class
 * @extends CISElement
 */
function Label() { throw new Error("Direct instantiation not allowed"); }
/**
 * @type LabelType
 * @default LabelType.Application
 */
Label.prototype.type = LabelType.Application;
/**
 * Application Name that this Label represents.
 * @type String
 */
Label.prototype.appName = "";
/**
 * Application ID that this Label represents.
 * @type String
 */
Label.prototype.appId = "";
/**
 * List of infrastructure items.
 * @type {Array.<integer>}
 * @see CISElement#id
 */
Label.prototype.items = [];
/**
 * List of access definitions.
 * @type {Array.<AccessEntry>}
 */
Label.prototype.acl = [];


/**
 * @class
 * @extends CISElement
 */
function Module() { throw new Error("Direct instantiation not allowed"); }
/**
 * List of infrastructure items.
 * @type {Array.<integer>}
 * @see CISElement#id
 */
Module.prototype.items = [];


/**
 * @class
 */
function Config() { throw new Error("Direct instantiation not allowed"); }

/**
 * @returns {@link ServerInfo}
 */
Config.prototype.serverSettings = function() {};
/**
 * Enumerate all instances of the given protocol on the target identified by agent.
 * Note: The special instance with the name "_Total" will NOT be returned in the set.
 * @param {Number|Agent} agent The {@link Agent#id} of an agent or an ionstance of {@link Agent} that
 * shall be queried. Note: The Agent must already have been committed in a previous upgrade transaction before it can be used.
 * @param {ServiceCheckProtocol} protocol the protocol used to enumerate based on the given query.
 * @param {String} query a protocol specific query to limit the returned instances.
 * @returns {Array.<string>} array of instance names.
 */
Config.prototype.enumInstances = function(agent, protocol, query) {};


/// START GENERATED DOCUMENTATION

/**
 * Selector callback for Server objects. May be invoked multiple times!
 * @callback selectorCallbackServer
 * @param {Server} server matching your selection criteria.
 */

/**
 * Searches for all Servers with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackServer=} callback invoked for all matches of the search.
 * @returns {Array.<Server>} matching Server objects in implementation defined order.
 * @example <caption>Search for all servers with reference 'ref' globally</caption>
 * var array = cfg.server('ref');
 * @example <caption>Update or create a new Server</caption>
 * cfg.server('ref', function(server) { ... });
 */
Config.prototype.server = function(reference, callback) {};
/**
 * Selector callback for Server objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackServer
 * @param {Server} server the Server instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Server instance.
 */

/**
 * Searches for all Servers with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackServer} callback invoked for all instances.
 * @return {Array.<Server>} all processed Server objects in implementation defined order.
 * @example <caption>Create Server instances for every element of the instances array</caption>
 * var servers = cfg.servers('ref', ['node1', 'node2'], function(server, instance) { ... });
 */
Config.prototype.servers = function(reference, instances, callback) {};


/**
 * Selector callback for ConfigObject objects. May be invoked multiple times!
 * @callback selectorCallbackConfigObject
 * @param {ConfigObject} configObject matching your selection criteria.
 */

/**
 * Searches for all ConfigObjects with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackConfigObject=} callback invoked for all matches of the search.
 * @returns {Array.<ConfigObject>} matching ConfigObject objects in implementation defined order.
 * @example <caption>Search for all configObjects with reference 'ref' globally</caption>
 * var array = cfg.configObject('ref');
 * @example <caption>Update or create a new ConfigObject</caption>
 * cfg.configObject('ref', function(configObject) { ... });
 */
Config.prototype.configObject = function(reference, callback) {};
/**
 * Selector callback for ConfigObject objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackConfigObject
 * @param {ConfigObject} configObject the ConfigObject instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed ConfigObject instance.
 */

/**
 * Searches for all ConfigObjects with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackConfigObject} callback invoked for all instances.
 * @return {Array.<ConfigObject>} all processed ConfigObject objects in implementation defined order.
 * @example <caption>Create ConfigObject instances for every element of the instances array</caption>
 * var configObjects = cfg.configObjects('ref', ['node1', 'node2'], function(configObject, instance) { ... });
 */
Config.prototype.configObjects = function(reference, instances, callback) {};


/**
 * Selector callback for AgentGroup objects. May be invoked multiple times!
 * @callback selectorCallbackAgentGroup
 * @param {AgentGroup} agentGroup matching your selection criteria.
 */

/**
 * Searches for all AgentGroups with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {AgentGroup=} parent the AgentGroup inside which the agentGroup must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackAgentGroup=} callback invoked for all matches of the search.
 * @returns {Array.<AgentGroup>} matching AgentGroup objects in implementation defined order.
 * @example <caption>Search for all agentGroups with reference 'ref' inside parent</caption>
 * var array = cfg.agentGroup(parent, 'ref');
 * @example <caption>Search for all agentGroups with reference 'ref' globally</caption>
 * var array = cfg.agentGroup('ref');
 * @example <caption>Update or create a new AgentGroup inside parent</caption>
 * cfg.agentGroup(parent, 'ref', function(agentGroup) { ... });
 */
Config.prototype.agentGroup = function(parent, reference, callback) {};
/**
 * Selector callback for AgentGroup objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackAgentGroup
 * @param {AgentGroup} agentGroup the AgentGroup instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed AgentGroup instance.
 */

/**
 * Searches for all AgentGroups with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {AgentGroup} parent the AgentGroup inside which the agentGroup must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackAgentGroup} callback invoked for all instances.
 * @return {Array.<AgentGroup>} all processed AgentGroup objects in implementation defined order.
 * @example <caption>Create AgentGroup instances for every element of the instances array</caption>
 * var agentGroups = cfg.agentGroups(parent, 'ref', ['node1', 'node2'], function(agentGroup, instance) { ... });
 */
Config.prototype.agentGroups = function(parent, reference, instances, callback) {};


/**
 * Selector callback for Agent objects. May be invoked multiple times!
 * @callback selectorCallbackAgent
 * @param {Agent} agent matching your selection criteria.
 */

/**
 * Searches for all Agents with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {AgentGroup=} parent the AgentGroup inside which the agent must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackAgent=} callback invoked for all matches of the search.
 * @returns {Array.<Agent>} matching Agent objects in implementation defined order.
 * @example <caption>Search for all agents with reference 'ref' inside parent</caption>
 * var array = cfg.agent(parent, 'ref');
 * @example <caption>Search for all agents with reference 'ref' globally</caption>
 * var array = cfg.agent('ref');
 * @example <caption>Update or create a new Agent inside parent</caption>
 * cfg.agent(parent, 'ref', function(agent) { ... });
 */
Config.prototype.agent = function(parent, reference, callback) {};
/**
 * Selector callback for Agent objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackAgent
 * @param {Agent} agent the Agent instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Agent instance.
 */

/**
 * Searches for all Agents with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {AgentGroup} parent the AgentGroup inside which the agent must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackAgent} callback invoked for all instances.
 * @return {Array.<Agent>} all processed Agent objects in implementation defined order.
 * @example <caption>Create Agent instances for every element of the instances array</caption>
 * var agents = cfg.agents(parent, 'ref', ['node1', 'node2'], function(agent, instance) { ... });
 */
Config.prototype.agents = function(parent, reference, instances, callback) {};


/**
 * Selector callback for ServiceGroup objects. May be invoked multiple times!
 * @callback selectorCallbackServiceGroup
 * @param {ServiceGroup} serviceGroup matching your selection criteria.
 */

/**
 * Searches for all ServiceGroups with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {ServiceGroup=} parent the ServiceGroup inside which the serviceGroup must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackServiceGroup=} callback invoked for all matches of the search.
 * @returns {Array.<ServiceGroup>} matching ServiceGroup objects in implementation defined order.
 * @example <caption>Search for all serviceGroups with reference 'ref' inside parent</caption>
 * var array = cfg.serviceGroup(parent, 'ref');
 * @example <caption>Search for all serviceGroups with reference 'ref' globally</caption>
 * var array = cfg.serviceGroup('ref');
 * @example <caption>Update or create a new ServiceGroup inside parent</caption>
 * cfg.serviceGroup(parent, 'ref', function(serviceGroup) { ... });
 */
Config.prototype.serviceGroup = function(parent, reference, callback) {};
/**
 * Selector callback for ServiceGroup objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackServiceGroup
 * @param {ServiceGroup} serviceGroup the ServiceGroup instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed ServiceGroup instance.
 */

/**
 * Searches for all ServiceGroups with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {ServiceGroup} parent the ServiceGroup inside which the serviceGroup must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackServiceGroup} callback invoked for all instances.
 * @return {Array.<ServiceGroup>} all processed ServiceGroup objects in implementation defined order.
 * @example <caption>Create ServiceGroup instances for every element of the instances array</caption>
 * var serviceGroups = cfg.serviceGroups(parent, 'ref', ['node1', 'node2'], function(serviceGroup, instance) { ... });
 */
Config.prototype.serviceGroups = function(parent, reference, instances, callback) {};


/**
 * Selector callback for Service objects. May be invoked multiple times!
 * @callback selectorCallbackService
 * @param {Service} service matching your selection criteria.
 */

/**
 * Searches for all Services with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {ServiceGroup=} parent the ServiceGroup inside which the service must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackService=} callback invoked for all matches of the search.
 * @returns {Array.<Service>} matching Service objects in implementation defined order.
 * @example <caption>Search for all services with reference 'ref' inside parent</caption>
 * var array = cfg.service(parent, 'ref');
 * @example <caption>Search for all services with reference 'ref' globally</caption>
 * var array = cfg.service('ref');
 * @example <caption>Update or create a new Service inside parent</caption>
 * cfg.service(parent, 'ref', function(service) { ... });
 */
Config.prototype.service = function(parent, reference, callback) {};
/**
 * Selector callback for Service objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackService
 * @param {Service} service the Service instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Service instance.
 */

/**
 * Searches for all Services with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {ServiceGroup} parent the ServiceGroup inside which the service must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackService} callback invoked for all instances.
 * @return {Array.<Service>} all processed Service objects in implementation defined order.
 * @example <caption>Create Service instances for every element of the instances array</caption>
 * var services = cfg.services(parent, 'ref', ['node1', 'node2'], function(service, instance) { ... });
 */
Config.prototype.services = function(parent, reference, instances, callback) {};


/**
 * Selector callback for ServiceCheck objects. May be invoked multiple times!
 * @callback selectorCallbackServiceCheck
 * @param {ServiceCheck} serviceCheck matching your selection criteria.
 */

/**
 * Searches for all ServiceChecks with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {Service=} parent the Service inside which the serviceCheck must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackServiceCheck=} callback invoked for all matches of the search.
 * @returns {Array.<ServiceCheck>} matching ServiceCheck objects in implementation defined order.
 * @example <caption>Search for all serviceChecks with reference 'ref' inside parent</caption>
 * var array = cfg.serviceCheck(parent, 'ref');
 * @example <caption>Search for all serviceChecks with reference 'ref' globally</caption>
 * var array = cfg.serviceCheck('ref');
 * @example <caption>Update or create a new ServiceCheck inside parent</caption>
 * cfg.serviceCheck(parent, 'ref', function(serviceCheck) { ... });
 */
Config.prototype.serviceCheck = function(parent, reference, callback) {};
/**
 * Selector callback for ServiceCheck objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackServiceCheck
 * @param {ServiceCheck} serviceCheck the ServiceCheck instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed ServiceCheck instance.
 */

/**
 * Searches for all ServiceChecks with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {Service} parent the Service inside which the serviceCheck must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackServiceCheck} callback invoked for all instances.
 * @return {Array.<ServiceCheck>} all processed ServiceCheck objects in implementation defined order.
 * @example <caption>Create ServiceCheck instances for every element of the instances array</caption>
 * var serviceChecks = cfg.serviceChecks(parent, 'ref', ['node1', 'node2'], function(serviceCheck, instance) { ... });
 */
Config.prototype.serviceChecks = function(parent, reference, instances, callback) {};


/**
 * Selector callback for Counter objects. May be invoked multiple times!
 * @callback selectorCallbackCounter
 * @param {Counter} counter matching your selection criteria.
 */

/**
 * Searches for all Counters with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackCounter=} callback invoked for all matches of the search.
 * @returns {Array.<Counter>} matching Counter objects in implementation defined order.
 * @example <caption>Search for all counters with reference 'ref' globally</caption>
 * var array = cfg.counter('ref');
 * @example <caption>Update or create a new Counter</caption>
 * cfg.counter('ref', function(counter) { ... });
 */
Config.prototype.counter = function(reference, callback) {};
/**
 * Selector callback for Counter objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackCounter
 * @param {Counter} counter the Counter instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Counter instance.
 */

/**
 * Searches for all Counters with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackCounter} callback invoked for all instances.
 * @return {Array.<Counter>} all processed Counter objects in implementation defined order.
 * @example <caption>Create Counter instances for every element of the instances array</caption>
 * var counters = cfg.counters('ref', ['node1', 'node2'], function(counter, instance) { ... });
 */
Config.prototype.counters = function(reference, instances, callback) {};


/**
 * Selector callback for NotificationChannel objects. May be invoked multiple times!
 * @callback selectorCallbackNotificationChannel
 * @param {NotificationChannel} notificationChannel matching your selection criteria.
 */

/**
 * Searches for all NotificationChannels with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackNotificationChannel=} callback invoked for all matches of the search.
 * @returns {Array.<NotificationChannel>} matching NotificationChannel objects in implementation defined order.
 * @example <caption>Search for all notificationChannels with reference 'ref' globally</caption>
 * var array = cfg.notificationChannel('ref');
 * @example <caption>Update or create a new NotificationChannel</caption>
 * cfg.notificationChannel('ref', function(notificationChannel) { ... });
 */
Config.prototype.notificationChannel = function(reference, callback) {};
/**
 * Selector callback for NotificationChannel objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackNotificationChannel
 * @param {NotificationChannel} notificationChannel the NotificationChannel instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed NotificationChannel instance.
 */

/**
 * Searches for all NotificationChannels with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackNotificationChannel} callback invoked for all instances.
 * @return {Array.<NotificationChannel>} all processed NotificationChannel objects in implementation defined order.
 * @example <caption>Create NotificationChannel instances for every element of the instances array</caption>
 * var notificationChannels = cfg.notificationChannels('ref', ['node1', 'node2'], function(notificationChannel, instance) { ... });
 */
Config.prototype.notificationChannels = function(reference, instances, callback) {};


/**
 * Selector callback for NotificationSchedule objects. May be invoked multiple times!
 * @callback selectorCallbackNotificationSchedule
 * @param {NotificationSchedule} notificationSchedule matching your selection criteria.
 */

/**
 * Searches for all NotificationSchedules with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackNotificationSchedule=} callback invoked for all matches of the search.
 * @returns {Array.<NotificationSchedule>} matching NotificationSchedule objects in implementation defined order.
 * @example <caption>Search for all notificationSchedules with reference 'ref' globally</caption>
 * var array = cfg.notificationSchedule('ref');
 * @example <caption>Update or create a new NotificationSchedule</caption>
 * cfg.notificationSchedule('ref', function(notificationSchedule) { ... });
 */
Config.prototype.notificationSchedule = function(reference, callback) {};
/**
 * Selector callback for NotificationSchedule objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackNotificationSchedule
 * @param {NotificationSchedule} notificationSchedule the NotificationSchedule instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed NotificationSchedule instance.
 */

/**
 * Searches for all NotificationSchedules with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackNotificationSchedule} callback invoked for all instances.
 * @return {Array.<NotificationSchedule>} all processed NotificationSchedule objects in implementation defined order.
 * @example <caption>Create NotificationSchedule instances for every element of the instances array</caption>
 * var notificationSchedules = cfg.notificationSchedules('ref', ['node1', 'node2'], function(notificationSchedule, instance) { ... });
 */
Config.prototype.notificationSchedules = function(reference, instances, callback) {};


/**
 * Selector callback for NotificationAccount objects. May be invoked multiple times!
 * @callback selectorCallbackNotificationAccount
 * @param {NotificationAccount} notificationAccount matching your selection criteria.
 */

/**
 * Searches for all NotificationAccounts with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {NotificationChannel=} parent the NotificationChannel inside which the notificationAccount must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackNotificationAccount=} callback invoked for all matches of the search.
 * @returns {Array.<NotificationAccount>} matching NotificationAccount objects in implementation defined order.
 * @example <caption>Search for all notificationAccounts with reference 'ref' inside parent</caption>
 * var array = cfg.notificationAccount(parent, 'ref');
 * @example <caption>Search for all notificationAccounts with reference 'ref' globally</caption>
 * var array = cfg.notificationAccount('ref');
 * @example <caption>Update or create a new NotificationAccount inside parent</caption>
 * cfg.notificationAccount(parent, 'ref', function(notificationAccount) { ... });
 */
Config.prototype.notificationAccount = function(parent, reference, callback) {};
/**
 * Selector callback for NotificationAccount objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackNotificationAccount
 * @param {NotificationAccount} notificationAccount the NotificationAccount instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed NotificationAccount instance.
 */

/**
 * Searches for all NotificationAccounts with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {NotificationChannel} parent the NotificationChannel inside which the notificationAccount must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackNotificationAccount} callback invoked for all instances.
 * @return {Array.<NotificationAccount>} all processed NotificationAccount objects in implementation defined order.
 * @example <caption>Create NotificationAccount instances for every element of the instances array</caption>
 * var notificationAccounts = cfg.notificationAccounts(parent, 'ref', ['node1', 'node2'], function(notificationAccount, instance) { ... });
 */
Config.prototype.notificationAccounts = function(parent, reference, instances, callback) {};


/**
 * Selector callback for ServiceDesk objects. May be invoked multiple times!
 * @callback selectorCallbackServiceDesk
 * @param {ServiceDesk} serviceDesk matching your selection criteria.
 */

/**
 * Searches for all ServiceDesks with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackServiceDesk=} callback invoked for all matches of the search.
 * @returns {Array.<ServiceDesk>} matching ServiceDesk objects in implementation defined order.
 * @example <caption>Search for all serviceDesks with reference 'ref' globally</caption>
 * var array = cfg.serviceDesk('ref');
 * @example <caption>Update or create a new ServiceDesk</caption>
 * cfg.serviceDesk('ref', function(serviceDesk) { ... });
 */
Config.prototype.serviceDesk = function(reference, callback) {};
/**
 * Selector callback for ServiceDesk objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackServiceDesk
 * @param {ServiceDesk} serviceDesk the ServiceDesk instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed ServiceDesk instance.
 */

/**
 * Searches for all ServiceDesks with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackServiceDesk} callback invoked for all instances.
 * @return {Array.<ServiceDesk>} all processed ServiceDesk objects in implementation defined order.
 * @example <caption>Create ServiceDesk instances for every element of the instances array</caption>
 * var serviceDesks = cfg.serviceDesks('ref', ['node1', 'node2'], function(serviceDesk, instance) { ... });
 */
Config.prototype.serviceDesks = function(reference, instances, callback) {};


/**
 * Selector callback for ServiceDeskFabasoftCloudTicketing objects. May be invoked multiple times!
 * @callback selectorCallbackServiceDeskFabasoftCloudTicketing
 * @param {ServiceDeskFabasoftCloudTicketing} fabasoftServiceDesk matching your selection criteria.
 */

/**
 * Searches for all ServiceDeskFabasoftCloudTicketings with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackServiceDeskFabasoftCloudTicketing=} callback invoked for all matches of the search.
 * @returns {Array.<ServiceDeskFabasoftCloudTicketing>} matching ServiceDeskFabasoftCloudTicketing objects in implementation defined order.
 * @example <caption>Search for all fabasoftServiceDesks with reference 'ref' globally</caption>
 * var array = cfg.fabasoftServiceDesk('ref');
 * @example <caption>Update or create a new ServiceDeskFabasoftCloudTicketing</caption>
 * cfg.fabasoftServiceDesk('ref', function(fabasoftServiceDesk) { ... });
 */
Config.prototype.fabasoftServiceDesk = function(reference, callback) {};
/**
 * Selector callback for ServiceDeskFabasoftCloudTicketing objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackServiceDeskFabasoftCloudTicketing
 * @param {ServiceDeskFabasoftCloudTicketing} fabasoftServiceDesk the ServiceDeskFabasoftCloudTicketing instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed ServiceDeskFabasoftCloudTicketing instance.
 */

/**
 * Searches for all ServiceDeskFabasoftCloudTicketings with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackServiceDeskFabasoftCloudTicketing} callback invoked for all instances.
 * @return {Array.<ServiceDeskFabasoftCloudTicketing>} all processed ServiceDeskFabasoftCloudTicketing objects in implementation defined order.
 * @example <caption>Create ServiceDeskFabasoftCloudTicketing instances for every element of the instances array</caption>
 * var fabasoftServiceDesks = cfg.fabasoftServiceDesks('ref', ['node1', 'node2'], function(fabasoftServiceDesk, instance) { ... });
 */
Config.prototype.fabasoftServiceDesks = function(reference, instances, callback) {};


/**
 * Selector callback for DatabaseConnectionOLEDB objects. May be invoked multiple times!
 * @callback selectorCallbackDatabaseConnectionOLEDB
 * @param {DatabaseConnectionOLEDB} mssqlDatabaseConnection matching your selection criteria.
 */

/**
 * Searches for all DatabaseConnectionOLEDBs with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackDatabaseConnectionOLEDB=} callback invoked for all matches of the search.
 * @returns {Array.<DatabaseConnectionOLEDB>} matching DatabaseConnectionOLEDB objects in implementation defined order.
 * @example <caption>Search for all mssqlDatabaseConnections with reference 'ref' globally</caption>
 * var array = cfg.mssqlDatabaseConnection('ref');
 * @example <caption>Update or create a new DatabaseConnectionOLEDB</caption>
 * cfg.mssqlDatabaseConnection('ref', function(mssqlDatabaseConnection) { ... });
 */
Config.prototype.mssqlDatabaseConnection = function(reference, callback) {};
/**
 * Selector callback for DatabaseConnectionOLEDB objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackDatabaseConnectionOLEDB
 * @param {DatabaseConnectionOLEDB} mssqlDatabaseConnection the DatabaseConnectionOLEDB instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed DatabaseConnectionOLEDB instance.
 */

/**
 * Searches for all DatabaseConnectionOLEDBs with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackDatabaseConnectionOLEDB} callback invoked for all instances.
 * @return {Array.<DatabaseConnectionOLEDB>} all processed DatabaseConnectionOLEDB objects in implementation defined order.
 * @example <caption>Create DatabaseConnectionOLEDB instances for every element of the instances array</caption>
 * var mssqlDatabaseConnections = cfg.mssqlDatabaseConnections('ref', ['node1', 'node2'], function(mssqlDatabaseConnection, instance) { ... });
 */
Config.prototype.mssqlDatabaseConnections = function(reference, instances, callback) {};


/**
 * Selector callback for DatabaseConnectionPostgreSQL objects. May be invoked multiple times!
 * @callback selectorCallbackDatabaseConnectionPostgreSQL
 * @param {DatabaseConnectionPostgreSQL} pgsqlDatabaseConnection matching your selection criteria.
 */

/**
 * Searches for all DatabaseConnectionPostgreSQLs with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackDatabaseConnectionPostgreSQL=} callback invoked for all matches of the search.
 * @returns {Array.<DatabaseConnectionPostgreSQL>} matching DatabaseConnectionPostgreSQL objects in implementation defined order.
 * @example <caption>Search for all pgsqlDatabaseConnections with reference 'ref' globally</caption>
 * var array = cfg.pgsqlDatabaseConnection('ref');
 * @example <caption>Update or create a new DatabaseConnectionPostgreSQL</caption>
 * cfg.pgsqlDatabaseConnection('ref', function(pgsqlDatabaseConnection) { ... });
 */
Config.prototype.pgsqlDatabaseConnection = function(reference, callback) {};
/**
 * Selector callback for DatabaseConnectionPostgreSQL objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackDatabaseConnectionPostgreSQL
 * @param {DatabaseConnectionPostgreSQL} pgsqlDatabaseConnection the DatabaseConnectionPostgreSQL instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed DatabaseConnectionPostgreSQL instance.
 */

/**
 * Searches for all DatabaseConnectionPostgreSQLs with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackDatabaseConnectionPostgreSQL} callback invoked for all instances.
 * @return {Array.<DatabaseConnectionPostgreSQL>} all processed DatabaseConnectionPostgreSQL objects in implementation defined order.
 * @example <caption>Create DatabaseConnectionPostgreSQL instances for every element of the instances array</caption>
 * var pgsqlDatabaseConnections = cfg.pgsqlDatabaseConnections('ref', ['node1', 'node2'], function(pgsqlDatabaseConnection, instance) { ... });
 */
Config.prototype.pgsqlDatabaseConnections = function(reference, instances, callback) {};


/**
 * Selector callback for APMLogAnalyzer objects. May be invoked multiple times!
 * @callback selectorCallbackAPMLogAnalyzer
 * @param {APMLogAnalyzer} logPool matching your selection criteria.
 */

/**
 * Searches for all APMLogAnalyzers with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackAPMLogAnalyzer=} callback invoked for all matches of the search.
 * @returns {Array.<APMLogAnalyzer>} matching APMLogAnalyzer objects in implementation defined order.
 * @example <caption>Search for all logPools with reference 'ref' globally</caption>
 * var array = cfg.logPool('ref');
 * @example <caption>Update or create a new APMLogAnalyzer</caption>
 * cfg.logPool('ref', function(logPool) { ... });
 */
Config.prototype.logPool = function(reference, callback) {};
/**
 * Selector callback for APMLogAnalyzer objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackAPMLogAnalyzer
 * @param {APMLogAnalyzer} logPool the APMLogAnalyzer instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed APMLogAnalyzer instance.
 */

/**
 * Searches for all APMLogAnalyzers with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackAPMLogAnalyzer} callback invoked for all instances.
 * @return {Array.<APMLogAnalyzer>} all processed APMLogAnalyzer objects in implementation defined order.
 * @example <caption>Create APMLogAnalyzer instances for every element of the instances array</caption>
 * var logPools = cfg.logPools('ref', ['node1', 'node2'], function(logPool, instance) { ... });
 */
Config.prototype.logPools = function(reference, instances, callback) {};


/**
 * Selector callback for FilterLogAnalyzer objects. May be invoked multiple times!
 * @callback selectorCallbackFilterLogAnalyzer
 * @param {FilterLogAnalyzer} filteredLogPool matching your selection criteria.
 */

/**
 * Searches for all FilterLogAnalyzers with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {APMLogAnalyzer|FilterLogAnalyzer|WebTimingLogAnalyzer|FormLogAnalyzer=} parent the APMLogAnalyzer|FilterLogAnalyzer|WebTimingLogAnalyzer|FormLogAnalyzer inside which the filteredLogPool must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackFilterLogAnalyzer=} callback invoked for all matches of the search.
 * @returns {Array.<FilterLogAnalyzer>} matching FilterLogAnalyzer objects in implementation defined order.
 * @example <caption>Search for all filteredLogPools with reference 'ref' inside parent</caption>
 * var array = cfg.filteredLogPool(parent, 'ref');
 * @example <caption>Search for all filteredLogPools with reference 'ref' globally</caption>
 * var array = cfg.filteredLogPool('ref');
 * @example <caption>Update or create a new FilterLogAnalyzer inside parent</caption>
 * cfg.filteredLogPool(parent, 'ref', function(filteredLogPool) { ... });
 */
Config.prototype.filteredLogPool = function(parent, reference, callback) {};
/**
 * Selector callback for FilterLogAnalyzer objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackFilterLogAnalyzer
 * @param {FilterLogAnalyzer} filteredLogPool the FilterLogAnalyzer instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed FilterLogAnalyzer instance.
 */

/**
 * Searches for all FilterLogAnalyzers with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {APMLogAnalyzer|FilterLogAnalyzer|WebTimingLogAnalyzer|FormLogAnalyzer} parent the APMLogAnalyzer|FilterLogAnalyzer|WebTimingLogAnalyzer|FormLogAnalyzer inside which the filteredLogPool must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackFilterLogAnalyzer} callback invoked for all instances.
 * @return {Array.<FilterLogAnalyzer>} all processed FilterLogAnalyzer objects in implementation defined order.
 * @example <caption>Create FilterLogAnalyzer instances for every element of the instances array</caption>
 * var filteredLogPools = cfg.filteredLogPools(parent, 'ref', ['node1', 'node2'], function(filteredLogPool, instance) { ... });
 */
Config.prototype.filteredLogPools = function(parent, reference, instances, callback) {};


/**
 * Selector callback for ViewLogAnalyzer objects. May be invoked multiple times!
 * @callback selectorCallbackViewLogAnalyzer
 * @param {ViewLogAnalyzer} viewLogPool matching your selection criteria.
 */

/**
 * Searches for all ViewLogAnalyzers with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {APMLogAnalyzer|FilterLogAnalyzer|ViewLogAnalyzer|WebTimingLogAnalyzer|FormLogAnalyzer=} parent the APMLogAnalyzer|FilterLogAnalyzer|ViewLogAnalyzer|WebTimingLogAnalyzer|FormLogAnalyzer inside which the viewLogPool must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackViewLogAnalyzer=} callback invoked for all matches of the search.
 * @returns {Array.<ViewLogAnalyzer>} matching ViewLogAnalyzer objects in implementation defined order.
 * @example <caption>Search for all viewLogPools with reference 'ref' inside parent</caption>
 * var array = cfg.viewLogPool(parent, 'ref');
 * @example <caption>Search for all viewLogPools with reference 'ref' globally</caption>
 * var array = cfg.viewLogPool('ref');
 * @example <caption>Update or create a new ViewLogAnalyzer inside parent</caption>
 * cfg.viewLogPool(parent, 'ref', function(viewLogPool) { ... });
 */
Config.prototype.viewLogPool = function(parent, reference, callback) {};
/**
 * Selector callback for ViewLogAnalyzer objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackViewLogAnalyzer
 * @param {ViewLogAnalyzer} viewLogPool the ViewLogAnalyzer instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed ViewLogAnalyzer instance.
 */

/**
 * Searches for all ViewLogAnalyzers with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {APMLogAnalyzer|FilterLogAnalyzer|ViewLogAnalyzer|WebTimingLogAnalyzer|FormLogAnalyzer} parent the APMLogAnalyzer|FilterLogAnalyzer|ViewLogAnalyzer|WebTimingLogAnalyzer|FormLogAnalyzer inside which the viewLogPool must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackViewLogAnalyzer} callback invoked for all instances.
 * @return {Array.<ViewLogAnalyzer>} all processed ViewLogAnalyzer objects in implementation defined order.
 * @example <caption>Create ViewLogAnalyzer instances for every element of the instances array</caption>
 * var viewLogPools = cfg.viewLogPools(parent, 'ref', ['node1', 'node2'], function(viewLogPool, instance) { ... });
 */
Config.prototype.viewLogPools = function(parent, reference, instances, callback) {};


/**
 * Selector callback for WebTimingLogAnalyzer objects. May be invoked multiple times!
 * @callback selectorCallbackWebTimingLogAnalyzer
 * @param {WebTimingLogAnalyzer} webTimingLogPool matching your selection criteria.
 */

/**
 * Searches for all WebTimingLogAnalyzers with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackWebTimingLogAnalyzer=} callback invoked for all matches of the search.
 * @returns {Array.<WebTimingLogAnalyzer>} matching WebTimingLogAnalyzer objects in implementation defined order.
 * @example <caption>Search for all webTimingLogPools with reference 'ref' globally</caption>
 * var array = cfg.webTimingLogPool('ref');
 * @example <caption>Update or create a new WebTimingLogAnalyzer</caption>
 * cfg.webTimingLogPool('ref', function(webTimingLogPool) { ... });
 */
Config.prototype.webTimingLogPool = function(reference, callback) {};
/**
 * Selector callback for WebTimingLogAnalyzer objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackWebTimingLogAnalyzer
 * @param {WebTimingLogAnalyzer} webTimingLogPool the WebTimingLogAnalyzer instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed WebTimingLogAnalyzer instance.
 */

/**
 * Searches for all WebTimingLogAnalyzers with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackWebTimingLogAnalyzer} callback invoked for all instances.
 * @return {Array.<WebTimingLogAnalyzer>} all processed WebTimingLogAnalyzer objects in implementation defined order.
 * @example <caption>Create WebTimingLogAnalyzer instances for every element of the instances array</caption>
 * var webTimingLogPools = cfg.webTimingLogPools('ref', ['node1', 'node2'], function(webTimingLogPool, instance) { ... });
 */
Config.prototype.webTimingLogPools = function(reference, instances, callback) {};


/**
 * Selector callback for FormLogAnalyzer objects. May be invoked multiple times!
 * @callback selectorCallbackFormLogAnalyzer
 * @param {FormLogAnalyzer} formLogPool matching your selection criteria.
 */

/**
 * Searches for all FormLogAnalyzers with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackFormLogAnalyzer=} callback invoked for all matches of the search.
 * @returns {Array.<FormLogAnalyzer>} matching FormLogAnalyzer objects in implementation defined order.
 * @example <caption>Search for all formLogPools with reference 'ref' globally</caption>
 * var array = cfg.formLogPool('ref');
 * @example <caption>Update or create a new FormLogAnalyzer</caption>
 * cfg.formLogPool('ref', function(formLogPool) { ... });
 */
Config.prototype.formLogPool = function(reference, callback) {};
/**
 * Selector callback for FormLogAnalyzer objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackFormLogAnalyzer
 * @param {FormLogAnalyzer} formLogPool the FormLogAnalyzer instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed FormLogAnalyzer instance.
 */

/**
 * Searches for all FormLogAnalyzers with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackFormLogAnalyzer} callback invoked for all instances.
 * @return {Array.<FormLogAnalyzer>} all processed FormLogAnalyzer objects in implementation defined order.
 * @example <caption>Create FormLogAnalyzer instances for every element of the instances array</caption>
 * var formLogPools = cfg.formLogPools('ref', ['node1', 'node2'], function(formLogPool, instance) { ... });
 */
Config.prototype.formLogPools = function(reference, instances, callback) {};


/**
 * Selector callback for LogStatistic objects. May be invoked multiple times!
 * @callback selectorCallbackLogStatistic
 * @param {LogStatistic} logPoolStatistic matching your selection criteria.
 */

/**
 * Searches for all LogStatistics with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackLogStatistic=} callback invoked for all matches of the search.
 * @returns {Array.<LogStatistic>} matching LogStatistic objects in implementation defined order.
 * @example <caption>Search for all logPoolStatistics with reference 'ref' globally</caption>
 * var array = cfg.logPoolStatistic('ref');
 * @example <caption>Update or create a new LogStatistic</caption>
 * cfg.logPoolStatistic('ref', function(logPoolStatistic) { ... });
 */
Config.prototype.logPoolStatistic = function(reference, callback) {};
/**
 * Selector callback for LogStatistic objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackLogStatistic
 * @param {LogStatistic} logPoolStatistic the LogStatistic instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed LogStatistic instance.
 */

/**
 * Searches for all LogStatistics with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackLogStatistic} callback invoked for all instances.
 * @return {Array.<LogStatistic>} all processed LogStatistic objects in implementation defined order.
 * @example <caption>Create LogStatistic instances for every element of the instances array</caption>
 * var logPoolStatistics = cfg.logPoolStatistics('ref', ['node1', 'node2'], function(logPoolStatistic, instance) { ... });
 */
Config.prototype.logPoolStatistics = function(reference, instances, callback) {};


/**
 * Selector callback for SLADefinition objects. May be invoked multiple times!
 * @callback selectorCallbackSLADefinition
 * @param {SLADefinition} slaDefinition matching your selection criteria.
 */

/**
 * Searches for all SLADefinitions with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackSLADefinition=} callback invoked for all matches of the search.
 * @returns {Array.<SLADefinition>} matching SLADefinition objects in implementation defined order.
 * @example <caption>Search for all slaDefinitions with reference 'ref' globally</caption>
 * var array = cfg.slaDefinition('ref');
 * @example <caption>Update or create a new SLADefinition</caption>
 * cfg.slaDefinition('ref', function(slaDefinition) { ... });
 */
Config.prototype.slaDefinition = function(reference, callback) {};
/**
 * Selector callback for SLADefinition objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackSLADefinition
 * @param {SLADefinition} slaDefinition the SLADefinition instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed SLADefinition instance.
 */

/**
 * Searches for all SLADefinitions with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackSLADefinition} callback invoked for all instances.
 * @return {Array.<SLADefinition>} all processed SLADefinition objects in implementation defined order.
 * @example <caption>Create SLADefinition instances for every element of the instances array</caption>
 * var slaDefinitions = cfg.slaDefinitions('ref', ['node1', 'node2'], function(slaDefinition, instance) { ... });
 */
Config.prototype.slaDefinitions = function(reference, instances, callback) {};


/**
 * Selector callback for Holiday objects. May be invoked multiple times!
 * @callback selectorCallbackHoliday
 * @param {Holiday} holiday matching your selection criteria.
 */

/**
 * Searches for all Holidays with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackHoliday=} callback invoked for all matches of the search.
 * @returns {Array.<Holiday>} matching Holiday objects in implementation defined order.
 * @example <caption>Search for all holidays with reference 'ref' globally</caption>
 * var array = cfg.holiday('ref');
 * @example <caption>Update or create a new Holiday</caption>
 * cfg.holiday('ref', function(holiday) { ... });
 */
Config.prototype.holiday = function(reference, callback) {};
/**
 * Selector callback for Holiday objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackHoliday
 * @param {Holiday} holiday the Holiday instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Holiday instance.
 */

/**
 * Searches for all Holidays with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackHoliday} callback invoked for all instances.
 * @return {Array.<Holiday>} all processed Holiday objects in implementation defined order.
 * @example <caption>Create Holiday instances for every element of the instances array</caption>
 * var holidays = cfg.holidays('ref', ['node1', 'node2'], function(holiday, instance) { ... });
 */
Config.prototype.holidays = function(reference, instances, callback) {};


/**
 * Selector callback for Dashboard objects. May be invoked multiple times!
 * @callback selectorCallbackDashboard
 * @param {Dashboard} dashboard matching your selection criteria.
 */

/**
 * Searches for all Dashboards with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackDashboard=} callback invoked for all matches of the search.
 * @returns {Array.<Dashboard>} matching Dashboard objects in implementation defined order.
 * @example <caption>Search for all dashboards with reference 'ref' globally</caption>
 * var array = cfg.dashboard('ref');
 * @example <caption>Update or create a new Dashboard</caption>
 * cfg.dashboard('ref', function(dashboard) { ... });
 */
Config.prototype.dashboard = function(reference, callback) {};
/**
 * Selector callback for Dashboard objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackDashboard
 * @param {Dashboard} dashboard the Dashboard instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Dashboard instance.
 */

/**
 * Searches for all Dashboards with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackDashboard} callback invoked for all instances.
 * @return {Array.<Dashboard>} all processed Dashboard objects in implementation defined order.
 * @example <caption>Create Dashboard instances for every element of the instances array</caption>
 * var dashboards = cfg.dashboards('ref', ['node1', 'node2'], function(dashboard, instance) { ... });
 */
Config.prototype.dashboards = function(reference, instances, callback) {};


/**
 * Selector callback for Chart objects. May be invoked multiple times!
 * @callback selectorCallbackChart
 * @param {Chart} chart matching your selection criteria.
 */

/**
 * Searches for all Charts with the given reference within the given parent (if any).
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * If the parent has been ommitted and the search yields no results and exception will be thrown.
 * @param {Dashboard=} parent the Dashboard inside which the chart must be.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackChart=} callback invoked for all matches of the search.
 * @returns {Array.<Chart>} matching Chart objects in implementation defined order.
 * @example <caption>Search for all charts with reference 'ref' inside parent</caption>
 * var array = cfg.chart(parent, 'ref');
 * @example <caption>Search for all charts with reference 'ref' globally</caption>
 * var array = cfg.chart('ref');
 * @example <caption>Update or create a new Chart inside parent</caption>
 * cfg.chart(parent, 'ref', function(chart) { ... });
 */
Config.prototype.chart = function(parent, reference, callback) {};
/**
 * Selector callback for Chart objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackChart
 * @param {Chart} chart the Chart instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Chart instance.
 */

/**
 * Searches for all Charts with the given reference within the given parent (if any).
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * If the parent has been ommitted but is required an exception will be thrown.
 * @param {Dashboard} parent the Dashboard inside which the chart must be.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackChart} callback invoked for all instances.
 * @return {Array.<Chart>} all processed Chart objects in implementation defined order.
 * @example <caption>Create Chart instances for every element of the instances array</caption>
 * var charts = cfg.charts(parent, 'ref', ['node1', 'node2'], function(chart, instance) { ... });
 */
Config.prototype.charts = function(parent, reference, instances, callback) {};


/**
 * Selector callback for Label objects. May be invoked multiple times!
 * @callback selectorCallbackLabel
 * @param {Label} label matching your selection criteria.
 */

/**
 * Searches for all Labels with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackLabel=} callback invoked for all matches of the search.
 * @returns {Array.<Label>} matching Label objects in implementation defined order.
 * @example <caption>Search for all labels with reference 'ref' globally</caption>
 * var array = cfg.label('ref');
 * @example <caption>Update or create a new Label</caption>
 * cfg.label('ref', function(label) { ... });
 */
Config.prototype.label = function(reference, callback) {};
/**
 * Selector callback for Label objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackLabel
 * @param {Label} label the Label instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Label instance.
 */

/**
 * Searches for all Labels with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackLabel} callback invoked for all instances.
 * @return {Array.<Label>} all processed Label objects in implementation defined order.
 * @example <caption>Create Label instances for every element of the instances array</caption>
 * var labels = cfg.labels('ref', ['node1', 'node2'], function(label, instance) { ... });
 */
Config.prototype.labels = function(reference, instances, callback) {};


/**
 * Selector callback for Module objects. May be invoked multiple times!
 * @callback selectorCallbackModule
 * @param {Module} module matching your selection criteria.
 */

/**
 * Searches for all Modules with the given reference.
 * The callback will be invoked for every found object. If there are no objects matching the reference
 * and the reference is also from the current module a new object will be created.
 * @param {String} reference identifies objects.
 * @param {selectorCallbackModule=} callback invoked for all matches of the search.
 * @returns {Array.<Module>} matching Module objects in implementation defined order.
 * @example <caption>Search for all modules with reference 'ref' globally</caption>
 * var array = cfg.module('ref');
 * @example <caption>Update or create a new Module</caption>
 * cfg.module('ref', function(module) { ... });
 */
Config.prototype.module = function(reference, callback) {};
/**
 * Selector callback for Module objects. Called for every unique instance passed to the selector function.
 * @callback selectorInstancedCallbackModule
 * @param {Module} module the Module instance that corresponds to the passed instance.
 * @param {String|CISElement} instance the instance name or object corresponding to the passed Module instance.
 */

/**
 * Searches for all Modules with the given reference.
 * The array of instances is made unique and then objects found that don't match any of the given instances
 * are deleted and missing objects are created. The callback will be invoked for every unique instance.
 * @param {String} reference the reference of the objects.
 * @param {Array.<String|CISElement>} instances the names of instances or instances of subclasses of CISElement.
 * @param {selectorInstancedCallbackModule} callback invoked for all instances.
 * @return {Array.<Module>} all processed Module objects in implementation defined order.
 * @example <caption>Create Module instances for every element of the instances array</caption>
 * var modules = cfg.modules('ref', ['node1', 'node2'], function(module, instance) { ... });
 */
Config.prototype.modules = function(reference, instances, callback) {};

/// END GENERATED DOCUMENTATION

