/*
 * Copyright (c) Fabasoft R&D GmbH, A-4020 Linz, 1995-2015.
 *
 * All rights reserved. All hardware and software names used are registered
 * trade names and/or registered trademarks of the respective manufacturers.
 *
 * The user of this computer program acknowledges that the above copyright
 * notice, which constitutes the Universal Copyright Convention, will be
 * attached at the position in the function of the computer program which the
 * author has deemed to sufficiently express the reservation of copyright.
 * It is prohibited for customers, users and/or third parties to remove,
 * modify or move this copyright notice.
 */
/*global Infrastructure, log, ElementStatus */
'use strict';

// Configure the stack trace to include all stack levels
Error.stackTraceLimit = Infinity;

//
// Internal API implementation
//

(function() {
  var objCache, moduleObj, objectModel, reValidModuleId, reValidReference;
  moduleObj = null;
  reValidModuleId = /^[A-Za-z0-9_\.+: \-]+$/;
  reValidReference = /^([A-Za-z0-9_\.+: \-]+@|)[A-Za-z0-9_\.+: \-]+$/;

  objectModel = {
    ServerInfo: {
      selector: 'serverSettings',
      singleton: true,
    },
    Server: {
      selector: 'server',
    },
    ConfigObject: {
      selector: 'configObject',
    },
    AgentGroup: {
      selector: 'agentGroup',
      validChildren: ['AgentGroup', 'Agent'],
      parent: {
        attribute: 'subgroups',
        onContext: true,
      },
      parentOptional: true,
    },
    Agent: {
      selector: 'agent',
      parent: {
        attribute: 'agents',
        onContext: true,
      },
    },
    ServiceGroup: {
      selector: 'serviceGroup',
      validChildren: ['ServiceGroup', 'Service'],
      parent: {
        attribute: 'subgroups',
        onContext: true,
      },
      parentOptional: true,
    },
    Service: {
      selector: 'service',
      validChildren: ['ServiceCheck'],
      parent: {
        attribute: 'services',
        onContext: true,
      },
    },
    ServiceCheck: {
      selector: 'serviceCheck',
      parent: {
        attribute: 'service',
        onContext: false,
      },
    },
    Counter: {
      selector: 'counter',
    },
    NotificationChannel: {
      selector: 'notificationChannel',
      validChildren: ['NotificationAccount'],
    },
    NotificationSchedule: {
      selector: 'notificationSchedule',
    },
    NotificationAccount: {
      selector: 'notificationAccount',
      parent: {
        attribute: 'channel',
        onContext: false,
      },
    },
    ServiceDesk: {
      selector: 'serviceDesk',
    },
    ServiceDeskFabasoftCloudTicketing: {
      selector: 'fabasoftServiceDesk',
    },
    DatabaseConnectionOLEDB: {
      selector: 'mssqlDatabaseConnection',
    },
    DatabaseConnectionPostgreSQL: {
      selector: 'pgsqlDatabaseConnection',
    },
    APMLogAnalyzer: {
      selector: 'logPool',
      validChildren: ['FilterLogAnalyzer', 'ViewLogAnalyzer'],
    },
    FilterLogAnalyzer: {
      selector: 'filteredLogPool',
      validChildren: ['FilterLogAnalyzer', 'ViewLogAnalyzer'],
      parent: {
        attribute: 'baseLogPoolId',
        onContext: false,
      },
    },
    ViewLogAnalyzer: {
      selector: 'viewLogPool',
      validChildren: ['ViewLogAnalyzer'],
      parent: {
        attribute: 'baseLogPoolId',
        onContext: false,
      },
    },
    WebTimingLogAnalyzer: {
      selector: 'webTimingLogPool',
      validChildren: ['FilterLogAnalyzer', 'ViewLogAnalyzer'],
    },
    FormLogAnalyzer: {
      selector: 'formLogPool',
      validChildren: ['FilterLogAnalyzer', 'ViewLogAnalyzer'],
    },
    LogStatistic: {
      selector: 'logPoolStatistic',
    },
    SLADefinition: {
      selector: 'slaDefinition',
    },
    Holiday: {
      selector: 'holiday',
    },
    Dashboard: {
      selector: 'dashboard',
      validChildren: ['Chart'],
    },
    Chart: {
      selector: 'chart',
      parent: {
        attribute: 'charts',
        onContext: true,
      },
    },
    Label: {
      selector: 'label',
    },
    Module: {
      selector: 'module',
    },
  };

  function throwUnless(success, message) {
    if (!success) {
      throw new Error(message);
    }
  }

  function clone(obj) {
    return JSON.parse(JSON.stringify(obj));
  }

  function extend(target) {
    for (var i = 1; i < arguments.length; ++i) {
      var obj = arguments[i];
      Object.keys(obj).forEach(function(key) {
        target[key] = obj[key];
      });
    }
    return target;
  }

  function jsonEquals(o1, o2) {
    var i, k1, k2, v1, v2;
    k1 = Object.keys(o1).sort();
    k2 = Object.keys(o2).sort();
    if (k1.length !== k2.length) {
      return false;
    }
    for (i = 0; i < k1.length; ++i) {
      if (k1[i] !== k2[i]) {
        return false;
      }
      v1 = o1[k1[i]];
      v2 = o2[k2[i]];
      if (typeof v1 !== typeof v2) {
        return false;
      }
      if ('object' === typeof v1 && null !== v1 && null !== v2) {
        if (!jsonEquals(v1, v2)) {
          return false;
        }
      }
      else if (v1 !== v2) {
        return false;
      }
    }
    return true;
  }

  objCache = {};
  objCache.cache = Object.create(null);
  objCache._protect = function(obj) {
    var descriptor = {
      configurable: false,
      enumerable: true,
      writable: false,
    };
    Object.defineProperties(obj, {
      "id": descriptor,
      "class": descriptor,
    });
  };
  objCache.put = function(cisclass, obj) {
    var entry = this.cache["" + obj.id] = {
      cisclass: cisclass,
      original: obj,
      ptr: clone(obj),
    };
    this._protect(entry.ptr);
    return entry;
  };
  objCache.create = function(cisclass) {
    var obj = Infrastructure.create(cisclass);
    throwUnless(obj, "Object creation failed, class: " + cisclass);
    return this.put(cisclass, obj).ptr;
  };
  objCache.get = function(cisclass, id) {
    var entry = this.cache["" + id], obj;
    if (undefined === entry) {
      obj = Infrastructure.getObject(id);
      throwUnless(obj, "Object not found, id: " + id);
      entry = this.put(obj['class'], obj);
    }
    throwUnless(cisclass === entry.ptr['class'], "Object class mismatch, object: " + entry.ptr['class'] + ", requested: " + cisclass);
    return entry.ptr;
  };
  objCache.search = function(cisclass, reference) {
    var cache = this.cache, matches = [];
    Object.keys(cache).forEach(function(id) {
      var entry = cache[id];
      if (entry.cisclass === cisclass && entry.ptr.reference === reference) {
        matches.push(entry.ptr);
      }
    });
    return matches;
  };
  objCache.getDirtyObjects = function() {
    var cache = this.cache, objects = [];
    Object.keys(cache).forEach(function(id) {
      if (!jsonEquals(cache[id].ptr, cache[id].original)) {
        objects.push(cache[id].ptr);
      }
    });
    return objects;
  };
  objCache.clear = function() {
    var cache = this.cache;
    Object.keys(cache).forEach(function(id) {
      delete cache[id];
    });
  };

  function reloadModule(id, moduleId) {
    var writeLog = (!moduleObj || moduleObj.moduleId !== moduleId);
    moduleObj = {
      id: id,
      obj: objCache.get('Module', id),
      moduleId: moduleId,
    };
    if (writeLog) {
      log("Switching to Module '" + moduleId + "' current version:", moduleObj.obj.properties.version || 0);
    }
  }

  function makeReferenceAbsolute(reference) {
    var ref = reference;
    if (-1 === reference.indexOf('@')) {
      ref = moduleObj.moduleId + '@' + reference;
    }
    return ref;
  }

  function parseReference(reference) {
    var referenceParts = reference.split('@');
    return {
      moduleId: referenceParts[0],
      reference: referenceParts[1],
    };
  }

  function isReferenceFromCurrentModule(reference) {
    var referenceParts = parseReference(reference);
    return referenceParts.moduleId === moduleObj.moduleId;
  }

  function getClassObjectsByReference(cisclass, absoluteReference) {
    var ids = Infrastructure.getObjectsByReference(cisclass, absoluteReference);
    objCache.search(cisclass, parseReference(absoluteReference).reference).forEach(function(obj) {
      ids.push(obj.id);
    });
    var seen = {};
    return ids.filter(function(id) {
      var ok = false;
      if (!seen[id]) {
        var obj = objCache.get(cisclass, id);
        seen[id] = true;
        ok = parseInt(obj.status, 10) !== ElementStatus.Deleted;
      }
      return ok;
    });
  }

  function createObjectForModule(cisclass, reference) {
    var newObj = objCache.create(cisclass);
    newObj.reference = reference;
    moduleObj.obj.items.push(newObj.id);
    return newObj;
  }

  function dispatchObjectCallbacks(cisclass, objects, callback) {
    var result = [];
    objects.forEach(function(idOrObject) {
      var id = idOrObject, instance = null;
      if ('object' === typeof idOrObject) {
        id = idOrObject.id;
        instance = idOrObject.instanceObj;
      }
      var obj = objCache.get(cisclass, id);
      if (callback) {
        if (null !== instance) {
          callback(obj, instance);
        }
        else {
          callback(obj);
        }
      }
      result.push(obj);
    });
    return result;
  }

  function parentFilter(cisclass, context) {
    var parentConfig = objectModel[cisclass].parent;
    throwUnless(parentConfig, "Can't create " + cisclass + " below " + context['class']);
    return function(id) {
      var obj = objCache.get(cisclass, id);
      throwUnless(obj, "Object " + id + " not found");
      var m = [context, obj];
      if (!parentConfig.onContext) {
        m = [obj, context];
      }
      var attr = m[0][parentConfig.attribute];
      return Array.isArray(attr) ? (attr.indexOf(m[1].id) !== -1) : (parseInt(attr, 10) === parseInt(m[1].id, 10));
    };
  }

  function setupParent(cisclass, context, obj) {
    var parentConfig = objectModel[cisclass].parent;
    throwUnless(parentConfig, cisclass + " can't be a child of " + context['class']);
    var m = [context, obj];
    if (!parentConfig.onContext) {
      m = [obj, context];
    }
    if (Array.isArray(m[0][parentConfig.attribute])) {
      m[0][parentConfig.attribute].push(m[1].id);
    }
    else {
      m[0][parentConfig.attribute] = m[1].id;
    }
  }

  function processInstances(cisclass, absoluteReference, objectIds, instances, context) {
    var uniqueInstanceMap = instances.reduce(function(m, v) {
      if ('object' === typeof v) {
        m['' + v.id] = v;
      }
      else {
        m['' + v] = '' + v;
      }
      return m;
    }, Object.create(null));
    var instanceNames = Object.keys(uniqueInstanceMap);
    var finalObjects = [];
    objectIds.forEach(function(id) {
      var obj = objCache.get(cisclass, id);
      var idx = instanceNames.indexOf(obj.properties['APM-INSTANCE']);
      if (-1 === idx) {
        obj.status = ElementStatus.Deleted;
      }
      else {
        finalObjects.push({
          id: obj.id,
          instanceObj: uniqueInstanceMap[instanceNames[idx]],
        });
        instanceNames.splice(idx, 1);
      }
    });
    var ref = parseReference(absoluteReference).reference;
    instanceNames.forEach(function(name) {
      var obj = createObjectForModule(cisclass, ref);
      if (context) {
        setupParent(cisclass, context, obj);
      }
      obj.properties['APM-INSTANCE'] = name;
      finalObjects.push({
        id: obj.id,
        instanceObj: uniqueInstanceMap[name],
      });
    });
    return finalObjects;
  }

  function findObjects(cisclass, context, reference, instances, callback) {
    var om = objectModel[cisclass], contextValid = false, sub;
    throwUnless(reValidReference.test(reference), "The reference '" + reference + "' contains invalid characters. Has to match: " + reValidReference);
    if (om.parent) {
      if (null !== context && 'object' === typeof context && context.id > 0) {
        sub = objectModel[context['class']].validChildren;
        throwUnless(sub.indexOf(cisclass) !== -1, "Can't create " + cisclass + " below a " + context['class']);
        contextValid = true;
      }
    }
    var ref = makeReferenceAbsolute(reference);
    var objectIds = getClassObjectsByReference(cisclass, ref);
    if (contextValid) {
      objectIds = objectIds.filter(parentFilter(cisclass, context));
    }
    var callbacksToTrigger = objectIds;
    if (instances) {
      throwUnless(isReferenceFromCurrentModule(ref), "Can't modify instances of a foreign module.");
      if (om.parent && !contextValid) {
        throwUnless(om.parentOptional, cisclass + " requires a parent, none given");
      }
      callbacksToTrigger = processInstances(cisclass, ref, objectIds, instances, (contextValid ? context : null));
    }
    else if (0 === objectIds.length && isReferenceFromCurrentModule(ref)) {
      if (om.parent && !contextValid) {
        throwUnless(om.parentOptional, cisclass + " requires a parent, none given");
      }
      var newObj = createObjectForModule(cisclass, parseReference(ref).reference);
      callbacksToTrigger.push(newObj.id);
      if (contextValid) {
        setupParent(cisclass, context, newObj);
      }
    }
    throwUnless(callbacksToTrigger.length > 0, "Unable to find or create objects with reference '" + ref + "'");
    return dispatchObjectCallbacks(cisclass, callbacksToTrigger, callback);
  }

  function mapCounterInfoInstanceToValue(counterInfo) {
    return counterInfo.instances.map(function(instance) {
      return instance.value;
    });
  }

  function enumInstances(agentIdOrAgentObj, protocol, query) {
    var agentId = 'object' === typeof(agentIdOrAgentObj) ? agentIdOrAgentObj.id : agentIdOrAgentObj;
    throwUnless(agentId > 0, "Invalid agent id.");
    throwUnless(protocol, "Protocol is required.");
    return mapCounterInfoInstanceToValue(Infrastructure.getCounterInfo(agentId, protocol, query)).filter(function(instance) {
      return instance !== '_Total';
    });
  }

  function resolveSearchFunction(cisclass, context, reference, callback) {
    if ('string' === typeof context) {
      return findObjects(cisclass, null, arguments[1], null, arguments[2]);
    }
    return findObjects(cisclass, arguments[1], arguments[2], null, arguments[3]);
  }

  function resolveSearchFunctionWithInstancing(cisclass, context, reference, instances, callback) {
    throwUnless(Array.isArray(instances), "Array required for argument 'instances', " + (null === instances ? null : typeof instances) + " given.");
    if ('string' === typeof context) {
      return findObjects(cisclass, null, arguments[1], arguments[2], arguments[3]);
    }
    return findObjects(cisclass, arguments[1], arguments[2], arguments[3], arguments[4]);
  }

  function Config() {}
  Config.prototype.serverSettings = function() {
    return objCache.get('ServerInfo', 0);
  };
  Config.prototype.enumInstances = enumInstances;

  Object.keys(objectModel).forEach(function(cls) {
    var om = objectModel[cls];
    if (!om.singleton) {
      Config.prototype[om.selector] = function(context, reference, callback) {
        return resolveSearchFunction(cls, context, reference, callback);
      };
      Config.prototype[om.selector + 's'] = function(context, reference, instances, callback) {
        return resolveSearchFunctionWithInstancing(cls, context, reference, instances, callback);
      };
    }
  });

  Internal = {
    config_module: function config_module(moduleId) {
      throwUnless(reValidModuleId.test(moduleId), "The moduleId contains invalid characters. Has to match: " + reValidModuleId);
      reloadModule(Infrastructure.getModule(moduleId), moduleId);
    },
    upgrade: function upgrade(version, fn) {
      throwUnless(moduleObj, "API not yet initialized, initialize with config_module() first");
      throwUnless('number' === typeof version, "'version' must be a number. Received: '" + (typeof version) + "'");
      throwUnless(version > 0, "'version' must be greater than 0. Received: '" + version + "'");
      throwUnless('function' === typeof fn, "'transactionFunction' must be a function. Received: '" + (typeof fn) + "'");
      if (parseInt(moduleObj.obj.properties.version, 10) < version) {
        log("Starting upgrade to version " + version + "...");
        fn(new Config());
        var changedObjects = objCache.getDirtyObjects();
        moduleObj.obj.properties.version = version;
        changedObjects.push(moduleObj.obj);
        Infrastructure.commit(changedObjects);
        log("Upgrade to version " + version + " succeeded");
        objCache.clear();
        reloadModule(moduleObj.id, moduleObj.moduleId);
      }
    },
    extend: extend,
    API: function API(major, minor) {
      var success = Infrastructure.checkVersion(major, minor);
      throwUnless(success, "Version requirement failed. Requested: " + major + "." + minor);
    },
  };
}());
