#!/bin/bash -e
# ===========================================================================
# Copyright (c) Fabasoft R&D GmbH, A-4020 Linz, 1995-2015.
#
# All rights reserved. All hardware and software names used are registered
# trade names and/or registered trademarks of the respective manufacturers.
#
# The user of this computer program acknowledges that the above copyright
# notice, which constitutes the Universal Copyright Convention, will be
# attached at the position in the function of the computer program which the
# author has deemed to sufficiently express the reservation of copyright.
# It is prohibited for customers, users and/or third parties to remove,
# modify or move this copyright notice.
# ==========================================================================

# check if SELinux is available
if [ -f /selinux/enforce -o -f /sys/fs/selinux/enforce ] ; then
	if [ -x /usr/sbin/semanage ] && [ -x /usr/sbin/semodule ] && [ -x /sbin/restorecon ] ; then
		echo "SELinux available and all required base tools (for loading prebuilt module) found."
	else 
		echo "Warning: SELinux available but not all required tools (semanage, semodule, restorecon) found - exiting!"
		exit 1	
	fi
else 
	echo "SELinux not available - exiting."
	exit 0
fi

REALDIR=`readlink -f $0`
DIRNAME=`dirname $REALDIR`
cd $DIRNAME

TARGET=$1
PORT=$2
ACTION=$3

SCRIPT="${DIRNAME}/${TARGET}.sh"

if [ $# -lt 3 ]; then
	SCRIPT="$0 component port"
fi

USAGE="USAGE: ${SCRIPT} [ --update | --rebuild | --uninstall | --skip-build ]"

if [ `id -u` != 0 ]; then
	echo 'You must be root to run this script'
	exit 1
fi

if [ $# -ne 3 ]; then
	echo -e $USAGE
	exit 1
fi

case "$ACTION" in
	--update)
		if ! [ -r /usr/share/selinux/devel/Makefile ] ; then
			echo "Rebuilding SELinux Policy requires additional SELinux tools (/usr/share/selinux/devel/Makefile) - on EL7 systems install RPM package 'selinux-policy-devel'!"
			exit 1
		fi
		time=`ls -l --time-style="+%x %X" $TARGET.te | awk '{ printf "%s %s", $6, $7 }'`
		rules=`ausearch --start $time -m avc --raw -se $TARGET`
		if [ x"$rules" != "x" ] ; then
			echo "Found AVC denies to update policy with"
			echo -e "$rules" | audit2allow -R
			echo "Do you want these changes added to policy [y/n]?"
			read ANS
			if [ "$ANS" = "y" -o "$ANS" = "Y" ] ; then
				echo "Updating policy"
				echo -e "$rules" | audit2allow -R >> $TARGET.te
				# Fall though and rebuild policy
			else
				exit 0
			fi
		else
			echo "No new AVC denies found"
			exit 0
		fi
		echo "Building SELinux Policy"
		set -x
		make -f /usr/share/selinux/devel/Makefile
		;;

	--rebuild)
		if ! [ -r /usr/share/selinux/devel/Makefile ] ; then
			echo "Rebuilding SELinux Policy requires additional SELinux tools (/usr/share/selinux/devel/Makefile) - on EL7 systems install RPM package 'selinux-policy-devel'!"
			exit 1
		fi
		echo "Rebuilding SELinux Policy on local system for $TARGET"
		set -x
		touch ./$TARGET.*
		make -f /usr/share/selinux/devel/Makefile
		;;

	--skip-build)
		echo "skipping build of SELinux policy module"
		;;
	
	--uninstall)
		echo "uninstalling SELinux policy module for $TARGET"
		if [ $PORT -gt 0 ]; then
			PORT_NUMBER=`semanage port -l | grep -e "^${TARGET}_port_t\s" | sed "s/[^0-9]*//"`
			/usr/sbin/semanage port -d -t "${TARGET}_port_t" -p tcp $PORT_NUMBER
		fi
		/usr/sbin/semodule -r $TARGET
		exit $?
		;;
	
	*)
		echo -e $USAGE
		exit 1
	;;
esac

echo "installing SELinux policy module for $TARGET"
/usr/sbin/semodule -i $TARGET.pp
exstat=$?

# Fixing the file contexts on directories for this target
/sbin/restorecon -F -R -v /opt/app.telemetry/
/sbin/restorecon -F -R -v /etc/rc.d/init.d/
/sbin/restorecon -F -R -v /etc/app.telemetry/
/sbin/restorecon -F -R -v /var/log/app.telemetry
/sbin/restorecon -F -R -v /var/opt/app.telemetry

function checkport {
	if ! /usr/sbin/semanage port -l | grep "^$1\s" 2>&1 >/dev/null; then
		/usr/sbin/semanage port -a -t "$1" -p tcp $2
	fi
}

# Adding SELinux tcp port
if [ $PORT -gt 0 ]; then
	checkport "${TARGET}_port_t" $PORT
fi
if [ "apptelemetryagent" = "${TARGET}" ] ; then
	checkport "apptelemetryagent_telemetry_port_t" 10002
fi

if [ "apptelemetryagent" = $TARGET ]; then
	if test -n "$(find /var/run/ -maxdepth 1 -name 'apptelemetry*' -print -quit)" ; then
		/sbin/restorecon -F -R -v "/var/run/apptelemetry*"
	fi
fi

exit $exstat
